#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

GRINNING_FACE = "\U0001f600"
GRINNING_FACE_WITH_BIG_EYES = "\U0001f603"
GRINNING_FACE_WITH_SMILING_EYES = "\U0001f604"
BEAMING_FACE_WITH_SMILING_EYES = "\U0001f601"
GRINNING_SQUINTING_FACE = "\U0001f606"
GRINNING_FACE_WITH_SWEAT = "\U0001f605"
ROLLING_ON_THE_FLOOR_LAUGHING = "\U0001f923"
FACE_WITH_TEARS_OF_JOY = "\U0001f602"
SLIGHTLY_SMILING_FACE = "\U0001f642"
UPSIDE_DOWN_FACE = "\U0001f643"
MELTING_FACE = "\U0001fae0"
WINKING_FACE = "\U0001f609"
SMILING_FACE_WITH_SMILING_EYES = "\U0001f60a"
SMILING_FACE_WITH_HALO = "\U0001f607"
SMILING_FACE_WITH_HEARTS = "\U0001f970"
SMILING_FACE_WITH_HEART_EYES = "\U0001f60d"
STAR_STRUCK = "\U0001f929"
FACE_BLOWING_A_KISS = "\U0001f618"
KISSING_FACE = "\U0001f617"
SMILING_FACE = "\u263a\ufe0f"
KISSING_FACE_WITH_CLOSED_EYES = "\U0001f61a"
KISSING_FACE_WITH_SMILING_EYES = "\U0001f619"
SMILING_FACE_WITH_TEAR = "\U0001f972"
FACE_SAVORING_FOOD = "\U0001f60b"
FACE_WITH_TONGUE = "\U0001f61b"
WINKING_FACE_WITH_TONGUE = "\U0001f61c"
ZANY_FACE = "\U0001f92a"
SQUINTING_FACE_WITH_TONGUE = "\U0001f61d"
MONEY_MOUTH_FACE = "\U0001f911"
SMILING_FACE_WITH_OPEN_HANDS = "\U0001f917"
FACE_WITH_HAND_OVER_MOUTH = "\U0001f92d"
FACE_WITH_OPEN_EYES_AND_HAND_OVER_MOUTH = "\U0001fae2"
FACE_WITH_PEEKING_EYE = "\U0001fae3"
SHUSHING_FACE = "\U0001f92b"
THINKING_FACE = "\U0001f914"
SALUTING_FACE = "\U0001fae1"
ZIPPER_MOUTH_FACE = "\U0001f910"
FACE_WITH_RAISED_EYEBROW = "\U0001f928"
NEUTRAL_FACE = "\U0001f610"
EXPRESSIONLESS_FACE = "\U0001f611"
FACE_WITHOUT_MOUTH = "\U0001f636"
DOTTED_LINE_FACE = "\U0001fae5"
FACE_IN_CLOUDS = "\U0001f636\u200d\U0001f32b\ufe0f"
SMIRKING_FACE = "\U0001f60f"
UNAMUSED_FACE = "\U0001f612"
FACE_WITH_ROLLING_EYES = "\U0001f644"
GRIMACING_FACE = "\U0001f62c"
FACE_EXHALING = "\U0001f62e\u200d\U0001f4a8"
LYING_FACE = "\U0001f925"
RELIEVED_FACE = "\U0001f60c"
PENSIVE_FACE = "\U0001f614"
SLEEPY_FACE = "\U0001f62a"
DROOLING_FACE = "\U0001f924"
SLEEPING_FACE = "\U0001f634"
FACE_WITH_MEDICAL_MASK = "\U0001f637"
FACE_WITH_THERMOMETER = "\U0001f912"
FACE_WITH_HEAD_BANDAGE = "\U0001f915"
NAUSEATED_FACE = "\U0001f922"
FACE_VOMITING = "\U0001f92e"
SNEEZING_FACE = "\U0001f927"
HOT_FACE = "\U0001f975"
COLD_FACE = "\U0001f976"
WOOZY_FACE = "\U0001f974"
FACE_WITH_CROSSED_OUT_EYES = "\U0001f635"
FACE_WITH_SPIRAL_EYES = "\U0001f635\u200d\U0001f4ab"
EXPLODING_HEAD = "\U0001f92f"
COWBOY_HAT_FACE = "\U0001f920"
PARTYING_FACE = "\U0001f973"
DISGUISED_FACE = "\U0001f978"
SMILING_FACE_WITH_SUNGLASSES = "\U0001f60e"
NERD_FACE = "\U0001f913"
FACE_WITH_MONOCLE = "\U0001f9d0"
CONFUSED_FACE = "\U0001f615"
FACE_WITH_DIAGONAL_MOUTH = "\U0001fae4"
WORRIED_FACE = "\U0001f61f"
SLIGHTLY_FROWNING_FACE = "\U0001f641"
FROWNING_FACE = "\u2639\ufe0f"
FACE_WITH_OPEN_MOUTH = "\U0001f62e"
HUSHED_FACE = "\U0001f62f"
ASTONISHED_FACE = "\U0001f632"
FLUSHED_FACE = "\U0001f633"
PLEADING_FACE = "\U0001f97a"
FACE_HOLDING_BACK_TEARS = "\U0001f979"
FROWNING_FACE_WITH_OPEN_MOUTH = "\U0001f626"
ANGUISHED_FACE = "\U0001f627"
FEARFUL_FACE = "\U0001f628"
ANXIOUS_FACE_WITH_SWEAT = "\U0001f630"
SAD_BUT_RELIEVED_FACE = "\U0001f625"
CRYING_FACE = "\U0001f622"
LOUDLY_CRYING_FACE = "\U0001f62d"
FACE_SCREAMING_IN_FEAR = "\U0001f631"
CONFOUNDED_FACE = "\U0001f616"
PERSEVERING_FACE = "\U0001f623"
DISAPPOINTED_FACE = "\U0001f61e"
DOWNCAST_FACE_WITH_SWEAT = "\U0001f613"
WEARY_FACE = "\U0001f629"
TIRED_FACE = "\U0001f62b"
YAWNING_FACE = "\U0001f971"
FACE_WITH_STEAM_FROM_NOSE = "\U0001f624"
ENRAGED_FACE = "\U0001f621"
ANGRY_FACE = "\U0001f620"
FACE_WITH_SYMBOLS_ON_MOUTH = "\U0001f92c"
SMILING_FACE_WITH_HORNS = "\U0001f608"
ANGRY_FACE_WITH_HORNS = "\U0001f47f"
SKULL = "\U0001f480"
SKULL_AND_CROSSBONES = "\u2620\ufe0f"
PILE_OF_POO = "\U0001f4a9"
CLOWN_FACE = "\U0001f921"
OGRE = "\U0001f479"
GOBLIN = "\U0001f47a"
GHOST = "\U0001f47b"
ALIEN = "\U0001f47d"
ALIEN_MONSTER = "\U0001f47e"
ROBOT = "\U0001f916"
GRINNING_CAT = "\U0001f63a"
GRINNING_CAT_WITH_SMILING_EYES = "\U0001f638"
CAT_WITH_TEARS_OF_JOY = "\U0001f639"
SMILING_CAT_WITH_HEART_EYES = "\U0001f63b"
CAT_WITH_WRY_SMILE = "\U0001f63c"
KISSING_CAT = "\U0001f63d"
WEARY_CAT = "\U0001f640"
CRYING_CAT = "\U0001f63f"
POUTING_CAT = "\U0001f63e"
SEE_NO_EVIL_MONKEY = "\U0001f648"
HEAR_NO_EVIL_MONKEY = "\U0001f649"
SPEAK_NO_EVIL_MONKEY = "\U0001f64a"
KISS_MARK = "\U0001f48b"
LOVE_LETTER = "\U0001f48c"
HEART_WITH_ARROW = "\U0001f498"
HEART_WITH_RIBBON = "\U0001f49d"
SPARKLING_HEART = "\U0001f496"
GROWING_HEART = "\U0001f497"
BEATING_HEART = "\U0001f493"
REVOLVING_HEARTS = "\U0001f49e"
TWO_HEARTS = "\U0001f495"
HEART_DECORATION = "\U0001f49f"
HEART_EXCLAMATION = "\u2763\ufe0f"
BROKEN_HEART = "\U0001f494"
HEART_ON_FIRE = "\u2764\ufe0f\u200d\U0001f525"
MENDING_HEART = "\u2764\ufe0f\u200d\U0001fa79"
RED_HEART = "\u2764\ufe0f"
ORANGE_HEART = "\U0001f9e1"
YELLOW_HEART = "\U0001f49b"
GREEN_HEART = "\U0001f49a"
BLUE_HEART = "\U0001f499"
PURPLE_HEART = "\U0001f49c"
BROWN_HEART = "\U0001f90e"
BLACK_HEART = "\U0001f5a4"
WHITE_HEART = "\U0001f90d"
HUNDRED_POINTS = "\U0001f4af"
ANGER_SYMBOL = "\U0001f4a2"
COLLISION = "\U0001f4a5"
DIZZY = "\U0001f4ab"
SWEAT_DROPLETS = "\U0001f4a6"
DASHING_AWAY = "\U0001f4a8"
HOLE = "\U0001f573\ufe0f"
BOMB = "\U0001f4a3"
SPEECH_BALLOON = "\U0001f4ac"
EYE_IN_SPEECH_BUBBLE = "\U0001f441\ufe0f\u200d\U0001f5e8\ufe0f"
LEFT_SPEECH_BUBBLE = "\U0001f5e8\ufe0f"
RIGHT_ANGER_BUBBLE = "\U0001f5ef\ufe0f"
THOUGHT_BALLOON = "\U0001f4ad"
ZZZ = "\U0001f4a4"
WAVING_HAND = "\U0001f44b"
WAVING_HAND_LIGHT_SKIN_TONE = "\U0001f44b\U0001f3fb"
WAVING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f44b\U0001f3fc"
WAVING_HAND_MEDIUM_SKIN_TONE = "\U0001f44b\U0001f3fd"
WAVING_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f44b\U0001f3fe"
WAVING_HAND_DARK_SKIN_TONE = "\U0001f44b\U0001f3ff"
RAISED_BACK_OF_HAND = "\U0001f91a"
RAISED_BACK_OF_HAND_LIGHT_SKIN_TONE = "\U0001f91a\U0001f3fb"
RAISED_BACK_OF_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f91a\U0001f3fc"
RAISED_BACK_OF_HAND_MEDIUM_SKIN_TONE = "\U0001f91a\U0001f3fd"
RAISED_BACK_OF_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f91a\U0001f3fe"
RAISED_BACK_OF_HAND_DARK_SKIN_TONE = "\U0001f91a\U0001f3ff"
HAND_WITH_FINGERS_SPLAYED = "\U0001f590\ufe0f"
HAND_WITH_FINGERS_SPLAYED_LIGHT_SKIN_TONE = "\U0001f590\U0001f3fb"
HAND_WITH_FINGERS_SPLAYED_MEDIUM_LIGHT_SKIN_TONE = "\U0001f590\U0001f3fc"
HAND_WITH_FINGERS_SPLAYED_MEDIUM_SKIN_TONE = "\U0001f590\U0001f3fd"
HAND_WITH_FINGERS_SPLAYED_MEDIUM_DARK_SKIN_TONE = "\U0001f590\U0001f3fe"
HAND_WITH_FINGERS_SPLAYED_DARK_SKIN_TONE = "\U0001f590\U0001f3ff"
RAISED_HAND = "\u270b"
RAISED_HAND_LIGHT_SKIN_TONE = "\u270b\U0001f3fb"
RAISED_HAND_MEDIUM_LIGHT_SKIN_TONE = "\u270b\U0001f3fc"
RAISED_HAND_MEDIUM_SKIN_TONE = "\u270b\U0001f3fd"
RAISED_HAND_MEDIUM_DARK_SKIN_TONE = "\u270b\U0001f3fe"
RAISED_HAND_DARK_SKIN_TONE = "\u270b\U0001f3ff"
VULCAN_SALUTE = "\U0001f596"
VULCAN_SALUTE_LIGHT_SKIN_TONE = "\U0001f596\U0001f3fb"
VULCAN_SALUTE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f596\U0001f3fc"
VULCAN_SALUTE_MEDIUM_SKIN_TONE = "\U0001f596\U0001f3fd"
VULCAN_SALUTE_MEDIUM_DARK_SKIN_TONE = "\U0001f596\U0001f3fe"
VULCAN_SALUTE_DARK_SKIN_TONE = "\U0001f596\U0001f3ff"
RIGHTWARDS_HAND = "\U0001faf1"
RIGHTWARDS_HAND_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3fb"
RIGHTWARDS_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3fc"
RIGHTWARDS_HAND_MEDIUM_SKIN_TONE = "\U0001faf1\U0001f3fd"
RIGHTWARDS_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001faf1\U0001f3fe"
RIGHTWARDS_HAND_DARK_SKIN_TONE = "\U0001faf1\U0001f3ff"
LEFTWARDS_HAND = "\U0001faf2"
LEFTWARDS_HAND_LIGHT_SKIN_TONE = "\U0001faf2\U0001f3fb"
LEFTWARDS_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf2\U0001f3fc"
LEFTWARDS_HAND_MEDIUM_SKIN_TONE = "\U0001faf2\U0001f3fd"
LEFTWARDS_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001faf2\U0001f3fe"
LEFTWARDS_HAND_DARK_SKIN_TONE = "\U0001faf2\U0001f3ff"
PALM_DOWN_HAND = "\U0001faf3"
PALM_DOWN_HAND_LIGHT_SKIN_TONE = "\U0001faf3\U0001f3fb"
PALM_DOWN_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf3\U0001f3fc"
PALM_DOWN_HAND_MEDIUM_SKIN_TONE = "\U0001faf3\U0001f3fd"
PALM_DOWN_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001faf3\U0001f3fe"
PALM_DOWN_HAND_DARK_SKIN_TONE = "\U0001faf3\U0001f3ff"
PALM_UP_HAND = "\U0001faf4"
PALM_UP_HAND_LIGHT_SKIN_TONE = "\U0001faf4\U0001f3fb"
PALM_UP_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf4\U0001f3fc"
PALM_UP_HAND_MEDIUM_SKIN_TONE = "\U0001faf4\U0001f3fd"
PALM_UP_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001faf4\U0001f3fe"
PALM_UP_HAND_DARK_SKIN_TONE = "\U0001faf4\U0001f3ff"
OK_HAND = "\U0001f44c"
OK_HAND_LIGHT_SKIN_TONE = "\U0001f44c\U0001f3fb"
OK_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f44c\U0001f3fc"
OK_HAND_MEDIUM_SKIN_TONE = "\U0001f44c\U0001f3fd"
OK_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f44c\U0001f3fe"
OK_HAND_DARK_SKIN_TONE = "\U0001f44c\U0001f3ff"
PINCHED_FINGERS = "\U0001f90c"
PINCHED_FINGERS_LIGHT_SKIN_TONE = "\U0001f90c\U0001f3fb"
PINCHED_FINGERS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f90c\U0001f3fc"
PINCHED_FINGERS_MEDIUM_SKIN_TONE = "\U0001f90c\U0001f3fd"
PINCHED_FINGERS_MEDIUM_DARK_SKIN_TONE = "\U0001f90c\U0001f3fe"
PINCHED_FINGERS_DARK_SKIN_TONE = "\U0001f90c\U0001f3ff"
PINCHING_HAND = "\U0001f90f"
PINCHING_HAND_LIGHT_SKIN_TONE = "\U0001f90f\U0001f3fb"
PINCHING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f90f\U0001f3fc"
PINCHING_HAND_MEDIUM_SKIN_TONE = "\U0001f90f\U0001f3fd"
PINCHING_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f90f\U0001f3fe"
PINCHING_HAND_DARK_SKIN_TONE = "\U0001f90f\U0001f3ff"
VICTORY_HAND = "\u270c\ufe0f"
VICTORY_HAND_LIGHT_SKIN_TONE = "\u270c\U0001f3fb"
VICTORY_HAND_MEDIUM_LIGHT_SKIN_TONE = "\u270c\U0001f3fc"
VICTORY_HAND_MEDIUM_SKIN_TONE = "\u270c\U0001f3fd"
VICTORY_HAND_MEDIUM_DARK_SKIN_TONE = "\u270c\U0001f3fe"
VICTORY_HAND_DARK_SKIN_TONE = "\u270c\U0001f3ff"
CROSSED_FINGERS = "\U0001f91e"
CROSSED_FINGERS_LIGHT_SKIN_TONE = "\U0001f91e\U0001f3fb"
CROSSED_FINGERS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f91e\U0001f3fc"
CROSSED_FINGERS_MEDIUM_SKIN_TONE = "\U0001f91e\U0001f3fd"
CROSSED_FINGERS_MEDIUM_DARK_SKIN_TONE = "\U0001f91e\U0001f3fe"
CROSSED_FINGERS_DARK_SKIN_TONE = "\U0001f91e\U0001f3ff"
HAND_WITH_INDEX_FINGER_AND_THUMB_CROSSED = "\U0001faf0"
HAND_WITH_INDEX_FINGER_AND_THUMB_CROSSED_LIGHT_SKIN_TONE = "\U0001faf0\U0001f3fb"
HAND_WITH_INDEX_FINGER_AND_THUMB_CROSSED_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf0\U0001f3fc"
HAND_WITH_INDEX_FINGER_AND_THUMB_CROSSED_MEDIUM_SKIN_TONE = "\U0001faf0\U0001f3fd"
HAND_WITH_INDEX_FINGER_AND_THUMB_CROSSED_MEDIUM_DARK_SKIN_TONE = "\U0001faf0\U0001f3fe"
HAND_WITH_INDEX_FINGER_AND_THUMB_CROSSED_DARK_SKIN_TONE = "\U0001faf0\U0001f3ff"
LOVE_YOU_GESTURE = "\U0001f91f"
LOVE_YOU_GESTURE_LIGHT_SKIN_TONE = "\U0001f91f\U0001f3fb"
LOVE_YOU_GESTURE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f91f\U0001f3fc"
LOVE_YOU_GESTURE_MEDIUM_SKIN_TONE = "\U0001f91f\U0001f3fd"
LOVE_YOU_GESTURE_MEDIUM_DARK_SKIN_TONE = "\U0001f91f\U0001f3fe"
LOVE_YOU_GESTURE_DARK_SKIN_TONE = "\U0001f91f\U0001f3ff"
SIGN_OF_THE_HORNS = "\U0001f918"
SIGN_OF_THE_HORNS_LIGHT_SKIN_TONE = "\U0001f918\U0001f3fb"
SIGN_OF_THE_HORNS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f918\U0001f3fc"
SIGN_OF_THE_HORNS_MEDIUM_SKIN_TONE = "\U0001f918\U0001f3fd"
SIGN_OF_THE_HORNS_MEDIUM_DARK_SKIN_TONE = "\U0001f918\U0001f3fe"
SIGN_OF_THE_HORNS_DARK_SKIN_TONE = "\U0001f918\U0001f3ff"
CALL_ME_HAND = "\U0001f919"
CALL_ME_HAND_LIGHT_SKIN_TONE = "\U0001f919\U0001f3fb"
CALL_ME_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f919\U0001f3fc"
CALL_ME_HAND_MEDIUM_SKIN_TONE = "\U0001f919\U0001f3fd"
CALL_ME_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f919\U0001f3fe"
CALL_ME_HAND_DARK_SKIN_TONE = "\U0001f919\U0001f3ff"
BACKHAND_INDEX_POINTING_LEFT = "\U0001f448"
BACKHAND_INDEX_POINTING_LEFT_LIGHT_SKIN_TONE = "\U0001f448\U0001f3fb"
BACKHAND_INDEX_POINTING_LEFT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f448\U0001f3fc"
BACKHAND_INDEX_POINTING_LEFT_MEDIUM_SKIN_TONE = "\U0001f448\U0001f3fd"
BACKHAND_INDEX_POINTING_LEFT_MEDIUM_DARK_SKIN_TONE = "\U0001f448\U0001f3fe"
BACKHAND_INDEX_POINTING_LEFT_DARK_SKIN_TONE = "\U0001f448\U0001f3ff"
BACKHAND_INDEX_POINTING_RIGHT = "\U0001f449"
BACKHAND_INDEX_POINTING_RIGHT_LIGHT_SKIN_TONE = "\U0001f449\U0001f3fb"
BACKHAND_INDEX_POINTING_RIGHT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f449\U0001f3fc"
BACKHAND_INDEX_POINTING_RIGHT_MEDIUM_SKIN_TONE = "\U0001f449\U0001f3fd"
BACKHAND_INDEX_POINTING_RIGHT_MEDIUM_DARK_SKIN_TONE = "\U0001f449\U0001f3fe"
BACKHAND_INDEX_POINTING_RIGHT_DARK_SKIN_TONE = "\U0001f449\U0001f3ff"
BACKHAND_INDEX_POINTING_UP = "\U0001f446"
BACKHAND_INDEX_POINTING_UP_LIGHT_SKIN_TONE = "\U0001f446\U0001f3fb"
BACKHAND_INDEX_POINTING_UP_MEDIUM_LIGHT_SKIN_TONE = "\U0001f446\U0001f3fc"
BACKHAND_INDEX_POINTING_UP_MEDIUM_SKIN_TONE = "\U0001f446\U0001f3fd"
BACKHAND_INDEX_POINTING_UP_MEDIUM_DARK_SKIN_TONE = "\U0001f446\U0001f3fe"
BACKHAND_INDEX_POINTING_UP_DARK_SKIN_TONE = "\U0001f446\U0001f3ff"
MIDDLE_FINGER = "\U0001f595"
MIDDLE_FINGER_LIGHT_SKIN_TONE = "\U0001f595\U0001f3fb"
MIDDLE_FINGER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f595\U0001f3fc"
MIDDLE_FINGER_MEDIUM_SKIN_TONE = "\U0001f595\U0001f3fd"
MIDDLE_FINGER_MEDIUM_DARK_SKIN_TONE = "\U0001f595\U0001f3fe"
MIDDLE_FINGER_DARK_SKIN_TONE = "\U0001f595\U0001f3ff"
BACKHAND_INDEX_POINTING_DOWN = "\U0001f447"
BACKHAND_INDEX_POINTING_DOWN_LIGHT_SKIN_TONE = "\U0001f447\U0001f3fb"
BACKHAND_INDEX_POINTING_DOWN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f447\U0001f3fc"
BACKHAND_INDEX_POINTING_DOWN_MEDIUM_SKIN_TONE = "\U0001f447\U0001f3fd"
BACKHAND_INDEX_POINTING_DOWN_MEDIUM_DARK_SKIN_TONE = "\U0001f447\U0001f3fe"
BACKHAND_INDEX_POINTING_DOWN_DARK_SKIN_TONE = "\U0001f447\U0001f3ff"
INDEX_POINTING_UP = "\u261d\ufe0f"
INDEX_POINTING_UP_LIGHT_SKIN_TONE = "\u261d\U0001f3fb"
INDEX_POINTING_UP_MEDIUM_LIGHT_SKIN_TONE = "\u261d\U0001f3fc"
INDEX_POINTING_UP_MEDIUM_SKIN_TONE = "\u261d\U0001f3fd"
INDEX_POINTING_UP_MEDIUM_DARK_SKIN_TONE = "\u261d\U0001f3fe"
INDEX_POINTING_UP_DARK_SKIN_TONE = "\u261d\U0001f3ff"
INDEX_POINTING_AT_THE_VIEWER = "\U0001faf5"
INDEX_POINTING_AT_THE_VIEWER_LIGHT_SKIN_TONE = "\U0001faf5\U0001f3fb"
INDEX_POINTING_AT_THE_VIEWER_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf5\U0001f3fc"
INDEX_POINTING_AT_THE_VIEWER_MEDIUM_SKIN_TONE = "\U0001faf5\U0001f3fd"
INDEX_POINTING_AT_THE_VIEWER_MEDIUM_DARK_SKIN_TONE = "\U0001faf5\U0001f3fe"
INDEX_POINTING_AT_THE_VIEWER_DARK_SKIN_TONE = "\U0001faf5\U0001f3ff"
THUMBS_UP = "\U0001f44d"
THUMBS_UP_LIGHT_SKIN_TONE = "\U0001f44d\U0001f3fb"
THUMBS_UP_MEDIUM_LIGHT_SKIN_TONE = "\U0001f44d\U0001f3fc"
THUMBS_UP_MEDIUM_SKIN_TONE = "\U0001f44d\U0001f3fd"
THUMBS_UP_MEDIUM_DARK_SKIN_TONE = "\U0001f44d\U0001f3fe"
THUMBS_UP_DARK_SKIN_TONE = "\U0001f44d\U0001f3ff"
THUMBS_DOWN = "\U0001f44e"
THUMBS_DOWN_LIGHT_SKIN_TONE = "\U0001f44e\U0001f3fb"
THUMBS_DOWN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f44e\U0001f3fc"
THUMBS_DOWN_MEDIUM_SKIN_TONE = "\U0001f44e\U0001f3fd"
THUMBS_DOWN_MEDIUM_DARK_SKIN_TONE = "\U0001f44e\U0001f3fe"
THUMBS_DOWN_DARK_SKIN_TONE = "\U0001f44e\U0001f3ff"
RAISED_FIST = "\u270a"
RAISED_FIST_LIGHT_SKIN_TONE = "\u270a\U0001f3fb"
RAISED_FIST_MEDIUM_LIGHT_SKIN_TONE = "\u270a\U0001f3fc"
RAISED_FIST_MEDIUM_SKIN_TONE = "\u270a\U0001f3fd"
RAISED_FIST_MEDIUM_DARK_SKIN_TONE = "\u270a\U0001f3fe"
RAISED_FIST_DARK_SKIN_TONE = "\u270a\U0001f3ff"
ONCOMING_FIST = "\U0001f44a"
ONCOMING_FIST_LIGHT_SKIN_TONE = "\U0001f44a\U0001f3fb"
ONCOMING_FIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f44a\U0001f3fc"
ONCOMING_FIST_MEDIUM_SKIN_TONE = "\U0001f44a\U0001f3fd"
ONCOMING_FIST_MEDIUM_DARK_SKIN_TONE = "\U0001f44a\U0001f3fe"
ONCOMING_FIST_DARK_SKIN_TONE = "\U0001f44a\U0001f3ff"
LEFT_FACING_FIST = "\U0001f91b"
LEFT_FACING_FIST_LIGHT_SKIN_TONE = "\U0001f91b\U0001f3fb"
LEFT_FACING_FIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f91b\U0001f3fc"
LEFT_FACING_FIST_MEDIUM_SKIN_TONE = "\U0001f91b\U0001f3fd"
LEFT_FACING_FIST_MEDIUM_DARK_SKIN_TONE = "\U0001f91b\U0001f3fe"
LEFT_FACING_FIST_DARK_SKIN_TONE = "\U0001f91b\U0001f3ff"
RIGHT_FACING_FIST = "\U0001f91c"
RIGHT_FACING_FIST_LIGHT_SKIN_TONE = "\U0001f91c\U0001f3fb"
RIGHT_FACING_FIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f91c\U0001f3fc"
RIGHT_FACING_FIST_MEDIUM_SKIN_TONE = "\U0001f91c\U0001f3fd"
RIGHT_FACING_FIST_MEDIUM_DARK_SKIN_TONE = "\U0001f91c\U0001f3fe"
RIGHT_FACING_FIST_DARK_SKIN_TONE = "\U0001f91c\U0001f3ff"
CLAPPING_HANDS = "\U0001f44f"
CLAPPING_HANDS_LIGHT_SKIN_TONE = "\U0001f44f\U0001f3fb"
CLAPPING_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f44f\U0001f3fc"
CLAPPING_HANDS_MEDIUM_SKIN_TONE = "\U0001f44f\U0001f3fd"
CLAPPING_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001f44f\U0001f3fe"
CLAPPING_HANDS_DARK_SKIN_TONE = "\U0001f44f\U0001f3ff"
RAISING_HANDS = "\U0001f64c"
RAISING_HANDS_LIGHT_SKIN_TONE = "\U0001f64c\U0001f3fb"
RAISING_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64c\U0001f3fc"
RAISING_HANDS_MEDIUM_SKIN_TONE = "\U0001f64c\U0001f3fd"
RAISING_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001f64c\U0001f3fe"
RAISING_HANDS_DARK_SKIN_TONE = "\U0001f64c\U0001f3ff"
HEART_HANDS = "\U0001faf6"
HEART_HANDS_LIGHT_SKIN_TONE = "\U0001faf6\U0001f3fb"
HEART_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf6\U0001f3fc"
HEART_HANDS_MEDIUM_SKIN_TONE = "\U0001faf6\U0001f3fd"
HEART_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001faf6\U0001f3fe"
HEART_HANDS_DARK_SKIN_TONE = "\U0001faf6\U0001f3ff"
OPEN_HANDS = "\U0001f450"
OPEN_HANDS_LIGHT_SKIN_TONE = "\U0001f450\U0001f3fb"
OPEN_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f450\U0001f3fc"
OPEN_HANDS_MEDIUM_SKIN_TONE = "\U0001f450\U0001f3fd"
OPEN_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001f450\U0001f3fe"
OPEN_HANDS_DARK_SKIN_TONE = "\U0001f450\U0001f3ff"
PALMS_UP_TOGETHER = "\U0001f932"
PALMS_UP_TOGETHER_LIGHT_SKIN_TONE = "\U0001f932\U0001f3fb"
PALMS_UP_TOGETHER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f932\U0001f3fc"
PALMS_UP_TOGETHER_MEDIUM_SKIN_TONE = "\U0001f932\U0001f3fd"
PALMS_UP_TOGETHER_MEDIUM_DARK_SKIN_TONE = "\U0001f932\U0001f3fe"
PALMS_UP_TOGETHER_DARK_SKIN_TONE = "\U0001f932\U0001f3ff"
HANDSHAKE = "\U0001f91d"
HANDSHAKE_LIGHT_SKIN_TONE = "\U0001f91d\U0001f3fb"
HANDSHAKE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f91d\U0001f3fc"
HANDSHAKE_MEDIUM_SKIN_TONE = "\U0001f91d\U0001f3fd"
HANDSHAKE_MEDIUM_DARK_SKIN_TONE = "\U0001f91d\U0001f3fe"
HANDSHAKE_DARK_SKIN_TONE = "\U0001f91d\U0001f3ff"
HANDSHAKE_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3fb\u200d\U0001faf2\U0001f3fc"
HANDSHAKE_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001faf1\U0001f3fb\u200d\U0001faf2\U0001f3fd"
HANDSHAKE_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = "\U0001faf1\U0001f3fb\u200d\U0001faf2\U0001f3fe"
HANDSHAKE_LIGHT_SKIN_TONE_DARK_SKIN_TONE = "\U0001faf1\U0001f3fb\u200d\U0001faf2\U0001f3ff"
HANDSHAKE_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3fc\u200d\U0001faf2\U0001f3fb"
HANDSHAKE_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001faf1\U0001f3fc\u200d\U0001faf2\U0001f3fd"
HANDSHAKE_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = "\U0001faf1\U0001f3fc\u200d\U0001faf2\U0001f3fe"
HANDSHAKE_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = "\U0001faf1\U0001f3fc\u200d\U0001faf2\U0001f3ff"
HANDSHAKE_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3fd\u200d\U0001faf2\U0001f3fb"
HANDSHAKE_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3fd\u200d\U0001faf2\U0001f3fc"
HANDSHAKE_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = "\U0001faf1\U0001f3fd\u200d\U0001faf2\U0001f3fe"
HANDSHAKE_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = "\U0001faf1\U0001f3fd\u200d\U0001faf2\U0001f3ff"
HANDSHAKE_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3fe\u200d\U0001faf2\U0001f3fb"
HANDSHAKE_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3fe\u200d\U0001faf2\U0001f3fc"
HANDSHAKE_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001faf1\U0001f3fe\u200d\U0001faf2\U0001f3fd"
HANDSHAKE_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = "\U0001faf1\U0001f3fe\u200d\U0001faf2\U0001f3ff"
HANDSHAKE_DARK_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3ff\u200d\U0001faf2\U0001f3fb"
HANDSHAKE_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = "\U0001faf1\U0001f3ff\u200d\U0001faf2\U0001f3fc"
HANDSHAKE_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001faf1\U0001f3ff\u200d\U0001faf2\U0001f3fd"
HANDSHAKE_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = "\U0001faf1\U0001f3ff\u200d\U0001faf2\U0001f3fe"
FOLDED_HANDS = "\U0001f64f"
FOLDED_HANDS_LIGHT_SKIN_TONE = "\U0001f64f\U0001f3fb"
FOLDED_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64f\U0001f3fc"
FOLDED_HANDS_MEDIUM_SKIN_TONE = "\U0001f64f\U0001f3fd"
FOLDED_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001f64f\U0001f3fe"
FOLDED_HANDS_DARK_SKIN_TONE = "\U0001f64f\U0001f3ff"
WRITING_HAND = "\u270d\ufe0f"
WRITING_HAND_LIGHT_SKIN_TONE = "\u270d\U0001f3fb"
WRITING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\u270d\U0001f3fc"
WRITING_HAND_MEDIUM_SKIN_TONE = "\u270d\U0001f3fd"
WRITING_HAND_MEDIUM_DARK_SKIN_TONE = "\u270d\U0001f3fe"
WRITING_HAND_DARK_SKIN_TONE = "\u270d\U0001f3ff"
NAIL_POLISH = "\U0001f485"
NAIL_POLISH_LIGHT_SKIN_TONE = "\U0001f485\U0001f3fb"
NAIL_POLISH_MEDIUM_LIGHT_SKIN_TONE = "\U0001f485\U0001f3fc"
NAIL_POLISH_MEDIUM_SKIN_TONE = "\U0001f485\U0001f3fd"
NAIL_POLISH_MEDIUM_DARK_SKIN_TONE = "\U0001f485\U0001f3fe"
NAIL_POLISH_DARK_SKIN_TONE = "\U0001f485\U0001f3ff"
SELFIE = "\U0001f933"
SELFIE_LIGHT_SKIN_TONE = "\U0001f933\U0001f3fb"
SELFIE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f933\U0001f3fc"
SELFIE_MEDIUM_SKIN_TONE = "\U0001f933\U0001f3fd"
SELFIE_MEDIUM_DARK_SKIN_TONE = "\U0001f933\U0001f3fe"
SELFIE_DARK_SKIN_TONE = "\U0001f933\U0001f3ff"
FLEXED_BICEPS = "\U0001f4aa"
FLEXED_BICEPS_LIGHT_SKIN_TONE = "\U0001f4aa\U0001f3fb"
FLEXED_BICEPS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f4aa\U0001f3fc"
FLEXED_BICEPS_MEDIUM_SKIN_TONE = "\U0001f4aa\U0001f3fd"
FLEXED_BICEPS_MEDIUM_DARK_SKIN_TONE = "\U0001f4aa\U0001f3fe"
FLEXED_BICEPS_DARK_SKIN_TONE = "\U0001f4aa\U0001f3ff"
MECHANICAL_ARM = "\U0001f9be"
MECHANICAL_LEG = "\U0001f9bf"
LEG = "\U0001f9b5"
LEG_LIGHT_SKIN_TONE = "\U0001f9b5\U0001f3fb"
LEG_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9b5\U0001f3fc"
LEG_MEDIUM_SKIN_TONE = "\U0001f9b5\U0001f3fd"
LEG_MEDIUM_DARK_SKIN_TONE = "\U0001f9b5\U0001f3fe"
LEG_DARK_SKIN_TONE = "\U0001f9b5\U0001f3ff"
FOOT = "\U0001f9b6"
FOOT_LIGHT_SKIN_TONE = "\U0001f9b6\U0001f3fb"
FOOT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9b6\U0001f3fc"
FOOT_MEDIUM_SKIN_TONE = "\U0001f9b6\U0001f3fd"
FOOT_MEDIUM_DARK_SKIN_TONE = "\U0001f9b6\U0001f3fe"
FOOT_DARK_SKIN_TONE = "\U0001f9b6\U0001f3ff"
EAR = "\U0001f442"
EAR_LIGHT_SKIN_TONE = "\U0001f442\U0001f3fb"
EAR_MEDIUM_LIGHT_SKIN_TONE = "\U0001f442\U0001f3fc"
EAR_MEDIUM_SKIN_TONE = "\U0001f442\U0001f3fd"
EAR_MEDIUM_DARK_SKIN_TONE = "\U0001f442\U0001f3fe"
EAR_DARK_SKIN_TONE = "\U0001f442\U0001f3ff"
EAR_WITH_HEARING_AID = "\U0001f9bb"
EAR_WITH_HEARING_AID_LIGHT_SKIN_TONE = "\U0001f9bb\U0001f3fb"
EAR_WITH_HEARING_AID_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9bb\U0001f3fc"
EAR_WITH_HEARING_AID_MEDIUM_SKIN_TONE = "\U0001f9bb\U0001f3fd"
EAR_WITH_HEARING_AID_MEDIUM_DARK_SKIN_TONE = "\U0001f9bb\U0001f3fe"
EAR_WITH_HEARING_AID_DARK_SKIN_TONE = "\U0001f9bb\U0001f3ff"
NOSE = "\U0001f443"
NOSE_LIGHT_SKIN_TONE = "\U0001f443\U0001f3fb"
NOSE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f443\U0001f3fc"
NOSE_MEDIUM_SKIN_TONE = "\U0001f443\U0001f3fd"
NOSE_MEDIUM_DARK_SKIN_TONE = "\U0001f443\U0001f3fe"
NOSE_DARK_SKIN_TONE = "\U0001f443\U0001f3ff"
BRAIN = "\U0001f9e0"
ANATOMICAL_HEART = "\U0001fac0"
LUNGS = "\U0001fac1"
TOOTH = "\U0001f9b7"
BONE = "\U0001f9b4"
EYES = "\U0001f440"
EYE = "\U0001f441\ufe0f"
TONGUE = "\U0001f445"
MOUTH = "\U0001f444"
BITING_LIP = "\U0001fae6"
BABY = "\U0001f476"
BABY_LIGHT_SKIN_TONE = "\U0001f476\U0001f3fb"
BABY_MEDIUM_LIGHT_SKIN_TONE = "\U0001f476\U0001f3fc"
BABY_MEDIUM_SKIN_TONE = "\U0001f476\U0001f3fd"
BABY_MEDIUM_DARK_SKIN_TONE = "\U0001f476\U0001f3fe"
BABY_DARK_SKIN_TONE = "\U0001f476\U0001f3ff"
CHILD = "\U0001f9d2"
CHILD_LIGHT_SKIN_TONE = "\U0001f9d2\U0001f3fb"
CHILD_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d2\U0001f3fc"
CHILD_MEDIUM_SKIN_TONE = "\U0001f9d2\U0001f3fd"
CHILD_MEDIUM_DARK_SKIN_TONE = "\U0001f9d2\U0001f3fe"
CHILD_DARK_SKIN_TONE = "\U0001f9d2\U0001f3ff"
BOY = "\U0001f466"
BOY_LIGHT_SKIN_TONE = "\U0001f466\U0001f3fb"
BOY_MEDIUM_LIGHT_SKIN_TONE = "\U0001f466\U0001f3fc"
BOY_MEDIUM_SKIN_TONE = "\U0001f466\U0001f3fd"
BOY_MEDIUM_DARK_SKIN_TONE = "\U0001f466\U0001f3fe"
BOY_DARK_SKIN_TONE = "\U0001f466\U0001f3ff"
GIRL = "\U0001f467"
GIRL_LIGHT_SKIN_TONE = "\U0001f467\U0001f3fb"
GIRL_MEDIUM_LIGHT_SKIN_TONE = "\U0001f467\U0001f3fc"
GIRL_MEDIUM_SKIN_TONE = "\U0001f467\U0001f3fd"
GIRL_MEDIUM_DARK_SKIN_TONE = "\U0001f467\U0001f3fe"
GIRL_DARK_SKIN_TONE = "\U0001f467\U0001f3ff"
PERSON = "\U0001f9d1"
PERSON_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb"
PERSON_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc"
PERSON_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd"
PERSON_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe"
PERSON_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff"
PERSON_BLOND_HAIR = "\U0001f471"
PERSON_LIGHT_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fb"
PERSON_MEDIUM_LIGHT_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fc"
PERSON_MEDIUM_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fd"
PERSON_MEDIUM_DARK_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fe"
PERSON_DARK_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3ff"
MAN = "\U0001f468"
MAN_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb"
MAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc"
MAN_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd"
MAN_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe"
MAN_DARK_SKIN_TONE = "\U0001f468\U0001f3ff"
PERSON_BEARD = "\U0001f9d4"
PERSON_LIGHT_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fb"
PERSON_MEDIUM_LIGHT_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fc"
PERSON_MEDIUM_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fd"
PERSON_MEDIUM_DARK_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fe"
PERSON_DARK_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3ff"
MAN_BEARD = "\U0001f9d4\u200d\u2642\ufe0f"
MAN_LIGHT_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fb\u200d\u2642\ufe0f"
MAN_MEDIUM_LIGHT_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fc\u200d\u2642\ufe0f"
MAN_MEDIUM_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fd\u200d\u2642\ufe0f"
MAN_MEDIUM_DARK_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fe\u200d\u2642\ufe0f"
MAN_DARK_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_BEARD = "\U0001f9d4\u200d\u2640\ufe0f"
WOMAN_LIGHT_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_MEDIUM_LIGHT_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_MEDIUM_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_MEDIUM_DARK_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_DARK_SKIN_TONE_BEARD = "\U0001f9d4\U0001f3ff\u200d\u2640\ufe0f"
MAN_RED_HAIR = "\U0001f468\u200d\U0001f9b0"
MAN_LIGHT_SKIN_TONE_RED_HAIR = "\U0001f468\U0001f3fb\u200d\U0001f9b0"
MAN_MEDIUM_LIGHT_SKIN_TONE_RED_HAIR = "\U0001f468\U0001f3fc\u200d\U0001f9b0"
MAN_MEDIUM_SKIN_TONE_RED_HAIR = "\U0001f468\U0001f3fd\u200d\U0001f9b0"
MAN_MEDIUM_DARK_SKIN_TONE_RED_HAIR = "\U0001f468\U0001f3fe\u200d\U0001f9b0"
MAN_DARK_SKIN_TONE_RED_HAIR = "\U0001f468\U0001f3ff\u200d\U0001f9b0"
MAN_CURLY_HAIR = "\U0001f468\u200d\U0001f9b1"
MAN_LIGHT_SKIN_TONE_CURLY_HAIR = "\U0001f468\U0001f3fb\u200d\U0001f9b1"
MAN_MEDIUM_LIGHT_SKIN_TONE_CURLY_HAIR = "\U0001f468\U0001f3fc\u200d\U0001f9b1"
MAN_MEDIUM_SKIN_TONE_CURLY_HAIR = "\U0001f468\U0001f3fd\u200d\U0001f9b1"
MAN_MEDIUM_DARK_SKIN_TONE_CURLY_HAIR = "\U0001f468\U0001f3fe\u200d\U0001f9b1"
MAN_DARK_SKIN_TONE_CURLY_HAIR = "\U0001f468\U0001f3ff\u200d\U0001f9b1"
MAN_WHITE_HAIR = "\U0001f468\u200d\U0001f9b3"
MAN_LIGHT_SKIN_TONE_WHITE_HAIR = "\U0001f468\U0001f3fb\u200d\U0001f9b3"
MAN_MEDIUM_LIGHT_SKIN_TONE_WHITE_HAIR = "\U0001f468\U0001f3fc\u200d\U0001f9b3"
MAN_MEDIUM_SKIN_TONE_WHITE_HAIR = "\U0001f468\U0001f3fd\u200d\U0001f9b3"
MAN_MEDIUM_DARK_SKIN_TONE_WHITE_HAIR = "\U0001f468\U0001f3fe\u200d\U0001f9b3"
MAN_DARK_SKIN_TONE_WHITE_HAIR = "\U0001f468\U0001f3ff\u200d\U0001f9b3"
MAN_BALD = "\U0001f468\u200d\U0001f9b2"
MAN_LIGHT_SKIN_TONE_BALD = "\U0001f468\U0001f3fb\u200d\U0001f9b2"
MAN_MEDIUM_LIGHT_SKIN_TONE_BALD = "\U0001f468\U0001f3fc\u200d\U0001f9b2"
MAN_MEDIUM_SKIN_TONE_BALD = "\U0001f468\U0001f3fd\u200d\U0001f9b2"
MAN_MEDIUM_DARK_SKIN_TONE_BALD = "\U0001f468\U0001f3fe\u200d\U0001f9b2"
MAN_DARK_SKIN_TONE_BALD = "\U0001f468\U0001f3ff\u200d\U0001f9b2"
WOMAN = "\U0001f469"
WOMAN_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb"
WOMAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc"
WOMAN_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd"
WOMAN_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe"
WOMAN_DARK_SKIN_TONE = "\U0001f469\U0001f3ff"
WOMAN_RED_HAIR = "\U0001f469\u200d\U0001f9b0"
WOMAN_LIGHT_SKIN_TONE_RED_HAIR = "\U0001f469\U0001f3fb\u200d\U0001f9b0"
WOMAN_MEDIUM_LIGHT_SKIN_TONE_RED_HAIR = "\U0001f469\U0001f3fc\u200d\U0001f9b0"
WOMAN_MEDIUM_SKIN_TONE_RED_HAIR = "\U0001f469\U0001f3fd\u200d\U0001f9b0"
WOMAN_MEDIUM_DARK_SKIN_TONE_RED_HAIR = "\U0001f469\U0001f3fe\u200d\U0001f9b0"
WOMAN_DARK_SKIN_TONE_RED_HAIR = "\U0001f469\U0001f3ff\u200d\U0001f9b0"
PERSON_RED_HAIR = "\U0001f9d1\u200d\U0001f9b0"
PERSON_LIGHT_SKIN_TONE_RED_HAIR = "\U0001f9d1\U0001f3fb\u200d\U0001f9b0"
PERSON_MEDIUM_LIGHT_SKIN_TONE_RED_HAIR = "\U0001f9d1\U0001f3fc\u200d\U0001f9b0"
PERSON_MEDIUM_SKIN_TONE_RED_HAIR = "\U0001f9d1\U0001f3fd\u200d\U0001f9b0"
PERSON_MEDIUM_DARK_SKIN_TONE_RED_HAIR = "\U0001f9d1\U0001f3fe\u200d\U0001f9b0"
PERSON_DARK_SKIN_TONE_RED_HAIR = "\U0001f9d1\U0001f3ff\u200d\U0001f9b0"
WOMAN_CURLY_HAIR = "\U0001f469\u200d\U0001f9b1"
WOMAN_LIGHT_SKIN_TONE_CURLY_HAIR = "\U0001f469\U0001f3fb\u200d\U0001f9b1"
WOMAN_MEDIUM_LIGHT_SKIN_TONE_CURLY_HAIR = "\U0001f469\U0001f3fc\u200d\U0001f9b1"
WOMAN_MEDIUM_SKIN_TONE_CURLY_HAIR = "\U0001f469\U0001f3fd\u200d\U0001f9b1"
WOMAN_MEDIUM_DARK_SKIN_TONE_CURLY_HAIR = "\U0001f469\U0001f3fe\u200d\U0001f9b1"
WOMAN_DARK_SKIN_TONE_CURLY_HAIR = "\U0001f469\U0001f3ff\u200d\U0001f9b1"
PERSON_CURLY_HAIR = "\U0001f9d1\u200d\U0001f9b1"
PERSON_LIGHT_SKIN_TONE_CURLY_HAIR = "\U0001f9d1\U0001f3fb\u200d\U0001f9b1"
PERSON_MEDIUM_LIGHT_SKIN_TONE_CURLY_HAIR = "\U0001f9d1\U0001f3fc\u200d\U0001f9b1"
PERSON_MEDIUM_SKIN_TONE_CURLY_HAIR = "\U0001f9d1\U0001f3fd\u200d\U0001f9b1"
PERSON_MEDIUM_DARK_SKIN_TONE_CURLY_HAIR = "\U0001f9d1\U0001f3fe\u200d\U0001f9b1"
PERSON_DARK_SKIN_TONE_CURLY_HAIR = "\U0001f9d1\U0001f3ff\u200d\U0001f9b1"
WOMAN_WHITE_HAIR = "\U0001f469\u200d\U0001f9b3"
WOMAN_LIGHT_SKIN_TONE_WHITE_HAIR = "\U0001f469\U0001f3fb\u200d\U0001f9b3"
WOMAN_MEDIUM_LIGHT_SKIN_TONE_WHITE_HAIR = "\U0001f469\U0001f3fc\u200d\U0001f9b3"
WOMAN_MEDIUM_SKIN_TONE_WHITE_HAIR = "\U0001f469\U0001f3fd\u200d\U0001f9b3"
WOMAN_MEDIUM_DARK_SKIN_TONE_WHITE_HAIR = "\U0001f469\U0001f3fe\u200d\U0001f9b3"
WOMAN_DARK_SKIN_TONE_WHITE_HAIR = "\U0001f469\U0001f3ff\u200d\U0001f9b3"
PERSON_WHITE_HAIR = "\U0001f9d1\u200d\U0001f9b3"
PERSON_LIGHT_SKIN_TONE_WHITE_HAIR = "\U0001f9d1\U0001f3fb\u200d\U0001f9b3"
PERSON_MEDIUM_LIGHT_SKIN_TONE_WHITE_HAIR = "\U0001f9d1\U0001f3fc\u200d\U0001f9b3"
PERSON_MEDIUM_SKIN_TONE_WHITE_HAIR = "\U0001f9d1\U0001f3fd\u200d\U0001f9b3"
PERSON_MEDIUM_DARK_SKIN_TONE_WHITE_HAIR = "\U0001f9d1\U0001f3fe\u200d\U0001f9b3"
PERSON_DARK_SKIN_TONE_WHITE_HAIR = "\U0001f9d1\U0001f3ff\u200d\U0001f9b3"
WOMAN_BALD = "\U0001f469\u200d\U0001f9b2"
WOMAN_LIGHT_SKIN_TONE_BALD = "\U0001f469\U0001f3fb\u200d\U0001f9b2"
WOMAN_MEDIUM_LIGHT_SKIN_TONE_BALD = "\U0001f469\U0001f3fc\u200d\U0001f9b2"
WOMAN_MEDIUM_SKIN_TONE_BALD = "\U0001f469\U0001f3fd\u200d\U0001f9b2"
WOMAN_MEDIUM_DARK_SKIN_TONE_BALD = "\U0001f469\U0001f3fe\u200d\U0001f9b2"
WOMAN_DARK_SKIN_TONE_BALD = "\U0001f469\U0001f3ff\u200d\U0001f9b2"
PERSON_BALD = "\U0001f9d1\u200d\U0001f9b2"
PERSON_LIGHT_SKIN_TONE_BALD = "\U0001f9d1\U0001f3fb\u200d\U0001f9b2"
PERSON_MEDIUM_LIGHT_SKIN_TONE_BALD = "\U0001f9d1\U0001f3fc\u200d\U0001f9b2"
PERSON_MEDIUM_SKIN_TONE_BALD = "\U0001f9d1\U0001f3fd\u200d\U0001f9b2"
PERSON_MEDIUM_DARK_SKIN_TONE_BALD = "\U0001f9d1\U0001f3fe\u200d\U0001f9b2"
PERSON_DARK_SKIN_TONE_BALD = "\U0001f9d1\U0001f3ff\u200d\U0001f9b2"
WOMAN_BLOND_HAIR = "\U0001f471\u200d\u2640\ufe0f"
WOMAN_LIGHT_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_MEDIUM_LIGHT_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_MEDIUM_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_MEDIUM_DARK_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_DARK_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3ff\u200d\u2640\ufe0f"
MAN_BLOND_HAIR = "\U0001f471\u200d\u2642\ufe0f"
MAN_LIGHT_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fb\u200d\u2642\ufe0f"
MAN_MEDIUM_LIGHT_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fc\u200d\u2642\ufe0f"
MAN_MEDIUM_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fd\u200d\u2642\ufe0f"
MAN_MEDIUM_DARK_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3fe\u200d\u2642\ufe0f"
MAN_DARK_SKIN_TONE_BLOND_HAIR = "\U0001f471\U0001f3ff\u200d\u2642\ufe0f"
OLDER_PERSON = "\U0001f9d3"
OLDER_PERSON_LIGHT_SKIN_TONE = "\U0001f9d3\U0001f3fb"
OLDER_PERSON_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d3\U0001f3fc"
OLDER_PERSON_MEDIUM_SKIN_TONE = "\U0001f9d3\U0001f3fd"
OLDER_PERSON_MEDIUM_DARK_SKIN_TONE = "\U0001f9d3\U0001f3fe"
OLDER_PERSON_DARK_SKIN_TONE = "\U0001f9d3\U0001f3ff"
OLD_MAN = "\U0001f474"
OLD_MAN_LIGHT_SKIN_TONE = "\U0001f474\U0001f3fb"
OLD_MAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f474\U0001f3fc"
OLD_MAN_MEDIUM_SKIN_TONE = "\U0001f474\U0001f3fd"
OLD_MAN_MEDIUM_DARK_SKIN_TONE = "\U0001f474\U0001f3fe"
OLD_MAN_DARK_SKIN_TONE = "\U0001f474\U0001f3ff"
OLD_WOMAN = "\U0001f475"
OLD_WOMAN_LIGHT_SKIN_TONE = "\U0001f475\U0001f3fb"
OLD_WOMAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f475\U0001f3fc"
OLD_WOMAN_MEDIUM_SKIN_TONE = "\U0001f475\U0001f3fd"
OLD_WOMAN_MEDIUM_DARK_SKIN_TONE = "\U0001f475\U0001f3fe"
OLD_WOMAN_DARK_SKIN_TONE = "\U0001f475\U0001f3ff"
PERSON_FROWNING = "\U0001f64d"
PERSON_FROWNING_LIGHT_SKIN_TONE = "\U0001f64d\U0001f3fb"
PERSON_FROWNING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64d\U0001f3fc"
PERSON_FROWNING_MEDIUM_SKIN_TONE = "\U0001f64d\U0001f3fd"
PERSON_FROWNING_MEDIUM_DARK_SKIN_TONE = "\U0001f64d\U0001f3fe"
PERSON_FROWNING_DARK_SKIN_TONE = "\U0001f64d\U0001f3ff"
MAN_FROWNING = "\U0001f64d\u200d\u2642\ufe0f"
MAN_FROWNING_LIGHT_SKIN_TONE = "\U0001f64d\U0001f3fb\u200d\u2642\ufe0f"
MAN_FROWNING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64d\U0001f3fc\u200d\u2642\ufe0f"
MAN_FROWNING_MEDIUM_SKIN_TONE = "\U0001f64d\U0001f3fd\u200d\u2642\ufe0f"
MAN_FROWNING_MEDIUM_DARK_SKIN_TONE = "\U0001f64d\U0001f3fe\u200d\u2642\ufe0f"
MAN_FROWNING_DARK_SKIN_TONE = "\U0001f64d\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_FROWNING = "\U0001f64d\u200d\u2640\ufe0f"
WOMAN_FROWNING_LIGHT_SKIN_TONE = "\U0001f64d\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_FROWNING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64d\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_FROWNING_MEDIUM_SKIN_TONE = "\U0001f64d\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_FROWNING_MEDIUM_DARK_SKIN_TONE = "\U0001f64d\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_FROWNING_DARK_SKIN_TONE = "\U0001f64d\U0001f3ff\u200d\u2640\ufe0f"
PERSON_POUTING = "\U0001f64e"
PERSON_POUTING_LIGHT_SKIN_TONE = "\U0001f64e\U0001f3fb"
PERSON_POUTING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64e\U0001f3fc"
PERSON_POUTING_MEDIUM_SKIN_TONE = "\U0001f64e\U0001f3fd"
PERSON_POUTING_MEDIUM_DARK_SKIN_TONE = "\U0001f64e\U0001f3fe"
PERSON_POUTING_DARK_SKIN_TONE = "\U0001f64e\U0001f3ff"
MAN_POUTING = "\U0001f64e\u200d\u2642\ufe0f"
MAN_POUTING_LIGHT_SKIN_TONE = "\U0001f64e\U0001f3fb\u200d\u2642\ufe0f"
MAN_POUTING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64e\U0001f3fc\u200d\u2642\ufe0f"
MAN_POUTING_MEDIUM_SKIN_TONE = "\U0001f64e\U0001f3fd\u200d\u2642\ufe0f"
MAN_POUTING_MEDIUM_DARK_SKIN_TONE = "\U0001f64e\U0001f3fe\u200d\u2642\ufe0f"
MAN_POUTING_DARK_SKIN_TONE = "\U0001f64e\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_POUTING = "\U0001f64e\u200d\u2640\ufe0f"
WOMAN_POUTING_LIGHT_SKIN_TONE = "\U0001f64e\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_POUTING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64e\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_POUTING_MEDIUM_SKIN_TONE = "\U0001f64e\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_POUTING_MEDIUM_DARK_SKIN_TONE = "\U0001f64e\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_POUTING_DARK_SKIN_TONE = "\U0001f64e\U0001f3ff\u200d\u2640\ufe0f"
PERSON_GESTURING_NO = "\U0001f645"
PERSON_GESTURING_NO_LIGHT_SKIN_TONE = "\U0001f645\U0001f3fb"
PERSON_GESTURING_NO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f645\U0001f3fc"
PERSON_GESTURING_NO_MEDIUM_SKIN_TONE = "\U0001f645\U0001f3fd"
PERSON_GESTURING_NO_MEDIUM_DARK_SKIN_TONE = "\U0001f645\U0001f3fe"
PERSON_GESTURING_NO_DARK_SKIN_TONE = "\U0001f645\U0001f3ff"
MAN_GESTURING_NO = "\U0001f645\u200d\u2642\ufe0f"
MAN_GESTURING_NO_LIGHT_SKIN_TONE = "\U0001f645\U0001f3fb\u200d\u2642\ufe0f"
MAN_GESTURING_NO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f645\U0001f3fc\u200d\u2642\ufe0f"
MAN_GESTURING_NO_MEDIUM_SKIN_TONE = "\U0001f645\U0001f3fd\u200d\u2642\ufe0f"
MAN_GESTURING_NO_MEDIUM_DARK_SKIN_TONE = "\U0001f645\U0001f3fe\u200d\u2642\ufe0f"
MAN_GESTURING_NO_DARK_SKIN_TONE = "\U0001f645\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_GESTURING_NO = "\U0001f645\u200d\u2640\ufe0f"
WOMAN_GESTURING_NO_LIGHT_SKIN_TONE = "\U0001f645\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_GESTURING_NO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f645\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_GESTURING_NO_MEDIUM_SKIN_TONE = "\U0001f645\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_GESTURING_NO_MEDIUM_DARK_SKIN_TONE = "\U0001f645\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_GESTURING_NO_DARK_SKIN_TONE = "\U0001f645\U0001f3ff\u200d\u2640\ufe0f"
PERSON_GESTURING_OK = "\U0001f646"
PERSON_GESTURING_OK_LIGHT_SKIN_TONE = "\U0001f646\U0001f3fb"
PERSON_GESTURING_OK_MEDIUM_LIGHT_SKIN_TONE = "\U0001f646\U0001f3fc"
PERSON_GESTURING_OK_MEDIUM_SKIN_TONE = "\U0001f646\U0001f3fd"
PERSON_GESTURING_OK_MEDIUM_DARK_SKIN_TONE = "\U0001f646\U0001f3fe"
PERSON_GESTURING_OK_DARK_SKIN_TONE = "\U0001f646\U0001f3ff"
MAN_GESTURING_OK = "\U0001f646\u200d\u2642\ufe0f"
MAN_GESTURING_OK_LIGHT_SKIN_TONE = "\U0001f646\U0001f3fb\u200d\u2642\ufe0f"
MAN_GESTURING_OK_MEDIUM_LIGHT_SKIN_TONE = "\U0001f646\U0001f3fc\u200d\u2642\ufe0f"
MAN_GESTURING_OK_MEDIUM_SKIN_TONE = "\U0001f646\U0001f3fd\u200d\u2642\ufe0f"
MAN_GESTURING_OK_MEDIUM_DARK_SKIN_TONE = "\U0001f646\U0001f3fe\u200d\u2642\ufe0f"
MAN_GESTURING_OK_DARK_SKIN_TONE = "\U0001f646\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_GESTURING_OK = "\U0001f646\u200d\u2640\ufe0f"
WOMAN_GESTURING_OK_LIGHT_SKIN_TONE = "\U0001f646\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_GESTURING_OK_MEDIUM_LIGHT_SKIN_TONE = "\U0001f646\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_GESTURING_OK_MEDIUM_SKIN_TONE = "\U0001f646\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_GESTURING_OK_MEDIUM_DARK_SKIN_TONE = "\U0001f646\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_GESTURING_OK_DARK_SKIN_TONE = "\U0001f646\U0001f3ff\u200d\u2640\ufe0f"
PERSON_TIPPING_HAND = "\U0001f481"
PERSON_TIPPING_HAND_LIGHT_SKIN_TONE = "\U0001f481\U0001f3fb"
PERSON_TIPPING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f481\U0001f3fc"
PERSON_TIPPING_HAND_MEDIUM_SKIN_TONE = "\U0001f481\U0001f3fd"
PERSON_TIPPING_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f481\U0001f3fe"
PERSON_TIPPING_HAND_DARK_SKIN_TONE = "\U0001f481\U0001f3ff"
MAN_TIPPING_HAND = "\U0001f481\u200d\u2642\ufe0f"
MAN_TIPPING_HAND_LIGHT_SKIN_TONE = "\U0001f481\U0001f3fb\u200d\u2642\ufe0f"
MAN_TIPPING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f481\U0001f3fc\u200d\u2642\ufe0f"
MAN_TIPPING_HAND_MEDIUM_SKIN_TONE = "\U0001f481\U0001f3fd\u200d\u2642\ufe0f"
MAN_TIPPING_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f481\U0001f3fe\u200d\u2642\ufe0f"
MAN_TIPPING_HAND_DARK_SKIN_TONE = "\U0001f481\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_TIPPING_HAND = "\U0001f481\u200d\u2640\ufe0f"
WOMAN_TIPPING_HAND_LIGHT_SKIN_TONE = "\U0001f481\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_TIPPING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f481\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_TIPPING_HAND_MEDIUM_SKIN_TONE = "\U0001f481\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_TIPPING_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f481\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_TIPPING_HAND_DARK_SKIN_TONE = "\U0001f481\U0001f3ff\u200d\u2640\ufe0f"
PERSON_RAISING_HAND = "\U0001f64b"
PERSON_RAISING_HAND_LIGHT_SKIN_TONE = "\U0001f64b\U0001f3fb"
PERSON_RAISING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64b\U0001f3fc"
PERSON_RAISING_HAND_MEDIUM_SKIN_TONE = "\U0001f64b\U0001f3fd"
PERSON_RAISING_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f64b\U0001f3fe"
PERSON_RAISING_HAND_DARK_SKIN_TONE = "\U0001f64b\U0001f3ff"
MAN_RAISING_HAND = "\U0001f64b\u200d\u2642\ufe0f"
MAN_RAISING_HAND_LIGHT_SKIN_TONE = "\U0001f64b\U0001f3fb\u200d\u2642\ufe0f"
MAN_RAISING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64b\U0001f3fc\u200d\u2642\ufe0f"
MAN_RAISING_HAND_MEDIUM_SKIN_TONE = "\U0001f64b\U0001f3fd\u200d\u2642\ufe0f"
MAN_RAISING_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f64b\U0001f3fe\u200d\u2642\ufe0f"
MAN_RAISING_HAND_DARK_SKIN_TONE = "\U0001f64b\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_RAISING_HAND = "\U0001f64b\u200d\u2640\ufe0f"
WOMAN_RAISING_HAND_LIGHT_SKIN_TONE = "\U0001f64b\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_RAISING_HAND_MEDIUM_LIGHT_SKIN_TONE = "\U0001f64b\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_RAISING_HAND_MEDIUM_SKIN_TONE = "\U0001f64b\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_RAISING_HAND_MEDIUM_DARK_SKIN_TONE = "\U0001f64b\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_RAISING_HAND_DARK_SKIN_TONE = "\U0001f64b\U0001f3ff\u200d\u2640\ufe0f"
DEAF_PERSON = "\U0001f9cf"
DEAF_PERSON_LIGHT_SKIN_TONE = "\U0001f9cf\U0001f3fb"
DEAF_PERSON_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9cf\U0001f3fc"
DEAF_PERSON_MEDIUM_SKIN_TONE = "\U0001f9cf\U0001f3fd"
DEAF_PERSON_MEDIUM_DARK_SKIN_TONE = "\U0001f9cf\U0001f3fe"
DEAF_PERSON_DARK_SKIN_TONE = "\U0001f9cf\U0001f3ff"
DEAF_MAN = "\U0001f9cf\u200d\u2642\ufe0f"
DEAF_MAN_LIGHT_SKIN_TONE = "\U0001f9cf\U0001f3fb\u200d\u2642\ufe0f"
DEAF_MAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9cf\U0001f3fc\u200d\u2642\ufe0f"
DEAF_MAN_MEDIUM_SKIN_TONE = "\U0001f9cf\U0001f3fd\u200d\u2642\ufe0f"
DEAF_MAN_MEDIUM_DARK_SKIN_TONE = "\U0001f9cf\U0001f3fe\u200d\u2642\ufe0f"
DEAF_MAN_DARK_SKIN_TONE = "\U0001f9cf\U0001f3ff\u200d\u2642\ufe0f"
DEAF_WOMAN = "\U0001f9cf\u200d\u2640\ufe0f"
DEAF_WOMAN_LIGHT_SKIN_TONE = "\U0001f9cf\U0001f3fb\u200d\u2640\ufe0f"
DEAF_WOMAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9cf\U0001f3fc\u200d\u2640\ufe0f"
DEAF_WOMAN_MEDIUM_SKIN_TONE = "\U0001f9cf\U0001f3fd\u200d\u2640\ufe0f"
DEAF_WOMAN_MEDIUM_DARK_SKIN_TONE = "\U0001f9cf\U0001f3fe\u200d\u2640\ufe0f"
DEAF_WOMAN_DARK_SKIN_TONE = "\U0001f9cf\U0001f3ff\u200d\u2640\ufe0f"
PERSON_BOWING = "\U0001f647"
PERSON_BOWING_LIGHT_SKIN_TONE = "\U0001f647\U0001f3fb"
PERSON_BOWING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f647\U0001f3fc"
PERSON_BOWING_MEDIUM_SKIN_TONE = "\U0001f647\U0001f3fd"
PERSON_BOWING_MEDIUM_DARK_SKIN_TONE = "\U0001f647\U0001f3fe"
PERSON_BOWING_DARK_SKIN_TONE = "\U0001f647\U0001f3ff"
MAN_BOWING = "\U0001f647\u200d\u2642\ufe0f"
MAN_BOWING_LIGHT_SKIN_TONE = "\U0001f647\U0001f3fb\u200d\u2642\ufe0f"
MAN_BOWING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f647\U0001f3fc\u200d\u2642\ufe0f"
MAN_BOWING_MEDIUM_SKIN_TONE = "\U0001f647\U0001f3fd\u200d\u2642\ufe0f"
MAN_BOWING_MEDIUM_DARK_SKIN_TONE = "\U0001f647\U0001f3fe\u200d\u2642\ufe0f"
MAN_BOWING_DARK_SKIN_TONE = "\U0001f647\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_BOWING = "\U0001f647\u200d\u2640\ufe0f"
WOMAN_BOWING_LIGHT_SKIN_TONE = "\U0001f647\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_BOWING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f647\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_BOWING_MEDIUM_SKIN_TONE = "\U0001f647\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_BOWING_MEDIUM_DARK_SKIN_TONE = "\U0001f647\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_BOWING_DARK_SKIN_TONE = "\U0001f647\U0001f3ff\u200d\u2640\ufe0f"
PERSON_FACEPALMING = "\U0001f926"
PERSON_FACEPALMING_LIGHT_SKIN_TONE = "\U0001f926\U0001f3fb"
PERSON_FACEPALMING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f926\U0001f3fc"
PERSON_FACEPALMING_MEDIUM_SKIN_TONE = "\U0001f926\U0001f3fd"
PERSON_FACEPALMING_MEDIUM_DARK_SKIN_TONE = "\U0001f926\U0001f3fe"
PERSON_FACEPALMING_DARK_SKIN_TONE = "\U0001f926\U0001f3ff"
MAN_FACEPALMING = "\U0001f926\u200d\u2642\ufe0f"
MAN_FACEPALMING_LIGHT_SKIN_TONE = "\U0001f926\U0001f3fb\u200d\u2642\ufe0f"
MAN_FACEPALMING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f926\U0001f3fc\u200d\u2642\ufe0f"
MAN_FACEPALMING_MEDIUM_SKIN_TONE = "\U0001f926\U0001f3fd\u200d\u2642\ufe0f"
MAN_FACEPALMING_MEDIUM_DARK_SKIN_TONE = "\U0001f926\U0001f3fe\u200d\u2642\ufe0f"
MAN_FACEPALMING_DARK_SKIN_TONE = "\U0001f926\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_FACEPALMING = "\U0001f926\u200d\u2640\ufe0f"
WOMAN_FACEPALMING_LIGHT_SKIN_TONE = "\U0001f926\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_FACEPALMING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f926\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_FACEPALMING_MEDIUM_SKIN_TONE = "\U0001f926\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_FACEPALMING_MEDIUM_DARK_SKIN_TONE = "\U0001f926\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_FACEPALMING_DARK_SKIN_TONE = "\U0001f926\U0001f3ff\u200d\u2640\ufe0f"
PERSON_SHRUGGING = "\U0001f937"
PERSON_SHRUGGING_LIGHT_SKIN_TONE = "\U0001f937\U0001f3fb"
PERSON_SHRUGGING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f937\U0001f3fc"
PERSON_SHRUGGING_MEDIUM_SKIN_TONE = "\U0001f937\U0001f3fd"
PERSON_SHRUGGING_MEDIUM_DARK_SKIN_TONE = "\U0001f937\U0001f3fe"
PERSON_SHRUGGING_DARK_SKIN_TONE = "\U0001f937\U0001f3ff"
MAN_SHRUGGING = "\U0001f937\u200d\u2642\ufe0f"
MAN_SHRUGGING_LIGHT_SKIN_TONE = "\U0001f937\U0001f3fb\u200d\u2642\ufe0f"
MAN_SHRUGGING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f937\U0001f3fc\u200d\u2642\ufe0f"
MAN_SHRUGGING_MEDIUM_SKIN_TONE = "\U0001f937\U0001f3fd\u200d\u2642\ufe0f"
MAN_SHRUGGING_MEDIUM_DARK_SKIN_TONE = "\U0001f937\U0001f3fe\u200d\u2642\ufe0f"
MAN_SHRUGGING_DARK_SKIN_TONE = "\U0001f937\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_SHRUGGING = "\U0001f937\u200d\u2640\ufe0f"
WOMAN_SHRUGGING_LIGHT_SKIN_TONE = "\U0001f937\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_SHRUGGING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f937\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_SHRUGGING_MEDIUM_SKIN_TONE = "\U0001f937\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_SHRUGGING_MEDIUM_DARK_SKIN_TONE = "\U0001f937\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_SHRUGGING_DARK_SKIN_TONE = "\U0001f937\U0001f3ff\u200d\u2640\ufe0f"
HEALTH_WORKER = "\U0001f9d1\u200d\u2695\ufe0f"
HEALTH_WORKER_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\u2695\ufe0f"
HEALTH_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\u2695\ufe0f"
HEALTH_WORKER_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\u2695\ufe0f"
HEALTH_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\u2695\ufe0f"
HEALTH_WORKER_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\u2695\ufe0f"
MAN_HEALTH_WORKER = "\U0001f468\u200d\u2695\ufe0f"
MAN_HEALTH_WORKER_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\u2695\ufe0f"
MAN_HEALTH_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\u2695\ufe0f"
MAN_HEALTH_WORKER_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\u2695\ufe0f"
MAN_HEALTH_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\u2695\ufe0f"
MAN_HEALTH_WORKER_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\u2695\ufe0f"
WOMAN_HEALTH_WORKER = "\U0001f469\u200d\u2695\ufe0f"
WOMAN_HEALTH_WORKER_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\u2695\ufe0f"
WOMAN_HEALTH_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\u2695\ufe0f"
WOMAN_HEALTH_WORKER_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\u2695\ufe0f"
WOMAN_HEALTH_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\u2695\ufe0f"
WOMAN_HEALTH_WORKER_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\u2695\ufe0f"
STUDENT = "\U0001f9d1\u200d\U0001f393"
STUDENT_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f393"
STUDENT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f393"
STUDENT_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f393"
STUDENT_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f393"
STUDENT_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f393"
MAN_STUDENT = "\U0001f468\u200d\U0001f393"
MAN_STUDENT_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f393"
MAN_STUDENT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f393"
MAN_STUDENT_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f393"
MAN_STUDENT_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f393"
MAN_STUDENT_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f393"
WOMAN_STUDENT = "\U0001f469\u200d\U0001f393"
WOMAN_STUDENT_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f393"
WOMAN_STUDENT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f393"
WOMAN_STUDENT_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f393"
WOMAN_STUDENT_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f393"
WOMAN_STUDENT_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f393"
TEACHER = "\U0001f9d1\u200d\U0001f3eb"
TEACHER_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f3eb"
TEACHER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f3eb"
TEACHER_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f3eb"
TEACHER_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f3eb"
TEACHER_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f3eb"
MAN_TEACHER = "\U0001f468\u200d\U0001f3eb"
MAN_TEACHER_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f3eb"
MAN_TEACHER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f3eb"
MAN_TEACHER_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f3eb"
MAN_TEACHER_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f3eb"
MAN_TEACHER_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f3eb"
WOMAN_TEACHER = "\U0001f469\u200d\U0001f3eb"
WOMAN_TEACHER_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f3eb"
WOMAN_TEACHER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f3eb"
WOMAN_TEACHER_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f3eb"
WOMAN_TEACHER_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f3eb"
WOMAN_TEACHER_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f3eb"
JUDGE = "\U0001f9d1\u200d\u2696\ufe0f"
JUDGE_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\u2696\ufe0f"
JUDGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\u2696\ufe0f"
JUDGE_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\u2696\ufe0f"
JUDGE_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\u2696\ufe0f"
JUDGE_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\u2696\ufe0f"
MAN_JUDGE = "\U0001f468\u200d\u2696\ufe0f"
MAN_JUDGE_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\u2696\ufe0f"
MAN_JUDGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\u2696\ufe0f"
MAN_JUDGE_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\u2696\ufe0f"
MAN_JUDGE_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\u2696\ufe0f"
MAN_JUDGE_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\u2696\ufe0f"
WOMAN_JUDGE = "\U0001f469\u200d\u2696\ufe0f"
WOMAN_JUDGE_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\u2696\ufe0f"
WOMAN_JUDGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\u2696\ufe0f"
WOMAN_JUDGE_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\u2696\ufe0f"
WOMAN_JUDGE_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\u2696\ufe0f"
WOMAN_JUDGE_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\u2696\ufe0f"
FARMER = "\U0001f9d1\u200d\U0001f33e"
FARMER_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f33e"
FARMER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f33e"
FARMER_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f33e"
FARMER_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f33e"
FARMER_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f33e"
MAN_FARMER = "\U0001f468\u200d\U0001f33e"
MAN_FARMER_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f33e"
MAN_FARMER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f33e"
MAN_FARMER_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f33e"
MAN_FARMER_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f33e"
MAN_FARMER_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f33e"
WOMAN_FARMER = "\U0001f469\u200d\U0001f33e"
WOMAN_FARMER_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f33e"
WOMAN_FARMER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f33e"
WOMAN_FARMER_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f33e"
WOMAN_FARMER_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f33e"
WOMAN_FARMER_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f33e"
COOK = "\U0001f9d1\u200d\U0001f373"
COOK_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f373"
COOK_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f373"
COOK_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f373"
COOK_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f373"
COOK_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f373"
MAN_COOK = "\U0001f468\u200d\U0001f373"
MAN_COOK_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f373"
MAN_COOK_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f373"
MAN_COOK_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f373"
MAN_COOK_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f373"
MAN_COOK_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f373"
WOMAN_COOK = "\U0001f469\u200d\U0001f373"
WOMAN_COOK_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f373"
WOMAN_COOK_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f373"
WOMAN_COOK_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f373"
WOMAN_COOK_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f373"
WOMAN_COOK_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f373"
MECHANIC = "\U0001f9d1\u200d\U0001f527"
MECHANIC_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f527"
MECHANIC_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f527"
MECHANIC_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f527"
MECHANIC_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f527"
MECHANIC_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f527"
MAN_MECHANIC = "\U0001f468\u200d\U0001f527"
MAN_MECHANIC_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f527"
MAN_MECHANIC_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f527"
MAN_MECHANIC_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f527"
MAN_MECHANIC_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f527"
MAN_MECHANIC_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f527"
WOMAN_MECHANIC = "\U0001f469\u200d\U0001f527"
WOMAN_MECHANIC_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f527"
WOMAN_MECHANIC_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f527"
WOMAN_MECHANIC_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f527"
WOMAN_MECHANIC_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f527"
WOMAN_MECHANIC_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f527"
FACTORY_WORKER = "\U0001f9d1\u200d\U0001f3ed"
FACTORY_WORKER_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f3ed"
FACTORY_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f3ed"
FACTORY_WORKER_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f3ed"
FACTORY_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f3ed"
FACTORY_WORKER_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f3ed"
MAN_FACTORY_WORKER = "\U0001f468\u200d\U0001f3ed"
MAN_FACTORY_WORKER_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f3ed"
MAN_FACTORY_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f3ed"
MAN_FACTORY_WORKER_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f3ed"
MAN_FACTORY_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f3ed"
MAN_FACTORY_WORKER_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f3ed"
WOMAN_FACTORY_WORKER = "\U0001f469\u200d\U0001f3ed"
WOMAN_FACTORY_WORKER_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f3ed"
WOMAN_FACTORY_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f3ed"
WOMAN_FACTORY_WORKER_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f3ed"
WOMAN_FACTORY_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f3ed"
WOMAN_FACTORY_WORKER_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f3ed"
OFFICE_WORKER = "\U0001f9d1\u200d\U0001f4bc"
OFFICE_WORKER_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f4bc"
OFFICE_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f4bc"
OFFICE_WORKER_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f4bc"
OFFICE_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f4bc"
OFFICE_WORKER_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f4bc"
MAN_OFFICE_WORKER = "\U0001f468\u200d\U0001f4bc"
MAN_OFFICE_WORKER_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f4bc"
MAN_OFFICE_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f4bc"
MAN_OFFICE_WORKER_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f4bc"
MAN_OFFICE_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f4bc"
MAN_OFFICE_WORKER_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f4bc"
WOMAN_OFFICE_WORKER = "\U0001f469\u200d\U0001f4bc"
WOMAN_OFFICE_WORKER_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f4bc"
WOMAN_OFFICE_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f4bc"
WOMAN_OFFICE_WORKER_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f4bc"
WOMAN_OFFICE_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f4bc"
WOMAN_OFFICE_WORKER_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f4bc"
SCIENTIST = "\U0001f9d1\u200d\U0001f52c"
SCIENTIST_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f52c"
SCIENTIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f52c"
SCIENTIST_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f52c"
SCIENTIST_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f52c"
SCIENTIST_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f52c"
MAN_SCIENTIST = "\U0001f468\u200d\U0001f52c"
MAN_SCIENTIST_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f52c"
MAN_SCIENTIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f52c"
MAN_SCIENTIST_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f52c"
MAN_SCIENTIST_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f52c"
MAN_SCIENTIST_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f52c"
WOMAN_SCIENTIST = "\U0001f469\u200d\U0001f52c"
WOMAN_SCIENTIST_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f52c"
WOMAN_SCIENTIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f52c"
WOMAN_SCIENTIST_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f52c"
WOMAN_SCIENTIST_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f52c"
WOMAN_SCIENTIST_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f52c"
TECHNOLOGIST = "\U0001f9d1\u200d\U0001f4bb"
TECHNOLOGIST_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f4bb"
TECHNOLOGIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f4bb"
TECHNOLOGIST_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f4bb"
TECHNOLOGIST_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f4bb"
TECHNOLOGIST_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f4bb"
MAN_TECHNOLOGIST = "\U0001f468\u200d\U0001f4bb"
MAN_TECHNOLOGIST_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f4bb"
MAN_TECHNOLOGIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f4bb"
MAN_TECHNOLOGIST_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f4bb"
MAN_TECHNOLOGIST_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f4bb"
MAN_TECHNOLOGIST_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f4bb"
WOMAN_TECHNOLOGIST = "\U0001f469\u200d\U0001f4bb"
WOMAN_TECHNOLOGIST_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f4bb"
WOMAN_TECHNOLOGIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f4bb"
WOMAN_TECHNOLOGIST_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f4bb"
WOMAN_TECHNOLOGIST_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f4bb"
WOMAN_TECHNOLOGIST_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f4bb"
SINGER = "\U0001f9d1\u200d\U0001f3a4"
SINGER_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f3a4"
SINGER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f3a4"
SINGER_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f3a4"
SINGER_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f3a4"
SINGER_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f3a4"
MAN_SINGER = "\U0001f468\u200d\U0001f3a4"
MAN_SINGER_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f3a4"
MAN_SINGER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f3a4"
MAN_SINGER_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f3a4"
MAN_SINGER_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f3a4"
MAN_SINGER_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f3a4"
WOMAN_SINGER = "\U0001f469\u200d\U0001f3a4"
WOMAN_SINGER_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f3a4"
WOMAN_SINGER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f3a4"
WOMAN_SINGER_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f3a4"
WOMAN_SINGER_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f3a4"
WOMAN_SINGER_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f3a4"
ARTIST = "\U0001f9d1\u200d\U0001f3a8"
ARTIST_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f3a8"
ARTIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f3a8"
ARTIST_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f3a8"
ARTIST_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f3a8"
ARTIST_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f3a8"
MAN_ARTIST = "\U0001f468\u200d\U0001f3a8"
MAN_ARTIST_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f3a8"
MAN_ARTIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f3a8"
MAN_ARTIST_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f3a8"
MAN_ARTIST_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f3a8"
MAN_ARTIST_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f3a8"
WOMAN_ARTIST = "\U0001f469\u200d\U0001f3a8"
WOMAN_ARTIST_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f3a8"
WOMAN_ARTIST_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f3a8"
WOMAN_ARTIST_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f3a8"
WOMAN_ARTIST_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f3a8"
WOMAN_ARTIST_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f3a8"
PILOT = "\U0001f9d1\u200d\u2708\ufe0f"
PILOT_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\u2708\ufe0f"
PILOT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\u2708\ufe0f"
PILOT_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\u2708\ufe0f"
PILOT_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\u2708\ufe0f"
PILOT_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\u2708\ufe0f"
MAN_PILOT = "\U0001f468\u200d\u2708\ufe0f"
MAN_PILOT_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\u2708\ufe0f"
MAN_PILOT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\u2708\ufe0f"
MAN_PILOT_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\u2708\ufe0f"
MAN_PILOT_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\u2708\ufe0f"
MAN_PILOT_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\u2708\ufe0f"
WOMAN_PILOT = "\U0001f469\u200d\u2708\ufe0f"
WOMAN_PILOT_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\u2708\ufe0f"
WOMAN_PILOT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\u2708\ufe0f"
WOMAN_PILOT_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\u2708\ufe0f"
WOMAN_PILOT_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\u2708\ufe0f"
WOMAN_PILOT_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\u2708\ufe0f"
ASTRONAUT = "\U0001f9d1\u200d\U0001f680"
ASTRONAUT_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f680"
ASTRONAUT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f680"
ASTRONAUT_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f680"
ASTRONAUT_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f680"
ASTRONAUT_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f680"
MAN_ASTRONAUT = "\U0001f468\u200d\U0001f680"
MAN_ASTRONAUT_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f680"
MAN_ASTRONAUT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f680"
MAN_ASTRONAUT_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f680"
MAN_ASTRONAUT_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f680"
MAN_ASTRONAUT_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f680"
WOMAN_ASTRONAUT = "\U0001f469\u200d\U0001f680"
WOMAN_ASTRONAUT_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f680"
WOMAN_ASTRONAUT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f680"
WOMAN_ASTRONAUT_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f680"
WOMAN_ASTRONAUT_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f680"
WOMAN_ASTRONAUT_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f680"
FIREFIGHTER = "\U0001f9d1\u200d\U0001f692"
FIREFIGHTER_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f692"
FIREFIGHTER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f692"
FIREFIGHTER_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f692"
FIREFIGHTER_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f692"
FIREFIGHTER_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f692"
MAN_FIREFIGHTER = "\U0001f468\u200d\U0001f692"
MAN_FIREFIGHTER_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f692"
MAN_FIREFIGHTER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f692"
MAN_FIREFIGHTER_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f692"
MAN_FIREFIGHTER_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f692"
MAN_FIREFIGHTER_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f692"
WOMAN_FIREFIGHTER = "\U0001f469\u200d\U0001f692"
WOMAN_FIREFIGHTER_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f692"
WOMAN_FIREFIGHTER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f692"
WOMAN_FIREFIGHTER_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f692"
WOMAN_FIREFIGHTER_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f692"
WOMAN_FIREFIGHTER_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f692"
POLICE_OFFICER = "\U0001f46e"
POLICE_OFFICER_LIGHT_SKIN_TONE = "\U0001f46e\U0001f3fb"
POLICE_OFFICER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f46e\U0001f3fc"
POLICE_OFFICER_MEDIUM_SKIN_TONE = "\U0001f46e\U0001f3fd"
POLICE_OFFICER_MEDIUM_DARK_SKIN_TONE = "\U0001f46e\U0001f3fe"
POLICE_OFFICER_DARK_SKIN_TONE = "\U0001f46e\U0001f3ff"
MAN_POLICE_OFFICER = "\U0001f46e\u200d\u2642\ufe0f"
MAN_POLICE_OFFICER_LIGHT_SKIN_TONE = "\U0001f46e\U0001f3fb\u200d\u2642\ufe0f"
MAN_POLICE_OFFICER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f46e\U0001f3fc\u200d\u2642\ufe0f"
MAN_POLICE_OFFICER_MEDIUM_SKIN_TONE = "\U0001f46e\U0001f3fd\u200d\u2642\ufe0f"
MAN_POLICE_OFFICER_MEDIUM_DARK_SKIN_TONE = "\U0001f46e\U0001f3fe\u200d\u2642\ufe0f"
MAN_POLICE_OFFICER_DARK_SKIN_TONE = "\U0001f46e\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_POLICE_OFFICER = "\U0001f46e\u200d\u2640\ufe0f"
WOMAN_POLICE_OFFICER_LIGHT_SKIN_TONE = "\U0001f46e\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_POLICE_OFFICER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f46e\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_POLICE_OFFICER_MEDIUM_SKIN_TONE = "\U0001f46e\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_POLICE_OFFICER_MEDIUM_DARK_SKIN_TONE = "\U0001f46e\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_POLICE_OFFICER_DARK_SKIN_TONE = "\U0001f46e\U0001f3ff\u200d\u2640\ufe0f"
DETECTIVE = "\U0001f575\ufe0f"
DETECTIVE_LIGHT_SKIN_TONE = "\U0001f575\U0001f3fb"
DETECTIVE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f575\U0001f3fc"
DETECTIVE_MEDIUM_SKIN_TONE = "\U0001f575\U0001f3fd"
DETECTIVE_MEDIUM_DARK_SKIN_TONE = "\U0001f575\U0001f3fe"
DETECTIVE_DARK_SKIN_TONE = "\U0001f575\U0001f3ff"
MAN_DETECTIVE = "\U0001f575\ufe0f\u200d\u2642\ufe0f"
MAN_DETECTIVE_LIGHT_SKIN_TONE = "\U0001f575\U0001f3fb\u200d\u2642\ufe0f"
MAN_DETECTIVE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f575\U0001f3fc\u200d\u2642\ufe0f"
MAN_DETECTIVE_MEDIUM_SKIN_TONE = "\U0001f575\U0001f3fd\u200d\u2642\ufe0f"
MAN_DETECTIVE_MEDIUM_DARK_SKIN_TONE = "\U0001f575\U0001f3fe\u200d\u2642\ufe0f"
MAN_DETECTIVE_DARK_SKIN_TONE = "\U0001f575\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_DETECTIVE = "\U0001f575\ufe0f\u200d\u2640\ufe0f"
WOMAN_DETECTIVE_LIGHT_SKIN_TONE = "\U0001f575\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_DETECTIVE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f575\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_DETECTIVE_MEDIUM_SKIN_TONE = "\U0001f575\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_DETECTIVE_MEDIUM_DARK_SKIN_TONE = "\U0001f575\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_DETECTIVE_DARK_SKIN_TONE = "\U0001f575\U0001f3ff\u200d\u2640\ufe0f"
GUARD = "\U0001f482"
GUARD_LIGHT_SKIN_TONE = "\U0001f482\U0001f3fb"
GUARD_MEDIUM_LIGHT_SKIN_TONE = "\U0001f482\U0001f3fc"
GUARD_MEDIUM_SKIN_TONE = "\U0001f482\U0001f3fd"
GUARD_MEDIUM_DARK_SKIN_TONE = "\U0001f482\U0001f3fe"
GUARD_DARK_SKIN_TONE = "\U0001f482\U0001f3ff"
MAN_GUARD = "\U0001f482\u200d\u2642\ufe0f"
MAN_GUARD_LIGHT_SKIN_TONE = "\U0001f482\U0001f3fb\u200d\u2642\ufe0f"
MAN_GUARD_MEDIUM_LIGHT_SKIN_TONE = "\U0001f482\U0001f3fc\u200d\u2642\ufe0f"
MAN_GUARD_MEDIUM_SKIN_TONE = "\U0001f482\U0001f3fd\u200d\u2642\ufe0f"
MAN_GUARD_MEDIUM_DARK_SKIN_TONE = "\U0001f482\U0001f3fe\u200d\u2642\ufe0f"
MAN_GUARD_DARK_SKIN_TONE = "\U0001f482\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_GUARD = "\U0001f482\u200d\u2640\ufe0f"
WOMAN_GUARD_LIGHT_SKIN_TONE = "\U0001f482\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_GUARD_MEDIUM_LIGHT_SKIN_TONE = "\U0001f482\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_GUARD_MEDIUM_SKIN_TONE = "\U0001f482\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_GUARD_MEDIUM_DARK_SKIN_TONE = "\U0001f482\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_GUARD_DARK_SKIN_TONE = "\U0001f482\U0001f3ff\u200d\u2640\ufe0f"
NINJA = "\U0001f977"
NINJA_LIGHT_SKIN_TONE = "\U0001f977\U0001f3fb"
NINJA_MEDIUM_LIGHT_SKIN_TONE = "\U0001f977\U0001f3fc"
NINJA_MEDIUM_SKIN_TONE = "\U0001f977\U0001f3fd"
NINJA_MEDIUM_DARK_SKIN_TONE = "\U0001f977\U0001f3fe"
NINJA_DARK_SKIN_TONE = "\U0001f977\U0001f3ff"
CONSTRUCTION_WORKER = "\U0001f477"
CONSTRUCTION_WORKER_LIGHT_SKIN_TONE = "\U0001f477\U0001f3fb"
CONSTRUCTION_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f477\U0001f3fc"
CONSTRUCTION_WORKER_MEDIUM_SKIN_TONE = "\U0001f477\U0001f3fd"
CONSTRUCTION_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f477\U0001f3fe"
CONSTRUCTION_WORKER_DARK_SKIN_TONE = "\U0001f477\U0001f3ff"
MAN_CONSTRUCTION_WORKER = "\U0001f477\u200d\u2642\ufe0f"
MAN_CONSTRUCTION_WORKER_LIGHT_SKIN_TONE = "\U0001f477\U0001f3fb\u200d\u2642\ufe0f"
MAN_CONSTRUCTION_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f477\U0001f3fc\u200d\u2642\ufe0f"
MAN_CONSTRUCTION_WORKER_MEDIUM_SKIN_TONE = "\U0001f477\U0001f3fd\u200d\u2642\ufe0f"
MAN_CONSTRUCTION_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f477\U0001f3fe\u200d\u2642\ufe0f"
MAN_CONSTRUCTION_WORKER_DARK_SKIN_TONE = "\U0001f477\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_CONSTRUCTION_WORKER = "\U0001f477\u200d\u2640\ufe0f"
WOMAN_CONSTRUCTION_WORKER_LIGHT_SKIN_TONE = "\U0001f477\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_CONSTRUCTION_WORKER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f477\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_CONSTRUCTION_WORKER_MEDIUM_SKIN_TONE = "\U0001f477\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_CONSTRUCTION_WORKER_MEDIUM_DARK_SKIN_TONE = "\U0001f477\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_CONSTRUCTION_WORKER_DARK_SKIN_TONE = "\U0001f477\U0001f3ff\u200d\u2640\ufe0f"
PERSON_WITH_CROWN = "\U0001fac5"
PERSON_WITH_CROWN_LIGHT_SKIN_TONE = "\U0001fac5\U0001f3fb"
PERSON_WITH_CROWN_MEDIUM_LIGHT_SKIN_TONE = "\U0001fac5\U0001f3fc"
PERSON_WITH_CROWN_MEDIUM_SKIN_TONE = "\U0001fac5\U0001f3fd"
PERSON_WITH_CROWN_MEDIUM_DARK_SKIN_TONE = "\U0001fac5\U0001f3fe"
PERSON_WITH_CROWN_DARK_SKIN_TONE = "\U0001fac5\U0001f3ff"
PRINCE = "\U0001f934"
PRINCE_LIGHT_SKIN_TONE = "\U0001f934\U0001f3fb"
PRINCE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f934\U0001f3fc"
PRINCE_MEDIUM_SKIN_TONE = "\U0001f934\U0001f3fd"
PRINCE_MEDIUM_DARK_SKIN_TONE = "\U0001f934\U0001f3fe"
PRINCE_DARK_SKIN_TONE = "\U0001f934\U0001f3ff"
PRINCESS = "\U0001f478"
PRINCESS_LIGHT_SKIN_TONE = "\U0001f478\U0001f3fb"
PRINCESS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f478\U0001f3fc"
PRINCESS_MEDIUM_SKIN_TONE = "\U0001f478\U0001f3fd"
PRINCESS_MEDIUM_DARK_SKIN_TONE = "\U0001f478\U0001f3fe"
PRINCESS_DARK_SKIN_TONE = "\U0001f478\U0001f3ff"
PERSON_WEARING_TURBAN = "\U0001f473"
PERSON_WEARING_TURBAN_LIGHT_SKIN_TONE = "\U0001f473\U0001f3fb"
PERSON_WEARING_TURBAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f473\U0001f3fc"
PERSON_WEARING_TURBAN_MEDIUM_SKIN_TONE = "\U0001f473\U0001f3fd"
PERSON_WEARING_TURBAN_MEDIUM_DARK_SKIN_TONE = "\U0001f473\U0001f3fe"
PERSON_WEARING_TURBAN_DARK_SKIN_TONE = "\U0001f473\U0001f3ff"
MAN_WEARING_TURBAN = "\U0001f473\u200d\u2642\ufe0f"
MAN_WEARING_TURBAN_LIGHT_SKIN_TONE = "\U0001f473\U0001f3fb\u200d\u2642\ufe0f"
MAN_WEARING_TURBAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f473\U0001f3fc\u200d\u2642\ufe0f"
MAN_WEARING_TURBAN_MEDIUM_SKIN_TONE = "\U0001f473\U0001f3fd\u200d\u2642\ufe0f"
MAN_WEARING_TURBAN_MEDIUM_DARK_SKIN_TONE = "\U0001f473\U0001f3fe\u200d\u2642\ufe0f"
MAN_WEARING_TURBAN_DARK_SKIN_TONE = "\U0001f473\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_WEARING_TURBAN = "\U0001f473\u200d\u2640\ufe0f"
WOMAN_WEARING_TURBAN_LIGHT_SKIN_TONE = "\U0001f473\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_WEARING_TURBAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f473\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_WEARING_TURBAN_MEDIUM_SKIN_TONE = "\U0001f473\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_WEARING_TURBAN_MEDIUM_DARK_SKIN_TONE = "\U0001f473\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_WEARING_TURBAN_DARK_SKIN_TONE = "\U0001f473\U0001f3ff\u200d\u2640\ufe0f"
PERSON_WITH_SKULLCAP = "\U0001f472"
PERSON_WITH_SKULLCAP_LIGHT_SKIN_TONE = "\U0001f472\U0001f3fb"
PERSON_WITH_SKULLCAP_MEDIUM_LIGHT_SKIN_TONE = "\U0001f472\U0001f3fc"
PERSON_WITH_SKULLCAP_MEDIUM_SKIN_TONE = "\U0001f472\U0001f3fd"
PERSON_WITH_SKULLCAP_MEDIUM_DARK_SKIN_TONE = "\U0001f472\U0001f3fe"
PERSON_WITH_SKULLCAP_DARK_SKIN_TONE = "\U0001f472\U0001f3ff"
WOMAN_WITH_HEADSCARF = "\U0001f9d5"
WOMAN_WITH_HEADSCARF_LIGHT_SKIN_TONE = "\U0001f9d5\U0001f3fb"
WOMAN_WITH_HEADSCARF_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d5\U0001f3fc"
WOMAN_WITH_HEADSCARF_MEDIUM_SKIN_TONE = "\U0001f9d5\U0001f3fd"
WOMAN_WITH_HEADSCARF_MEDIUM_DARK_SKIN_TONE = "\U0001f9d5\U0001f3fe"
WOMAN_WITH_HEADSCARF_DARK_SKIN_TONE = "\U0001f9d5\U0001f3ff"
PERSON_IN_TUXEDO = "\U0001f935"
PERSON_IN_TUXEDO_LIGHT_SKIN_TONE = "\U0001f935\U0001f3fb"
PERSON_IN_TUXEDO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f935\U0001f3fc"
PERSON_IN_TUXEDO_MEDIUM_SKIN_TONE = "\U0001f935\U0001f3fd"
PERSON_IN_TUXEDO_MEDIUM_DARK_SKIN_TONE = "\U0001f935\U0001f3fe"
PERSON_IN_TUXEDO_DARK_SKIN_TONE = "\U0001f935\U0001f3ff"
MAN_IN_TUXEDO = "\U0001f935\u200d\u2642\ufe0f"
MAN_IN_TUXEDO_LIGHT_SKIN_TONE = "\U0001f935\U0001f3fb\u200d\u2642\ufe0f"
MAN_IN_TUXEDO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f935\U0001f3fc\u200d\u2642\ufe0f"
MAN_IN_TUXEDO_MEDIUM_SKIN_TONE = "\U0001f935\U0001f3fd\u200d\u2642\ufe0f"
MAN_IN_TUXEDO_MEDIUM_DARK_SKIN_TONE = "\U0001f935\U0001f3fe\u200d\u2642\ufe0f"
MAN_IN_TUXEDO_DARK_SKIN_TONE = "\U0001f935\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_IN_TUXEDO = "\U0001f935\u200d\u2640\ufe0f"
WOMAN_IN_TUXEDO_LIGHT_SKIN_TONE = "\U0001f935\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_IN_TUXEDO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f935\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_IN_TUXEDO_MEDIUM_SKIN_TONE = "\U0001f935\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_IN_TUXEDO_MEDIUM_DARK_SKIN_TONE = "\U0001f935\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_IN_TUXEDO_DARK_SKIN_TONE = "\U0001f935\U0001f3ff\u200d\u2640\ufe0f"
PERSON_WITH_VEIL = "\U0001f470"
PERSON_WITH_VEIL_LIGHT_SKIN_TONE = "\U0001f470\U0001f3fb"
PERSON_WITH_VEIL_MEDIUM_LIGHT_SKIN_TONE = "\U0001f470\U0001f3fc"
PERSON_WITH_VEIL_MEDIUM_SKIN_TONE = "\U0001f470\U0001f3fd"
PERSON_WITH_VEIL_MEDIUM_DARK_SKIN_TONE = "\U0001f470\U0001f3fe"
PERSON_WITH_VEIL_DARK_SKIN_TONE = "\U0001f470\U0001f3ff"
MAN_WITH_VEIL = "\U0001f470\u200d\u2642\ufe0f"
MAN_WITH_VEIL_LIGHT_SKIN_TONE = "\U0001f470\U0001f3fb\u200d\u2642\ufe0f"
MAN_WITH_VEIL_MEDIUM_LIGHT_SKIN_TONE = "\U0001f470\U0001f3fc\u200d\u2642\ufe0f"
MAN_WITH_VEIL_MEDIUM_SKIN_TONE = "\U0001f470\U0001f3fd\u200d\u2642\ufe0f"
MAN_WITH_VEIL_MEDIUM_DARK_SKIN_TONE = "\U0001f470\U0001f3fe\u200d\u2642\ufe0f"
MAN_WITH_VEIL_DARK_SKIN_TONE = "\U0001f470\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_WITH_VEIL = "\U0001f470\u200d\u2640\ufe0f"
WOMAN_WITH_VEIL_LIGHT_SKIN_TONE = "\U0001f470\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_WITH_VEIL_MEDIUM_LIGHT_SKIN_TONE = "\U0001f470\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_WITH_VEIL_MEDIUM_SKIN_TONE = "\U0001f470\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_WITH_VEIL_MEDIUM_DARK_SKIN_TONE = "\U0001f470\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_WITH_VEIL_DARK_SKIN_TONE = "\U0001f470\U0001f3ff\u200d\u2640\ufe0f"
PREGNANT_WOMAN = "\U0001f930"
PREGNANT_WOMAN_LIGHT_SKIN_TONE = "\U0001f930\U0001f3fb"
PREGNANT_WOMAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f930\U0001f3fc"
PREGNANT_WOMAN_MEDIUM_SKIN_TONE = "\U0001f930\U0001f3fd"
PREGNANT_WOMAN_MEDIUM_DARK_SKIN_TONE = "\U0001f930\U0001f3fe"
PREGNANT_WOMAN_DARK_SKIN_TONE = "\U0001f930\U0001f3ff"
PREGNANT_MAN = "\U0001fac3"
PREGNANT_MAN_LIGHT_SKIN_TONE = "\U0001fac3\U0001f3fb"
PREGNANT_MAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001fac3\U0001f3fc"
PREGNANT_MAN_MEDIUM_SKIN_TONE = "\U0001fac3\U0001f3fd"
PREGNANT_MAN_MEDIUM_DARK_SKIN_TONE = "\U0001fac3\U0001f3fe"
PREGNANT_MAN_DARK_SKIN_TONE = "\U0001fac3\U0001f3ff"
PREGNANT_PERSON = "\U0001fac4"
PREGNANT_PERSON_LIGHT_SKIN_TONE = "\U0001fac4\U0001f3fb"
PREGNANT_PERSON_MEDIUM_LIGHT_SKIN_TONE = "\U0001fac4\U0001f3fc"
PREGNANT_PERSON_MEDIUM_SKIN_TONE = "\U0001fac4\U0001f3fd"
PREGNANT_PERSON_MEDIUM_DARK_SKIN_TONE = "\U0001fac4\U0001f3fe"
PREGNANT_PERSON_DARK_SKIN_TONE = "\U0001fac4\U0001f3ff"
BREAST_FEEDING = "\U0001f931"
BREAST_FEEDING_LIGHT_SKIN_TONE = "\U0001f931\U0001f3fb"
BREAST_FEEDING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f931\U0001f3fc"
BREAST_FEEDING_MEDIUM_SKIN_TONE = "\U0001f931\U0001f3fd"
BREAST_FEEDING_MEDIUM_DARK_SKIN_TONE = "\U0001f931\U0001f3fe"
BREAST_FEEDING_DARK_SKIN_TONE = "\U0001f931\U0001f3ff"
WOMAN_FEEDING_BABY = "\U0001f469\u200d\U0001f37c"
WOMAN_FEEDING_BABY_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f37c"
WOMAN_FEEDING_BABY_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f37c"
WOMAN_FEEDING_BABY_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f37c"
WOMAN_FEEDING_BABY_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f37c"
WOMAN_FEEDING_BABY_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f37c"
MAN_FEEDING_BABY = "\U0001f468\u200d\U0001f37c"
MAN_FEEDING_BABY_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f37c"
MAN_FEEDING_BABY_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f37c"
MAN_FEEDING_BABY_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f37c"
MAN_FEEDING_BABY_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f37c"
MAN_FEEDING_BABY_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f37c"
PERSON_FEEDING_BABY = "\U0001f9d1\u200d\U0001f37c"
PERSON_FEEDING_BABY_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f37c"
PERSON_FEEDING_BABY_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f37c"
PERSON_FEEDING_BABY_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f37c"
PERSON_FEEDING_BABY_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f37c"
PERSON_FEEDING_BABY_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f37c"
BABY_ANGEL = "\U0001f47c"
BABY_ANGEL_LIGHT_SKIN_TONE = "\U0001f47c\U0001f3fb"
BABY_ANGEL_MEDIUM_LIGHT_SKIN_TONE = "\U0001f47c\U0001f3fc"
BABY_ANGEL_MEDIUM_SKIN_TONE = "\U0001f47c\U0001f3fd"
BABY_ANGEL_MEDIUM_DARK_SKIN_TONE = "\U0001f47c\U0001f3fe"
BABY_ANGEL_DARK_SKIN_TONE = "\U0001f47c\U0001f3ff"
SANTA_CLAUS = "\U0001f385"
SANTA_CLAUS_LIGHT_SKIN_TONE = "\U0001f385\U0001f3fb"
SANTA_CLAUS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f385\U0001f3fc"
SANTA_CLAUS_MEDIUM_SKIN_TONE = "\U0001f385\U0001f3fd"
SANTA_CLAUS_MEDIUM_DARK_SKIN_TONE = "\U0001f385\U0001f3fe"
SANTA_CLAUS_DARK_SKIN_TONE = "\U0001f385\U0001f3ff"
MRS_CLAUS = "\U0001f936"
MRS_CLAUS_LIGHT_SKIN_TONE = "\U0001f936\U0001f3fb"
MRS_CLAUS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f936\U0001f3fc"
MRS_CLAUS_MEDIUM_SKIN_TONE = "\U0001f936\U0001f3fd"
MRS_CLAUS_MEDIUM_DARK_SKIN_TONE = "\U0001f936\U0001f3fe"
MRS_CLAUS_DARK_SKIN_TONE = "\U0001f936\U0001f3ff"
MX_CLAUS = "\U0001f9d1\u200d\U0001f384"
MX_CLAUS_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f384"
MX_CLAUS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f384"
MX_CLAUS_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f384"
MX_CLAUS_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f384"
MX_CLAUS_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f384"
SUPERHERO = "\U0001f9b8"
SUPERHERO_LIGHT_SKIN_TONE = "\U0001f9b8\U0001f3fb"
SUPERHERO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9b8\U0001f3fc"
SUPERHERO_MEDIUM_SKIN_TONE = "\U0001f9b8\U0001f3fd"
SUPERHERO_MEDIUM_DARK_SKIN_TONE = "\U0001f9b8\U0001f3fe"
SUPERHERO_DARK_SKIN_TONE = "\U0001f9b8\U0001f3ff"
MAN_SUPERHERO = "\U0001f9b8\u200d\u2642\ufe0f"
MAN_SUPERHERO_LIGHT_SKIN_TONE = "\U0001f9b8\U0001f3fb\u200d\u2642\ufe0f"
MAN_SUPERHERO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9b8\U0001f3fc\u200d\u2642\ufe0f"
MAN_SUPERHERO_MEDIUM_SKIN_TONE = "\U0001f9b8\U0001f3fd\u200d\u2642\ufe0f"
MAN_SUPERHERO_MEDIUM_DARK_SKIN_TONE = "\U0001f9b8\U0001f3fe\u200d\u2642\ufe0f"
MAN_SUPERHERO_DARK_SKIN_TONE = "\U0001f9b8\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_SUPERHERO = "\U0001f9b8\u200d\u2640\ufe0f"
WOMAN_SUPERHERO_LIGHT_SKIN_TONE = "\U0001f9b8\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_SUPERHERO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9b8\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_SUPERHERO_MEDIUM_SKIN_TONE = "\U0001f9b8\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_SUPERHERO_MEDIUM_DARK_SKIN_TONE = "\U0001f9b8\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_SUPERHERO_DARK_SKIN_TONE = "\U0001f9b8\U0001f3ff\u200d\u2640\ufe0f"
SUPERVILLAIN = "\U0001f9b9"
SUPERVILLAIN_LIGHT_SKIN_TONE = "\U0001f9b9\U0001f3fb"
SUPERVILLAIN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9b9\U0001f3fc"
SUPERVILLAIN_MEDIUM_SKIN_TONE = "\U0001f9b9\U0001f3fd"
SUPERVILLAIN_MEDIUM_DARK_SKIN_TONE = "\U0001f9b9\U0001f3fe"
SUPERVILLAIN_DARK_SKIN_TONE = "\U0001f9b9\U0001f3ff"
MAN_SUPERVILLAIN = "\U0001f9b9\u200d\u2642\ufe0f"
MAN_SUPERVILLAIN_LIGHT_SKIN_TONE = "\U0001f9b9\U0001f3fb\u200d\u2642\ufe0f"
MAN_SUPERVILLAIN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9b9\U0001f3fc\u200d\u2642\ufe0f"
MAN_SUPERVILLAIN_MEDIUM_SKIN_TONE = "\U0001f9b9\U0001f3fd\u200d\u2642\ufe0f"
MAN_SUPERVILLAIN_MEDIUM_DARK_SKIN_TONE = "\U0001f9b9\U0001f3fe\u200d\u2642\ufe0f"
MAN_SUPERVILLAIN_DARK_SKIN_TONE = "\U0001f9b9\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_SUPERVILLAIN = "\U0001f9b9\u200d\u2640\ufe0f"
WOMAN_SUPERVILLAIN_LIGHT_SKIN_TONE = "\U0001f9b9\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_SUPERVILLAIN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9b9\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_SUPERVILLAIN_MEDIUM_SKIN_TONE = "\U0001f9b9\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_SUPERVILLAIN_MEDIUM_DARK_SKIN_TONE = "\U0001f9b9\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_SUPERVILLAIN_DARK_SKIN_TONE = "\U0001f9b9\U0001f3ff\u200d\u2640\ufe0f"
MAGE = "\U0001f9d9"
MAGE_LIGHT_SKIN_TONE = "\U0001f9d9\U0001f3fb"
MAGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d9\U0001f3fc"
MAGE_MEDIUM_SKIN_TONE = "\U0001f9d9\U0001f3fd"
MAGE_MEDIUM_DARK_SKIN_TONE = "\U0001f9d9\U0001f3fe"
MAGE_DARK_SKIN_TONE = "\U0001f9d9\U0001f3ff"
MAN_MAGE = "\U0001f9d9\u200d\u2642\ufe0f"
MAN_MAGE_LIGHT_SKIN_TONE = "\U0001f9d9\U0001f3fb\u200d\u2642\ufe0f"
MAN_MAGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d9\U0001f3fc\u200d\u2642\ufe0f"
MAN_MAGE_MEDIUM_SKIN_TONE = "\U0001f9d9\U0001f3fd\u200d\u2642\ufe0f"
MAN_MAGE_MEDIUM_DARK_SKIN_TONE = "\U0001f9d9\U0001f3fe\u200d\u2642\ufe0f"
MAN_MAGE_DARK_SKIN_TONE = "\U0001f9d9\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_MAGE = "\U0001f9d9\u200d\u2640\ufe0f"
WOMAN_MAGE_LIGHT_SKIN_TONE = "\U0001f9d9\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_MAGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d9\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_MAGE_MEDIUM_SKIN_TONE = "\U0001f9d9\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_MAGE_MEDIUM_DARK_SKIN_TONE = "\U0001f9d9\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_MAGE_DARK_SKIN_TONE = "\U0001f9d9\U0001f3ff\u200d\u2640\ufe0f"
FAIRY = "\U0001f9da"
FAIRY_LIGHT_SKIN_TONE = "\U0001f9da\U0001f3fb"
FAIRY_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9da\U0001f3fc"
FAIRY_MEDIUM_SKIN_TONE = "\U0001f9da\U0001f3fd"
FAIRY_MEDIUM_DARK_SKIN_TONE = "\U0001f9da\U0001f3fe"
FAIRY_DARK_SKIN_TONE = "\U0001f9da\U0001f3ff"
MAN_FAIRY = "\U0001f9da\u200d\u2642\ufe0f"
MAN_FAIRY_LIGHT_SKIN_TONE = "\U0001f9da\U0001f3fb\u200d\u2642\ufe0f"
MAN_FAIRY_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9da\U0001f3fc\u200d\u2642\ufe0f"
MAN_FAIRY_MEDIUM_SKIN_TONE = "\U0001f9da\U0001f3fd\u200d\u2642\ufe0f"
MAN_FAIRY_MEDIUM_DARK_SKIN_TONE = "\U0001f9da\U0001f3fe\u200d\u2642\ufe0f"
MAN_FAIRY_DARK_SKIN_TONE = "\U0001f9da\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_FAIRY = "\U0001f9da\u200d\u2640\ufe0f"
WOMAN_FAIRY_LIGHT_SKIN_TONE = "\U0001f9da\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_FAIRY_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9da\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_FAIRY_MEDIUM_SKIN_TONE = "\U0001f9da\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_FAIRY_MEDIUM_DARK_SKIN_TONE = "\U0001f9da\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_FAIRY_DARK_SKIN_TONE = "\U0001f9da\U0001f3ff\u200d\u2640\ufe0f"
VAMPIRE = "\U0001f9db"
VAMPIRE_LIGHT_SKIN_TONE = "\U0001f9db\U0001f3fb"
VAMPIRE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9db\U0001f3fc"
VAMPIRE_MEDIUM_SKIN_TONE = "\U0001f9db\U0001f3fd"
VAMPIRE_MEDIUM_DARK_SKIN_TONE = "\U0001f9db\U0001f3fe"
VAMPIRE_DARK_SKIN_TONE = "\U0001f9db\U0001f3ff"
MAN_VAMPIRE = "\U0001f9db\u200d\u2642\ufe0f"
MAN_VAMPIRE_LIGHT_SKIN_TONE = "\U0001f9db\U0001f3fb\u200d\u2642\ufe0f"
MAN_VAMPIRE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9db\U0001f3fc\u200d\u2642\ufe0f"
MAN_VAMPIRE_MEDIUM_SKIN_TONE = "\U0001f9db\U0001f3fd\u200d\u2642\ufe0f"
MAN_VAMPIRE_MEDIUM_DARK_SKIN_TONE = "\U0001f9db\U0001f3fe\u200d\u2642\ufe0f"
MAN_VAMPIRE_DARK_SKIN_TONE = "\U0001f9db\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_VAMPIRE = "\U0001f9db\u200d\u2640\ufe0f"
WOMAN_VAMPIRE_LIGHT_SKIN_TONE = "\U0001f9db\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_VAMPIRE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9db\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_VAMPIRE_MEDIUM_SKIN_TONE = "\U0001f9db\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_VAMPIRE_MEDIUM_DARK_SKIN_TONE = "\U0001f9db\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_VAMPIRE_DARK_SKIN_TONE = "\U0001f9db\U0001f3ff\u200d\u2640\ufe0f"
MERPERSON = "\U0001f9dc"
MERPERSON_LIGHT_SKIN_TONE = "\U0001f9dc\U0001f3fb"
MERPERSON_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9dc\U0001f3fc"
MERPERSON_MEDIUM_SKIN_TONE = "\U0001f9dc\U0001f3fd"
MERPERSON_MEDIUM_DARK_SKIN_TONE = "\U0001f9dc\U0001f3fe"
MERPERSON_DARK_SKIN_TONE = "\U0001f9dc\U0001f3ff"
MERMAN = "\U0001f9dc\u200d\u2642\ufe0f"
MERMAN_LIGHT_SKIN_TONE = "\U0001f9dc\U0001f3fb\u200d\u2642\ufe0f"
MERMAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9dc\U0001f3fc\u200d\u2642\ufe0f"
MERMAN_MEDIUM_SKIN_TONE = "\U0001f9dc\U0001f3fd\u200d\u2642\ufe0f"
MERMAN_MEDIUM_DARK_SKIN_TONE = "\U0001f9dc\U0001f3fe\u200d\u2642\ufe0f"
MERMAN_DARK_SKIN_TONE = "\U0001f9dc\U0001f3ff\u200d\u2642\ufe0f"
MERMAID = "\U0001f9dc\u200d\u2640\ufe0f"
MERMAID_LIGHT_SKIN_TONE = "\U0001f9dc\U0001f3fb\u200d\u2640\ufe0f"
MERMAID_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9dc\U0001f3fc\u200d\u2640\ufe0f"
MERMAID_MEDIUM_SKIN_TONE = "\U0001f9dc\U0001f3fd\u200d\u2640\ufe0f"
MERMAID_MEDIUM_DARK_SKIN_TONE = "\U0001f9dc\U0001f3fe\u200d\u2640\ufe0f"
MERMAID_DARK_SKIN_TONE = "\U0001f9dc\U0001f3ff\u200d\u2640\ufe0f"
ELF = "\U0001f9dd"
ELF_LIGHT_SKIN_TONE = "\U0001f9dd\U0001f3fb"
ELF_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9dd\U0001f3fc"
ELF_MEDIUM_SKIN_TONE = "\U0001f9dd\U0001f3fd"
ELF_MEDIUM_DARK_SKIN_TONE = "\U0001f9dd\U0001f3fe"
ELF_DARK_SKIN_TONE = "\U0001f9dd\U0001f3ff"
MAN_ELF = "\U0001f9dd\u200d\u2642\ufe0f"
MAN_ELF_LIGHT_SKIN_TONE = "\U0001f9dd\U0001f3fb\u200d\u2642\ufe0f"
MAN_ELF_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9dd\U0001f3fc\u200d\u2642\ufe0f"
MAN_ELF_MEDIUM_SKIN_TONE = "\U0001f9dd\U0001f3fd\u200d\u2642\ufe0f"
MAN_ELF_MEDIUM_DARK_SKIN_TONE = "\U0001f9dd\U0001f3fe\u200d\u2642\ufe0f"
MAN_ELF_DARK_SKIN_TONE = "\U0001f9dd\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_ELF = "\U0001f9dd\u200d\u2640\ufe0f"
WOMAN_ELF_LIGHT_SKIN_TONE = "\U0001f9dd\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_ELF_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9dd\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_ELF_MEDIUM_SKIN_TONE = "\U0001f9dd\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_ELF_MEDIUM_DARK_SKIN_TONE = "\U0001f9dd\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_ELF_DARK_SKIN_TONE = "\U0001f9dd\U0001f3ff\u200d\u2640\ufe0f"
GENIE = "\U0001f9de"
MAN_GENIE = "\U0001f9de\u200d\u2642\ufe0f"
WOMAN_GENIE = "\U0001f9de\u200d\u2640\ufe0f"
ZOMBIE = "\U0001f9df"
MAN_ZOMBIE = "\U0001f9df\u200d\u2642\ufe0f"
WOMAN_ZOMBIE = "\U0001f9df\u200d\u2640\ufe0f"
TROLL = "\U0001f9cc"
PERSON_GETTING_MASSAGE = "\U0001f486"
PERSON_GETTING_MASSAGE_LIGHT_SKIN_TONE = "\U0001f486\U0001f3fb"
PERSON_GETTING_MASSAGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f486\U0001f3fc"
PERSON_GETTING_MASSAGE_MEDIUM_SKIN_TONE = "\U0001f486\U0001f3fd"
PERSON_GETTING_MASSAGE_MEDIUM_DARK_SKIN_TONE = "\U0001f486\U0001f3fe"
PERSON_GETTING_MASSAGE_DARK_SKIN_TONE = "\U0001f486\U0001f3ff"
MAN_GETTING_MASSAGE = "\U0001f486\u200d\u2642\ufe0f"
MAN_GETTING_MASSAGE_LIGHT_SKIN_TONE = "\U0001f486\U0001f3fb\u200d\u2642\ufe0f"
MAN_GETTING_MASSAGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f486\U0001f3fc\u200d\u2642\ufe0f"
MAN_GETTING_MASSAGE_MEDIUM_SKIN_TONE = "\U0001f486\U0001f3fd\u200d\u2642\ufe0f"
MAN_GETTING_MASSAGE_MEDIUM_DARK_SKIN_TONE = "\U0001f486\U0001f3fe\u200d\u2642\ufe0f"
MAN_GETTING_MASSAGE_DARK_SKIN_TONE = "\U0001f486\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_GETTING_MASSAGE = "\U0001f486\u200d\u2640\ufe0f"
WOMAN_GETTING_MASSAGE_LIGHT_SKIN_TONE = "\U0001f486\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_GETTING_MASSAGE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f486\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_GETTING_MASSAGE_MEDIUM_SKIN_TONE = "\U0001f486\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_GETTING_MASSAGE_MEDIUM_DARK_SKIN_TONE = "\U0001f486\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_GETTING_MASSAGE_DARK_SKIN_TONE = "\U0001f486\U0001f3ff\u200d\u2640\ufe0f"
PERSON_GETTING_HAIRCUT = "\U0001f487"
PERSON_GETTING_HAIRCUT_LIGHT_SKIN_TONE = "\U0001f487\U0001f3fb"
PERSON_GETTING_HAIRCUT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f487\U0001f3fc"
PERSON_GETTING_HAIRCUT_MEDIUM_SKIN_TONE = "\U0001f487\U0001f3fd"
PERSON_GETTING_HAIRCUT_MEDIUM_DARK_SKIN_TONE = "\U0001f487\U0001f3fe"
PERSON_GETTING_HAIRCUT_DARK_SKIN_TONE = "\U0001f487\U0001f3ff"
MAN_GETTING_HAIRCUT = "\U0001f487\u200d\u2642\ufe0f"
MAN_GETTING_HAIRCUT_LIGHT_SKIN_TONE = "\U0001f487\U0001f3fb\u200d\u2642\ufe0f"
MAN_GETTING_HAIRCUT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f487\U0001f3fc\u200d\u2642\ufe0f"
MAN_GETTING_HAIRCUT_MEDIUM_SKIN_TONE = "\U0001f487\U0001f3fd\u200d\u2642\ufe0f"
MAN_GETTING_HAIRCUT_MEDIUM_DARK_SKIN_TONE = "\U0001f487\U0001f3fe\u200d\u2642\ufe0f"
MAN_GETTING_HAIRCUT_DARK_SKIN_TONE = "\U0001f487\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_GETTING_HAIRCUT = "\U0001f487\u200d\u2640\ufe0f"
WOMAN_GETTING_HAIRCUT_LIGHT_SKIN_TONE = "\U0001f487\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_GETTING_HAIRCUT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f487\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_GETTING_HAIRCUT_MEDIUM_SKIN_TONE = "\U0001f487\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_GETTING_HAIRCUT_MEDIUM_DARK_SKIN_TONE = "\U0001f487\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_GETTING_HAIRCUT_DARK_SKIN_TONE = "\U0001f487\U0001f3ff\u200d\u2640\ufe0f"
PERSON_WALKING = "\U0001f6b6"
PERSON_WALKING_LIGHT_SKIN_TONE = "\U0001f6b6\U0001f3fb"
PERSON_WALKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b6\U0001f3fc"
PERSON_WALKING_MEDIUM_SKIN_TONE = "\U0001f6b6\U0001f3fd"
PERSON_WALKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b6\U0001f3fe"
PERSON_WALKING_DARK_SKIN_TONE = "\U0001f6b6\U0001f3ff"
MAN_WALKING = "\U0001f6b6\u200d\u2642\ufe0f"
MAN_WALKING_LIGHT_SKIN_TONE = "\U0001f6b6\U0001f3fb\u200d\u2642\ufe0f"
MAN_WALKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b6\U0001f3fc\u200d\u2642\ufe0f"
MAN_WALKING_MEDIUM_SKIN_TONE = "\U0001f6b6\U0001f3fd\u200d\u2642\ufe0f"
MAN_WALKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b6\U0001f3fe\u200d\u2642\ufe0f"
MAN_WALKING_DARK_SKIN_TONE = "\U0001f6b6\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_WALKING = "\U0001f6b6\u200d\u2640\ufe0f"
WOMAN_WALKING_LIGHT_SKIN_TONE = "\U0001f6b6\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_WALKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b6\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_WALKING_MEDIUM_SKIN_TONE = "\U0001f6b6\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_WALKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b6\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_WALKING_DARK_SKIN_TONE = "\U0001f6b6\U0001f3ff\u200d\u2640\ufe0f"
PERSON_STANDING = "\U0001f9cd"
PERSON_STANDING_LIGHT_SKIN_TONE = "\U0001f9cd\U0001f3fb"
PERSON_STANDING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9cd\U0001f3fc"
PERSON_STANDING_MEDIUM_SKIN_TONE = "\U0001f9cd\U0001f3fd"
PERSON_STANDING_MEDIUM_DARK_SKIN_TONE = "\U0001f9cd\U0001f3fe"
PERSON_STANDING_DARK_SKIN_TONE = "\U0001f9cd\U0001f3ff"
MAN_STANDING = "\U0001f9cd\u200d\u2642\ufe0f"
MAN_STANDING_LIGHT_SKIN_TONE = "\U0001f9cd\U0001f3fb\u200d\u2642\ufe0f"
MAN_STANDING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9cd\U0001f3fc\u200d\u2642\ufe0f"
MAN_STANDING_MEDIUM_SKIN_TONE = "\U0001f9cd\U0001f3fd\u200d\u2642\ufe0f"
MAN_STANDING_MEDIUM_DARK_SKIN_TONE = "\U0001f9cd\U0001f3fe\u200d\u2642\ufe0f"
MAN_STANDING_DARK_SKIN_TONE = "\U0001f9cd\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_STANDING = "\U0001f9cd\u200d\u2640\ufe0f"
WOMAN_STANDING_LIGHT_SKIN_TONE = "\U0001f9cd\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_STANDING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9cd\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_STANDING_MEDIUM_SKIN_TONE = "\U0001f9cd\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_STANDING_MEDIUM_DARK_SKIN_TONE = "\U0001f9cd\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_STANDING_DARK_SKIN_TONE = "\U0001f9cd\U0001f3ff\u200d\u2640\ufe0f"
PERSON_KNEELING = "\U0001f9ce"
PERSON_KNEELING_LIGHT_SKIN_TONE = "\U0001f9ce\U0001f3fb"
PERSON_KNEELING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9ce\U0001f3fc"
PERSON_KNEELING_MEDIUM_SKIN_TONE = "\U0001f9ce\U0001f3fd"
PERSON_KNEELING_MEDIUM_DARK_SKIN_TONE = "\U0001f9ce\U0001f3fe"
PERSON_KNEELING_DARK_SKIN_TONE = "\U0001f9ce\U0001f3ff"
MAN_KNEELING = "\U0001f9ce\u200d\u2642\ufe0f"
MAN_KNEELING_LIGHT_SKIN_TONE = "\U0001f9ce\U0001f3fb\u200d\u2642\ufe0f"
MAN_KNEELING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9ce\U0001f3fc\u200d\u2642\ufe0f"
MAN_KNEELING_MEDIUM_SKIN_TONE = "\U0001f9ce\U0001f3fd\u200d\u2642\ufe0f"
MAN_KNEELING_MEDIUM_DARK_SKIN_TONE = "\U0001f9ce\U0001f3fe\u200d\u2642\ufe0f"
MAN_KNEELING_DARK_SKIN_TONE = "\U0001f9ce\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_KNEELING = "\U0001f9ce\u200d\u2640\ufe0f"
WOMAN_KNEELING_LIGHT_SKIN_TONE = "\U0001f9ce\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_KNEELING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9ce\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_KNEELING_MEDIUM_SKIN_TONE = "\U0001f9ce\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_KNEELING_MEDIUM_DARK_SKIN_TONE = "\U0001f9ce\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_KNEELING_DARK_SKIN_TONE = "\U0001f9ce\U0001f3ff\u200d\u2640\ufe0f"
PERSON_WITH_WHITE_CANE = "\U0001f9d1\u200d\U0001f9af"
PERSON_WITH_WHITE_CANE_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f9af"
PERSON_WITH_WHITE_CANE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f9af"
PERSON_WITH_WHITE_CANE_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f9af"
PERSON_WITH_WHITE_CANE_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f9af"
PERSON_WITH_WHITE_CANE_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f9af"
MAN_WITH_WHITE_CANE = "\U0001f468\u200d\U0001f9af"
MAN_WITH_WHITE_CANE_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f9af"
MAN_WITH_WHITE_CANE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f9af"
MAN_WITH_WHITE_CANE_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f9af"
MAN_WITH_WHITE_CANE_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f9af"
MAN_WITH_WHITE_CANE_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f9af"
WOMAN_WITH_WHITE_CANE = "\U0001f469\u200d\U0001f9af"
WOMAN_WITH_WHITE_CANE_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f9af"
WOMAN_WITH_WHITE_CANE_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f9af"
WOMAN_WITH_WHITE_CANE_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f9af"
WOMAN_WITH_WHITE_CANE_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f9af"
WOMAN_WITH_WHITE_CANE_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f9af"
PERSON_IN_MOTORIZED_WHEELCHAIR = "\U0001f9d1\u200d\U0001f9bc"
PERSON_IN_MOTORIZED_WHEELCHAIR_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f9bc"
PERSON_IN_MOTORIZED_WHEELCHAIR_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f9bc"
PERSON_IN_MOTORIZED_WHEELCHAIR_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f9bc"
PERSON_IN_MOTORIZED_WHEELCHAIR_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f9bc"
PERSON_IN_MOTORIZED_WHEELCHAIR_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f9bc"
MAN_IN_MOTORIZED_WHEELCHAIR = "\U0001f468\u200d\U0001f9bc"
MAN_IN_MOTORIZED_WHEELCHAIR_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f9bc"
MAN_IN_MOTORIZED_WHEELCHAIR_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f9bc"
MAN_IN_MOTORIZED_WHEELCHAIR_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f9bc"
MAN_IN_MOTORIZED_WHEELCHAIR_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f9bc"
MAN_IN_MOTORIZED_WHEELCHAIR_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f9bc"
WOMAN_IN_MOTORIZED_WHEELCHAIR = "\U0001f469\u200d\U0001f9bc"
WOMAN_IN_MOTORIZED_WHEELCHAIR_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f9bc"
WOMAN_IN_MOTORIZED_WHEELCHAIR_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f9bc"
WOMAN_IN_MOTORIZED_WHEELCHAIR_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f9bc"
WOMAN_IN_MOTORIZED_WHEELCHAIR_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f9bc"
WOMAN_IN_MOTORIZED_WHEELCHAIR_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f9bc"
PERSON_IN_MANUAL_WHEELCHAIR = "\U0001f9d1\u200d\U0001f9bd"
PERSON_IN_MANUAL_WHEELCHAIR_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f9bd"
PERSON_IN_MANUAL_WHEELCHAIR_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f9bd"
PERSON_IN_MANUAL_WHEELCHAIR_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f9bd"
PERSON_IN_MANUAL_WHEELCHAIR_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f9bd"
PERSON_IN_MANUAL_WHEELCHAIR_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f9bd"
MAN_IN_MANUAL_WHEELCHAIR = "\U0001f468\u200d\U0001f9bd"
MAN_IN_MANUAL_WHEELCHAIR_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f9bd"
MAN_IN_MANUAL_WHEELCHAIR_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\U0001f9bd"
MAN_IN_MANUAL_WHEELCHAIR_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f9bd"
MAN_IN_MANUAL_WHEELCHAIR_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\U0001f9bd"
MAN_IN_MANUAL_WHEELCHAIR_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f9bd"
WOMAN_IN_MANUAL_WHEELCHAIR = "\U0001f469\u200d\U0001f9bd"
WOMAN_IN_MANUAL_WHEELCHAIR_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f9bd"
WOMAN_IN_MANUAL_WHEELCHAIR_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\U0001f9bd"
WOMAN_IN_MANUAL_WHEELCHAIR_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f9bd"
WOMAN_IN_MANUAL_WHEELCHAIR_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\U0001f9bd"
WOMAN_IN_MANUAL_WHEELCHAIR_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f9bd"
PERSON_RUNNING = "\U0001f3c3"
PERSON_RUNNING_LIGHT_SKIN_TONE = "\U0001f3c3\U0001f3fb"
PERSON_RUNNING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3c3\U0001f3fc"
PERSON_RUNNING_MEDIUM_SKIN_TONE = "\U0001f3c3\U0001f3fd"
PERSON_RUNNING_MEDIUM_DARK_SKIN_TONE = "\U0001f3c3\U0001f3fe"
PERSON_RUNNING_DARK_SKIN_TONE = "\U0001f3c3\U0001f3ff"
MAN_RUNNING = "\U0001f3c3\u200d\u2642\ufe0f"
MAN_RUNNING_LIGHT_SKIN_TONE = "\U0001f3c3\U0001f3fb\u200d\u2642\ufe0f"
MAN_RUNNING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3c3\U0001f3fc\u200d\u2642\ufe0f"
MAN_RUNNING_MEDIUM_SKIN_TONE = "\U0001f3c3\U0001f3fd\u200d\u2642\ufe0f"
MAN_RUNNING_MEDIUM_DARK_SKIN_TONE = "\U0001f3c3\U0001f3fe\u200d\u2642\ufe0f"
MAN_RUNNING_DARK_SKIN_TONE = "\U0001f3c3\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_RUNNING = "\U0001f3c3\u200d\u2640\ufe0f"
WOMAN_RUNNING_LIGHT_SKIN_TONE = "\U0001f3c3\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_RUNNING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3c3\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_RUNNING_MEDIUM_SKIN_TONE = "\U0001f3c3\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_RUNNING_MEDIUM_DARK_SKIN_TONE = "\U0001f3c3\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_RUNNING_DARK_SKIN_TONE = "\U0001f3c3\U0001f3ff\u200d\u2640\ufe0f"
WOMAN_DANCING = "\U0001f483"
WOMAN_DANCING_LIGHT_SKIN_TONE = "\U0001f483\U0001f3fb"
WOMAN_DANCING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f483\U0001f3fc"
WOMAN_DANCING_MEDIUM_SKIN_TONE = "\U0001f483\U0001f3fd"
WOMAN_DANCING_MEDIUM_DARK_SKIN_TONE = "\U0001f483\U0001f3fe"
WOMAN_DANCING_DARK_SKIN_TONE = "\U0001f483\U0001f3ff"
MAN_DANCING = "\U0001f57a"
MAN_DANCING_LIGHT_SKIN_TONE = "\U0001f57a\U0001f3fb"
MAN_DANCING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f57a\U0001f3fc"
MAN_DANCING_MEDIUM_SKIN_TONE = "\U0001f57a\U0001f3fd"
MAN_DANCING_MEDIUM_DARK_SKIN_TONE = "\U0001f57a\U0001f3fe"
MAN_DANCING_DARK_SKIN_TONE = "\U0001f57a\U0001f3ff"
PERSON_IN_SUIT_LEVITATING = "\U0001f574\ufe0f"
PERSON_IN_SUIT_LEVITATING_LIGHT_SKIN_TONE = "\U0001f574\U0001f3fb"
PERSON_IN_SUIT_LEVITATING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f574\U0001f3fc"
PERSON_IN_SUIT_LEVITATING_MEDIUM_SKIN_TONE = "\U0001f574\U0001f3fd"
PERSON_IN_SUIT_LEVITATING_MEDIUM_DARK_SKIN_TONE = "\U0001f574\U0001f3fe"
PERSON_IN_SUIT_LEVITATING_DARK_SKIN_TONE = "\U0001f574\U0001f3ff"
PEOPLE_WITH_BUNNY_EARS = "\U0001f46f"
MEN_WITH_BUNNY_EARS = "\U0001f46f\u200d\u2642\ufe0f"
WOMEN_WITH_BUNNY_EARS = "\U0001f46f\u200d\u2640\ufe0f"
PERSON_IN_STEAMY_ROOM = "\U0001f9d6"
PERSON_IN_STEAMY_ROOM_LIGHT_SKIN_TONE = "\U0001f9d6\U0001f3fb"
PERSON_IN_STEAMY_ROOM_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d6\U0001f3fc"
PERSON_IN_STEAMY_ROOM_MEDIUM_SKIN_TONE = "\U0001f9d6\U0001f3fd"
PERSON_IN_STEAMY_ROOM_MEDIUM_DARK_SKIN_TONE = "\U0001f9d6\U0001f3fe"
PERSON_IN_STEAMY_ROOM_DARK_SKIN_TONE = "\U0001f9d6\U0001f3ff"
MAN_IN_STEAMY_ROOM = "\U0001f9d6\u200d\u2642\ufe0f"
MAN_IN_STEAMY_ROOM_LIGHT_SKIN_TONE = "\U0001f9d6\U0001f3fb\u200d\u2642\ufe0f"
MAN_IN_STEAMY_ROOM_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d6\U0001f3fc\u200d\u2642\ufe0f"
MAN_IN_STEAMY_ROOM_MEDIUM_SKIN_TONE = "\U0001f9d6\U0001f3fd\u200d\u2642\ufe0f"
MAN_IN_STEAMY_ROOM_MEDIUM_DARK_SKIN_TONE = "\U0001f9d6\U0001f3fe\u200d\u2642\ufe0f"
MAN_IN_STEAMY_ROOM_DARK_SKIN_TONE = "\U0001f9d6\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_IN_STEAMY_ROOM = "\U0001f9d6\u200d\u2640\ufe0f"
WOMAN_IN_STEAMY_ROOM_LIGHT_SKIN_TONE = "\U0001f9d6\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_IN_STEAMY_ROOM_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d6\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_IN_STEAMY_ROOM_MEDIUM_SKIN_TONE = "\U0001f9d6\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_IN_STEAMY_ROOM_MEDIUM_DARK_SKIN_TONE = "\U0001f9d6\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_IN_STEAMY_ROOM_DARK_SKIN_TONE = "\U0001f9d6\U0001f3ff\u200d\u2640\ufe0f"
PERSON_CLIMBING = "\U0001f9d7"
PERSON_CLIMBING_LIGHT_SKIN_TONE = "\U0001f9d7\U0001f3fb"
PERSON_CLIMBING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d7\U0001f3fc"
PERSON_CLIMBING_MEDIUM_SKIN_TONE = "\U0001f9d7\U0001f3fd"
PERSON_CLIMBING_MEDIUM_DARK_SKIN_TONE = "\U0001f9d7\U0001f3fe"
PERSON_CLIMBING_DARK_SKIN_TONE = "\U0001f9d7\U0001f3ff"
MAN_CLIMBING = "\U0001f9d7\u200d\u2642\ufe0f"
MAN_CLIMBING_LIGHT_SKIN_TONE = "\U0001f9d7\U0001f3fb\u200d\u2642\ufe0f"
MAN_CLIMBING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d7\U0001f3fc\u200d\u2642\ufe0f"
MAN_CLIMBING_MEDIUM_SKIN_TONE = "\U0001f9d7\U0001f3fd\u200d\u2642\ufe0f"
MAN_CLIMBING_MEDIUM_DARK_SKIN_TONE = "\U0001f9d7\U0001f3fe\u200d\u2642\ufe0f"
MAN_CLIMBING_DARK_SKIN_TONE = "\U0001f9d7\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_CLIMBING = "\U0001f9d7\u200d\u2640\ufe0f"
WOMAN_CLIMBING_LIGHT_SKIN_TONE = "\U0001f9d7\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_CLIMBING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d7\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_CLIMBING_MEDIUM_SKIN_TONE = "\U0001f9d7\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_CLIMBING_MEDIUM_DARK_SKIN_TONE = "\U0001f9d7\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_CLIMBING_DARK_SKIN_TONE = "\U0001f9d7\U0001f3ff\u200d\u2640\ufe0f"
PERSON_FENCING = "\U0001f93a"
HORSE_RACING = "\U0001f3c7"
HORSE_RACING_LIGHT_SKIN_TONE = "\U0001f3c7\U0001f3fb"
HORSE_RACING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3c7\U0001f3fc"
HORSE_RACING_MEDIUM_SKIN_TONE = "\U0001f3c7\U0001f3fd"
HORSE_RACING_MEDIUM_DARK_SKIN_TONE = "\U0001f3c7\U0001f3fe"
HORSE_RACING_DARK_SKIN_TONE = "\U0001f3c7\U0001f3ff"
SKIER = "\u26f7\ufe0f"
SNOWBOARDER = "\U0001f3c2"
SNOWBOARDER_LIGHT_SKIN_TONE = "\U0001f3c2\U0001f3fb"
SNOWBOARDER_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3c2\U0001f3fc"
SNOWBOARDER_MEDIUM_SKIN_TONE = "\U0001f3c2\U0001f3fd"
SNOWBOARDER_MEDIUM_DARK_SKIN_TONE = "\U0001f3c2\U0001f3fe"
SNOWBOARDER_DARK_SKIN_TONE = "\U0001f3c2\U0001f3ff"
PERSON_GOLFING = "\U0001f3cc\ufe0f"
PERSON_GOLFING_LIGHT_SKIN_TONE = "\U0001f3cc\U0001f3fb"
PERSON_GOLFING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3cc\U0001f3fc"
PERSON_GOLFING_MEDIUM_SKIN_TONE = "\U0001f3cc\U0001f3fd"
PERSON_GOLFING_MEDIUM_DARK_SKIN_TONE = "\U0001f3cc\U0001f3fe"
PERSON_GOLFING_DARK_SKIN_TONE = "\U0001f3cc\U0001f3ff"
MAN_GOLFING = "\U0001f3cc\ufe0f\u200d\u2642\ufe0f"
MAN_GOLFING_LIGHT_SKIN_TONE = "\U0001f3cc\U0001f3fb\u200d\u2642\ufe0f"
MAN_GOLFING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3cc\U0001f3fc\u200d\u2642\ufe0f"
MAN_GOLFING_MEDIUM_SKIN_TONE = "\U0001f3cc\U0001f3fd\u200d\u2642\ufe0f"
MAN_GOLFING_MEDIUM_DARK_SKIN_TONE = "\U0001f3cc\U0001f3fe\u200d\u2642\ufe0f"
MAN_GOLFING_DARK_SKIN_TONE = "\U0001f3cc\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_GOLFING = "\U0001f3cc\ufe0f\u200d\u2640\ufe0f"
WOMAN_GOLFING_LIGHT_SKIN_TONE = "\U0001f3cc\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_GOLFING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3cc\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_GOLFING_MEDIUM_SKIN_TONE = "\U0001f3cc\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_GOLFING_MEDIUM_DARK_SKIN_TONE = "\U0001f3cc\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_GOLFING_DARK_SKIN_TONE = "\U0001f3cc\U0001f3ff\u200d\u2640\ufe0f"
PERSON_SURFING = "\U0001f3c4"
PERSON_SURFING_LIGHT_SKIN_TONE = "\U0001f3c4\U0001f3fb"
PERSON_SURFING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3c4\U0001f3fc"
PERSON_SURFING_MEDIUM_SKIN_TONE = "\U0001f3c4\U0001f3fd"
PERSON_SURFING_MEDIUM_DARK_SKIN_TONE = "\U0001f3c4\U0001f3fe"
PERSON_SURFING_DARK_SKIN_TONE = "\U0001f3c4\U0001f3ff"
MAN_SURFING = "\U0001f3c4\u200d\u2642\ufe0f"
MAN_SURFING_LIGHT_SKIN_TONE = "\U0001f3c4\U0001f3fb\u200d\u2642\ufe0f"
MAN_SURFING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3c4\U0001f3fc\u200d\u2642\ufe0f"
MAN_SURFING_MEDIUM_SKIN_TONE = "\U0001f3c4\U0001f3fd\u200d\u2642\ufe0f"
MAN_SURFING_MEDIUM_DARK_SKIN_TONE = "\U0001f3c4\U0001f3fe\u200d\u2642\ufe0f"
MAN_SURFING_DARK_SKIN_TONE = "\U0001f3c4\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_SURFING = "\U0001f3c4\u200d\u2640\ufe0f"
WOMAN_SURFING_LIGHT_SKIN_TONE = "\U0001f3c4\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_SURFING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3c4\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_SURFING_MEDIUM_SKIN_TONE = "\U0001f3c4\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_SURFING_MEDIUM_DARK_SKIN_TONE = "\U0001f3c4\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_SURFING_DARK_SKIN_TONE = "\U0001f3c4\U0001f3ff\u200d\u2640\ufe0f"
PERSON_ROWING_BOAT = "\U0001f6a3"
PERSON_ROWING_BOAT_LIGHT_SKIN_TONE = "\U0001f6a3\U0001f3fb"
PERSON_ROWING_BOAT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6a3\U0001f3fc"
PERSON_ROWING_BOAT_MEDIUM_SKIN_TONE = "\U0001f6a3\U0001f3fd"
PERSON_ROWING_BOAT_MEDIUM_DARK_SKIN_TONE = "\U0001f6a3\U0001f3fe"
PERSON_ROWING_BOAT_DARK_SKIN_TONE = "\U0001f6a3\U0001f3ff"
MAN_ROWING_BOAT = "\U0001f6a3\u200d\u2642\ufe0f"
MAN_ROWING_BOAT_LIGHT_SKIN_TONE = "\U0001f6a3\U0001f3fb\u200d\u2642\ufe0f"
MAN_ROWING_BOAT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6a3\U0001f3fc\u200d\u2642\ufe0f"
MAN_ROWING_BOAT_MEDIUM_SKIN_TONE = "\U0001f6a3\U0001f3fd\u200d\u2642\ufe0f"
MAN_ROWING_BOAT_MEDIUM_DARK_SKIN_TONE = "\U0001f6a3\U0001f3fe\u200d\u2642\ufe0f"
MAN_ROWING_BOAT_DARK_SKIN_TONE = "\U0001f6a3\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_ROWING_BOAT = "\U0001f6a3\u200d\u2640\ufe0f"
WOMAN_ROWING_BOAT_LIGHT_SKIN_TONE = "\U0001f6a3\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_ROWING_BOAT_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6a3\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_ROWING_BOAT_MEDIUM_SKIN_TONE = "\U0001f6a3\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_ROWING_BOAT_MEDIUM_DARK_SKIN_TONE = "\U0001f6a3\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_ROWING_BOAT_DARK_SKIN_TONE = "\U0001f6a3\U0001f3ff\u200d\u2640\ufe0f"
PERSON_SWIMMING = "\U0001f3ca"
PERSON_SWIMMING_LIGHT_SKIN_TONE = "\U0001f3ca\U0001f3fb"
PERSON_SWIMMING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3ca\U0001f3fc"
PERSON_SWIMMING_MEDIUM_SKIN_TONE = "\U0001f3ca\U0001f3fd"
PERSON_SWIMMING_MEDIUM_DARK_SKIN_TONE = "\U0001f3ca\U0001f3fe"
PERSON_SWIMMING_DARK_SKIN_TONE = "\U0001f3ca\U0001f3ff"
MAN_SWIMMING = "\U0001f3ca\u200d\u2642\ufe0f"
MAN_SWIMMING_LIGHT_SKIN_TONE = "\U0001f3ca\U0001f3fb\u200d\u2642\ufe0f"
MAN_SWIMMING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3ca\U0001f3fc\u200d\u2642\ufe0f"
MAN_SWIMMING_MEDIUM_SKIN_TONE = "\U0001f3ca\U0001f3fd\u200d\u2642\ufe0f"
MAN_SWIMMING_MEDIUM_DARK_SKIN_TONE = "\U0001f3ca\U0001f3fe\u200d\u2642\ufe0f"
MAN_SWIMMING_DARK_SKIN_TONE = "\U0001f3ca\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_SWIMMING = "\U0001f3ca\u200d\u2640\ufe0f"
WOMAN_SWIMMING_LIGHT_SKIN_TONE = "\U0001f3ca\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_SWIMMING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3ca\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_SWIMMING_MEDIUM_SKIN_TONE = "\U0001f3ca\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_SWIMMING_MEDIUM_DARK_SKIN_TONE = "\U0001f3ca\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_SWIMMING_DARK_SKIN_TONE = "\U0001f3ca\U0001f3ff\u200d\u2640\ufe0f"
PERSON_BOUNCING_BALL = "\u26f9\ufe0f"
PERSON_BOUNCING_BALL_LIGHT_SKIN_TONE = "\u26f9\U0001f3fb"
PERSON_BOUNCING_BALL_MEDIUM_LIGHT_SKIN_TONE = "\u26f9\U0001f3fc"
PERSON_BOUNCING_BALL_MEDIUM_SKIN_TONE = "\u26f9\U0001f3fd"
PERSON_BOUNCING_BALL_MEDIUM_DARK_SKIN_TONE = "\u26f9\U0001f3fe"
PERSON_BOUNCING_BALL_DARK_SKIN_TONE = "\u26f9\U0001f3ff"
MAN_BOUNCING_BALL = "\u26f9\ufe0f\u200d\u2642\ufe0f"
MAN_BOUNCING_BALL_LIGHT_SKIN_TONE = "\u26f9\U0001f3fb\u200d\u2642\ufe0f"
MAN_BOUNCING_BALL_MEDIUM_LIGHT_SKIN_TONE = "\u26f9\U0001f3fc\u200d\u2642\ufe0f"
MAN_BOUNCING_BALL_MEDIUM_SKIN_TONE = "\u26f9\U0001f3fd\u200d\u2642\ufe0f"
MAN_BOUNCING_BALL_MEDIUM_DARK_SKIN_TONE = "\u26f9\U0001f3fe\u200d\u2642\ufe0f"
MAN_BOUNCING_BALL_DARK_SKIN_TONE = "\u26f9\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_BOUNCING_BALL = "\u26f9\ufe0f\u200d\u2640\ufe0f"
WOMAN_BOUNCING_BALL_LIGHT_SKIN_TONE = "\u26f9\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_BOUNCING_BALL_MEDIUM_LIGHT_SKIN_TONE = "\u26f9\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_BOUNCING_BALL_MEDIUM_SKIN_TONE = "\u26f9\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_BOUNCING_BALL_MEDIUM_DARK_SKIN_TONE = "\u26f9\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_BOUNCING_BALL_DARK_SKIN_TONE = "\u26f9\U0001f3ff\u200d\u2640\ufe0f"
PERSON_LIFTING_WEIGHTS = "\U0001f3cb\ufe0f"
PERSON_LIFTING_WEIGHTS_LIGHT_SKIN_TONE = "\U0001f3cb\U0001f3fb"
PERSON_LIFTING_WEIGHTS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3cb\U0001f3fc"
PERSON_LIFTING_WEIGHTS_MEDIUM_SKIN_TONE = "\U0001f3cb\U0001f3fd"
PERSON_LIFTING_WEIGHTS_MEDIUM_DARK_SKIN_TONE = "\U0001f3cb\U0001f3fe"
PERSON_LIFTING_WEIGHTS_DARK_SKIN_TONE = "\U0001f3cb\U0001f3ff"
MAN_LIFTING_WEIGHTS = "\U0001f3cb\ufe0f\u200d\u2642\ufe0f"
MAN_LIFTING_WEIGHTS_LIGHT_SKIN_TONE = "\U0001f3cb\U0001f3fb\u200d\u2642\ufe0f"
MAN_LIFTING_WEIGHTS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3cb\U0001f3fc\u200d\u2642\ufe0f"
MAN_LIFTING_WEIGHTS_MEDIUM_SKIN_TONE = "\U0001f3cb\U0001f3fd\u200d\u2642\ufe0f"
MAN_LIFTING_WEIGHTS_MEDIUM_DARK_SKIN_TONE = "\U0001f3cb\U0001f3fe\u200d\u2642\ufe0f"
MAN_LIFTING_WEIGHTS_DARK_SKIN_TONE = "\U0001f3cb\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_LIFTING_WEIGHTS = "\U0001f3cb\ufe0f\u200d\u2640\ufe0f"
WOMAN_LIFTING_WEIGHTS_LIGHT_SKIN_TONE = "\U0001f3cb\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_LIFTING_WEIGHTS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f3cb\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_LIFTING_WEIGHTS_MEDIUM_SKIN_TONE = "\U0001f3cb\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_LIFTING_WEIGHTS_MEDIUM_DARK_SKIN_TONE = "\U0001f3cb\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_LIFTING_WEIGHTS_DARK_SKIN_TONE = "\U0001f3cb\U0001f3ff\u200d\u2640\ufe0f"
PERSON_BIKING = "\U0001f6b4"
PERSON_BIKING_LIGHT_SKIN_TONE = "\U0001f6b4\U0001f3fb"
PERSON_BIKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b4\U0001f3fc"
PERSON_BIKING_MEDIUM_SKIN_TONE = "\U0001f6b4\U0001f3fd"
PERSON_BIKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b4\U0001f3fe"
PERSON_BIKING_DARK_SKIN_TONE = "\U0001f6b4\U0001f3ff"
MAN_BIKING = "\U0001f6b4\u200d\u2642\ufe0f"
MAN_BIKING_LIGHT_SKIN_TONE = "\U0001f6b4\U0001f3fb\u200d\u2642\ufe0f"
MAN_BIKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b4\U0001f3fc\u200d\u2642\ufe0f"
MAN_BIKING_MEDIUM_SKIN_TONE = "\U0001f6b4\U0001f3fd\u200d\u2642\ufe0f"
MAN_BIKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b4\U0001f3fe\u200d\u2642\ufe0f"
MAN_BIKING_DARK_SKIN_TONE = "\U0001f6b4\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_BIKING = "\U0001f6b4\u200d\u2640\ufe0f"
WOMAN_BIKING_LIGHT_SKIN_TONE = "\U0001f6b4\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_BIKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b4\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_BIKING_MEDIUM_SKIN_TONE = "\U0001f6b4\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_BIKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b4\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_BIKING_DARK_SKIN_TONE = "\U0001f6b4\U0001f3ff\u200d\u2640\ufe0f"
PERSON_MOUNTAIN_BIKING = "\U0001f6b5"
PERSON_MOUNTAIN_BIKING_LIGHT_SKIN_TONE = "\U0001f6b5\U0001f3fb"
PERSON_MOUNTAIN_BIKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b5\U0001f3fc"
PERSON_MOUNTAIN_BIKING_MEDIUM_SKIN_TONE = "\U0001f6b5\U0001f3fd"
PERSON_MOUNTAIN_BIKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b5\U0001f3fe"
PERSON_MOUNTAIN_BIKING_DARK_SKIN_TONE = "\U0001f6b5\U0001f3ff"
MAN_MOUNTAIN_BIKING = "\U0001f6b5\u200d\u2642\ufe0f"
MAN_MOUNTAIN_BIKING_LIGHT_SKIN_TONE = "\U0001f6b5\U0001f3fb\u200d\u2642\ufe0f"
MAN_MOUNTAIN_BIKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b5\U0001f3fc\u200d\u2642\ufe0f"
MAN_MOUNTAIN_BIKING_MEDIUM_SKIN_TONE = "\U0001f6b5\U0001f3fd\u200d\u2642\ufe0f"
MAN_MOUNTAIN_BIKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b5\U0001f3fe\u200d\u2642\ufe0f"
MAN_MOUNTAIN_BIKING_DARK_SKIN_TONE = "\U0001f6b5\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_MOUNTAIN_BIKING = "\U0001f6b5\u200d\u2640\ufe0f"
WOMAN_MOUNTAIN_BIKING_LIGHT_SKIN_TONE = "\U0001f6b5\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_MOUNTAIN_BIKING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6b5\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_MOUNTAIN_BIKING_MEDIUM_SKIN_TONE = "\U0001f6b5\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_MOUNTAIN_BIKING_MEDIUM_DARK_SKIN_TONE = "\U0001f6b5\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_MOUNTAIN_BIKING_DARK_SKIN_TONE = "\U0001f6b5\U0001f3ff\u200d\u2640\ufe0f"
PERSON_CARTWHEELING = "\U0001f938"
PERSON_CARTWHEELING_LIGHT_SKIN_TONE = "\U0001f938\U0001f3fb"
PERSON_CARTWHEELING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f938\U0001f3fc"
PERSON_CARTWHEELING_MEDIUM_SKIN_TONE = "\U0001f938\U0001f3fd"
PERSON_CARTWHEELING_MEDIUM_DARK_SKIN_TONE = "\U0001f938\U0001f3fe"
PERSON_CARTWHEELING_DARK_SKIN_TONE = "\U0001f938\U0001f3ff"
MAN_CARTWHEELING = "\U0001f938\u200d\u2642\ufe0f"
MAN_CARTWHEELING_LIGHT_SKIN_TONE = "\U0001f938\U0001f3fb\u200d\u2642\ufe0f"
MAN_CARTWHEELING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f938\U0001f3fc\u200d\u2642\ufe0f"
MAN_CARTWHEELING_MEDIUM_SKIN_TONE = "\U0001f938\U0001f3fd\u200d\u2642\ufe0f"
MAN_CARTWHEELING_MEDIUM_DARK_SKIN_TONE = "\U0001f938\U0001f3fe\u200d\u2642\ufe0f"
MAN_CARTWHEELING_DARK_SKIN_TONE = "\U0001f938\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_CARTWHEELING = "\U0001f938\u200d\u2640\ufe0f"
WOMAN_CARTWHEELING_LIGHT_SKIN_TONE = "\U0001f938\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_CARTWHEELING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f938\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_CARTWHEELING_MEDIUM_SKIN_TONE = "\U0001f938\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_CARTWHEELING_MEDIUM_DARK_SKIN_TONE = "\U0001f938\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_CARTWHEELING_DARK_SKIN_TONE = "\U0001f938\U0001f3ff\u200d\u2640\ufe0f"
PEOPLE_WRESTLING = "\U0001f93c"
MEN_WRESTLING = "\U0001f93c\u200d\u2642\ufe0f"
WOMEN_WRESTLING = "\U0001f93c\u200d\u2640\ufe0f"
PERSON_PLAYING_WATER_POLO = "\U0001f93d"
PERSON_PLAYING_WATER_POLO_LIGHT_SKIN_TONE = "\U0001f93d\U0001f3fb"
PERSON_PLAYING_WATER_POLO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f93d\U0001f3fc"
PERSON_PLAYING_WATER_POLO_MEDIUM_SKIN_TONE = "\U0001f93d\U0001f3fd"
PERSON_PLAYING_WATER_POLO_MEDIUM_DARK_SKIN_TONE = "\U0001f93d\U0001f3fe"
PERSON_PLAYING_WATER_POLO_DARK_SKIN_TONE = "\U0001f93d\U0001f3ff"
MAN_PLAYING_WATER_POLO = "\U0001f93d\u200d\u2642\ufe0f"
MAN_PLAYING_WATER_POLO_LIGHT_SKIN_TONE = "\U0001f93d\U0001f3fb\u200d\u2642\ufe0f"
MAN_PLAYING_WATER_POLO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f93d\U0001f3fc\u200d\u2642\ufe0f"
MAN_PLAYING_WATER_POLO_MEDIUM_SKIN_TONE = "\U0001f93d\U0001f3fd\u200d\u2642\ufe0f"
MAN_PLAYING_WATER_POLO_MEDIUM_DARK_SKIN_TONE = "\U0001f93d\U0001f3fe\u200d\u2642\ufe0f"
MAN_PLAYING_WATER_POLO_DARK_SKIN_TONE = "\U0001f93d\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_PLAYING_WATER_POLO = "\U0001f93d\u200d\u2640\ufe0f"
WOMAN_PLAYING_WATER_POLO_LIGHT_SKIN_TONE = "\U0001f93d\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_PLAYING_WATER_POLO_MEDIUM_LIGHT_SKIN_TONE = "\U0001f93d\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_PLAYING_WATER_POLO_MEDIUM_SKIN_TONE = "\U0001f93d\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_PLAYING_WATER_POLO_MEDIUM_DARK_SKIN_TONE = "\U0001f93d\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_PLAYING_WATER_POLO_DARK_SKIN_TONE = "\U0001f93d\U0001f3ff\u200d\u2640\ufe0f"
PERSON_PLAYING_HANDBALL = "\U0001f93e"
PERSON_PLAYING_HANDBALL_LIGHT_SKIN_TONE = "\U0001f93e\U0001f3fb"
PERSON_PLAYING_HANDBALL_MEDIUM_LIGHT_SKIN_TONE = "\U0001f93e\U0001f3fc"
PERSON_PLAYING_HANDBALL_MEDIUM_SKIN_TONE = "\U0001f93e\U0001f3fd"
PERSON_PLAYING_HANDBALL_MEDIUM_DARK_SKIN_TONE = "\U0001f93e\U0001f3fe"
PERSON_PLAYING_HANDBALL_DARK_SKIN_TONE = "\U0001f93e\U0001f3ff"
MAN_PLAYING_HANDBALL = "\U0001f93e\u200d\u2642\ufe0f"
MAN_PLAYING_HANDBALL_LIGHT_SKIN_TONE = "\U0001f93e\U0001f3fb\u200d\u2642\ufe0f"
MAN_PLAYING_HANDBALL_MEDIUM_LIGHT_SKIN_TONE = "\U0001f93e\U0001f3fc\u200d\u2642\ufe0f"
MAN_PLAYING_HANDBALL_MEDIUM_SKIN_TONE = "\U0001f93e\U0001f3fd\u200d\u2642\ufe0f"
MAN_PLAYING_HANDBALL_MEDIUM_DARK_SKIN_TONE = "\U0001f93e\U0001f3fe\u200d\u2642\ufe0f"
MAN_PLAYING_HANDBALL_DARK_SKIN_TONE = "\U0001f93e\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_PLAYING_HANDBALL = "\U0001f93e\u200d\u2640\ufe0f"
WOMAN_PLAYING_HANDBALL_LIGHT_SKIN_TONE = "\U0001f93e\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_PLAYING_HANDBALL_MEDIUM_LIGHT_SKIN_TONE = "\U0001f93e\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_PLAYING_HANDBALL_MEDIUM_SKIN_TONE = "\U0001f93e\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_PLAYING_HANDBALL_MEDIUM_DARK_SKIN_TONE = "\U0001f93e\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_PLAYING_HANDBALL_DARK_SKIN_TONE = "\U0001f93e\U0001f3ff\u200d\u2640\ufe0f"
PERSON_JUGGLING = "\U0001f939"
PERSON_JUGGLING_LIGHT_SKIN_TONE = "\U0001f939\U0001f3fb"
PERSON_JUGGLING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f939\U0001f3fc"
PERSON_JUGGLING_MEDIUM_SKIN_TONE = "\U0001f939\U0001f3fd"
PERSON_JUGGLING_MEDIUM_DARK_SKIN_TONE = "\U0001f939\U0001f3fe"
PERSON_JUGGLING_DARK_SKIN_TONE = "\U0001f939\U0001f3ff"
MAN_JUGGLING = "\U0001f939\u200d\u2642\ufe0f"
MAN_JUGGLING_LIGHT_SKIN_TONE = "\U0001f939\U0001f3fb\u200d\u2642\ufe0f"
MAN_JUGGLING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f939\U0001f3fc\u200d\u2642\ufe0f"
MAN_JUGGLING_MEDIUM_SKIN_TONE = "\U0001f939\U0001f3fd\u200d\u2642\ufe0f"
MAN_JUGGLING_MEDIUM_DARK_SKIN_TONE = "\U0001f939\U0001f3fe\u200d\u2642\ufe0f"
MAN_JUGGLING_DARK_SKIN_TONE = "\U0001f939\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_JUGGLING = "\U0001f939\u200d\u2640\ufe0f"
WOMAN_JUGGLING_LIGHT_SKIN_TONE = "\U0001f939\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_JUGGLING_MEDIUM_LIGHT_SKIN_TONE = "\U0001f939\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_JUGGLING_MEDIUM_SKIN_TONE = "\U0001f939\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_JUGGLING_MEDIUM_DARK_SKIN_TONE = "\U0001f939\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_JUGGLING_DARK_SKIN_TONE = "\U0001f939\U0001f3ff\u200d\u2640\ufe0f"
PERSON_IN_LOTUS_POSITION = "\U0001f9d8"
PERSON_IN_LOTUS_POSITION_LIGHT_SKIN_TONE = "\U0001f9d8\U0001f3fb"
PERSON_IN_LOTUS_POSITION_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d8\U0001f3fc"
PERSON_IN_LOTUS_POSITION_MEDIUM_SKIN_TONE = "\U0001f9d8\U0001f3fd"
PERSON_IN_LOTUS_POSITION_MEDIUM_DARK_SKIN_TONE = "\U0001f9d8\U0001f3fe"
PERSON_IN_LOTUS_POSITION_DARK_SKIN_TONE = "\U0001f9d8\U0001f3ff"
MAN_IN_LOTUS_POSITION = "\U0001f9d8\u200d\u2642\ufe0f"
MAN_IN_LOTUS_POSITION_LIGHT_SKIN_TONE = "\U0001f9d8\U0001f3fb\u200d\u2642\ufe0f"
MAN_IN_LOTUS_POSITION_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d8\U0001f3fc\u200d\u2642\ufe0f"
MAN_IN_LOTUS_POSITION_MEDIUM_SKIN_TONE = "\U0001f9d8\U0001f3fd\u200d\u2642\ufe0f"
MAN_IN_LOTUS_POSITION_MEDIUM_DARK_SKIN_TONE = "\U0001f9d8\U0001f3fe\u200d\u2642\ufe0f"
MAN_IN_LOTUS_POSITION_DARK_SKIN_TONE = "\U0001f9d8\U0001f3ff\u200d\u2642\ufe0f"
WOMAN_IN_LOTUS_POSITION = "\U0001f9d8\u200d\u2640\ufe0f"
WOMAN_IN_LOTUS_POSITION_LIGHT_SKIN_TONE = "\U0001f9d8\U0001f3fb\u200d\u2640\ufe0f"
WOMAN_IN_LOTUS_POSITION_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d8\U0001f3fc\u200d\u2640\ufe0f"
WOMAN_IN_LOTUS_POSITION_MEDIUM_SKIN_TONE = "\U0001f9d8\U0001f3fd\u200d\u2640\ufe0f"
WOMAN_IN_LOTUS_POSITION_MEDIUM_DARK_SKIN_TONE = "\U0001f9d8\U0001f3fe\u200d\u2640\ufe0f"
WOMAN_IN_LOTUS_POSITION_DARK_SKIN_TONE = "\U0001f9d8\U0001f3ff\u200d\u2640\ufe0f"
PERSON_TAKING_BATH = "\U0001f6c0"
PERSON_TAKING_BATH_LIGHT_SKIN_TONE = "\U0001f6c0\U0001f3fb"
PERSON_TAKING_BATH_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6c0\U0001f3fc"
PERSON_TAKING_BATH_MEDIUM_SKIN_TONE = "\U0001f6c0\U0001f3fd"
PERSON_TAKING_BATH_MEDIUM_DARK_SKIN_TONE = "\U0001f6c0\U0001f3fe"
PERSON_TAKING_BATH_DARK_SKIN_TONE = "\U0001f6c0\U0001f3ff"
PERSON_IN_BED = "\U0001f6cc"
PERSON_IN_BED_LIGHT_SKIN_TONE = "\U0001f6cc\U0001f3fb"
PERSON_IN_BED_MEDIUM_LIGHT_SKIN_TONE = "\U0001f6cc\U0001f3fc"
PERSON_IN_BED_MEDIUM_SKIN_TONE = "\U0001f6cc\U0001f3fd"
PERSON_IN_BED_MEDIUM_DARK_SKIN_TONE = "\U0001f6cc\U0001f3fe"
PERSON_IN_BED_DARK_SKIN_TONE = "\U0001f6cc\U0001f3ff"
PEOPLE_HOLDING_HANDS = "\U0001f9d1\u200d\U0001f91d\u200d\U0001f9d1"
PEOPLE_HOLDING_HANDS_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fb"
PEOPLE_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fc"
PEOPLE_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fd"
PEOPLE_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fe"
PEOPLE_HOLDING_HANDS_LIGHT_SKIN_TONE_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fb\u200d\U0001f91d\u200d\U0001f9d1\U0001f3ff"
PEOPLE_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fb"
PEOPLE_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3fc\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fc"
PEOPLE_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fd"
PEOPLE_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fe"
PEOPLE_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\U0001f91d\u200d\U0001f9d1\U0001f3ff"
PEOPLE_HOLDING_HANDS_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fb"
PEOPLE_HOLDING_HANDS_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fc"
PEOPLE_HOLDING_HANDS_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fd"
PEOPLE_HOLDING_HANDS_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fe"
PEOPLE_HOLDING_HANDS_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fd\u200d\U0001f91d\u200d\U0001f9d1\U0001f3ff"
PEOPLE_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fb"
PEOPLE_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fc"
PEOPLE_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fd"
PEOPLE_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001f9d1\U0001f3fe\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fe"
PEOPLE_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\U0001f91d\u200d\U0001f9d1\U0001f3ff"
PEOPLE_HOLDING_HANDS_DARK_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fb"
PEOPLE_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fc"
PEOPLE_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fd"
PEOPLE_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\U0001f91d\u200d\U0001f9d1\U0001f3fe"
PEOPLE_HOLDING_HANDS_DARK_SKIN_TONE = "\U0001f9d1\U0001f3ff\u200d\U0001f91d\u200d\U0001f9d1\U0001f3ff"
WOMEN_HOLDING_HANDS = "\U0001f46d"
WOMEN_HOLDING_HANDS_LIGHT_SKIN_TONE = "\U0001f46d\U0001f3fb"
WOMEN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\U0001f91d\u200d\U0001f469\U0001f3fc"
WOMEN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f91d\u200d\U0001f469\U0001f3fd"
WOMEN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\U0001f91d\u200d\U0001f469\U0001f3fe"
WOMEN_HOLDING_HANDS_LIGHT_SKIN_TONE_DARK_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\U0001f91d\u200d\U0001f469\U0001f3ff"
WOMEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\U0001f91d\u200d\U0001f469\U0001f3fb"
WOMEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f46d\U0001f3fc"
WOMEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\U0001f91d\u200d\U0001f469\U0001f3fd"
WOMEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\U0001f91d\u200d\U0001f469\U0001f3fe"
WOMEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\U0001f91d\u200d\U0001f469\U0001f3ff"
WOMEN_HOLDING_HANDS_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f91d\u200d\U0001f469\U0001f3fb"
WOMEN_HOLDING_HANDS_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\U0001f91d\u200d\U0001f469\U0001f3fc"
WOMEN_HOLDING_HANDS_MEDIUM_SKIN_TONE = "\U0001f46d\U0001f3fd"
WOMEN_HOLDING_HANDS_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\U0001f91d\u200d\U0001f469\U0001f3fe"
WOMEN_HOLDING_HANDS_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\U0001f91d\u200d\U0001f469\U0001f3ff"
WOMEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\U0001f91d\u200d\U0001f469\U0001f3fb"
WOMEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\U0001f91d\u200d\U0001f469\U0001f3fc"
WOMEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\U0001f91d\u200d\U0001f469\U0001f3fd"
WOMEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001f46d\U0001f3fe"
WOMEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\U0001f91d\u200d\U0001f469\U0001f3ff"
WOMEN_HOLDING_HANDS_DARK_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f91d\u200d\U0001f469\U0001f3fb"
WOMEN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\U0001f91d\u200d\U0001f469\U0001f3fc"
WOMEN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\U0001f91d\u200d\U0001f469\U0001f3fd"
WOMEN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\U0001f91d\u200d\U0001f469\U0001f3fe"
WOMEN_HOLDING_HANDS_DARK_SKIN_TONE = "\U0001f46d\U0001f3ff"
WOMAN_AND_MAN_HOLDING_HANDS = "\U0001f46b"
WOMAN_AND_MAN_HOLDING_HANDS_LIGHT_SKIN_TONE = "\U0001f46b\U0001f3fb"
WOMAN_AND_MAN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\U0001f91d\u200d\U0001f468\U0001f3fc"
WOMAN_AND_MAN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\U0001f91d\u200d\U0001f468\U0001f3fd"
WOMAN_AND_MAN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\U0001f91d\u200d\U0001f468\U0001f3fe"
WOMAN_AND_MAN_HOLDING_HANDS_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\U0001f91d\u200d\U0001f468\U0001f3ff"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\U0001f91d\u200d\U0001f468\U0001f3fb"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f46b\U0001f3fc"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\U0001f91d\u200d\U0001f468\U0001f3fd"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\U0001f91d\u200d\U0001f468\U0001f3fe"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\U0001f91d\u200d\U0001f468\U0001f3ff"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\U0001f91d\u200d\U0001f468\U0001f3fb"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\U0001f91d\u200d\U0001f468\U0001f3fc"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_SKIN_TONE = "\U0001f46b\U0001f3fd"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\U0001f91d\u200d\U0001f468\U0001f3fe"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\U0001f91d\u200d\U0001f468\U0001f3ff"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\U0001f91d\u200d\U0001f468\U0001f3fb"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\U0001f91d\u200d\U0001f468\U0001f3fc"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\U0001f91d\u200d\U0001f468\U0001f3fd"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001f46b\U0001f3fe"
WOMAN_AND_MAN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\U0001f91d\u200d\U0001f468\U0001f3ff"
WOMAN_AND_MAN_HOLDING_HANDS_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\U0001f91d\u200d\U0001f468\U0001f3fb"
WOMAN_AND_MAN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\U0001f91d\u200d\U0001f468\U0001f3fc"
WOMAN_AND_MAN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\U0001f91d\u200d\U0001f468\U0001f3fd"
WOMAN_AND_MAN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\U0001f91d\u200d\U0001f468\U0001f3fe"
WOMAN_AND_MAN_HOLDING_HANDS_DARK_SKIN_TONE = "\U0001f46b\U0001f3ff"
MEN_HOLDING_HANDS = "\U0001f46c"
MEN_HOLDING_HANDS_LIGHT_SKIN_TONE = "\U0001f46c\U0001f3fb"
MEN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\U0001f91d\u200d\U0001f468\U0001f3fc"
MEN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f91d\u200d\U0001f468\U0001f3fd"
MEN_HOLDING_HANDS_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\U0001f91d\u200d\U0001f468\U0001f3fe"
MEN_HOLDING_HANDS_LIGHT_SKIN_TONE_DARK_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\U0001f91d\u200d\U0001f468\U0001f3ff"
MEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\U0001f91d\u200d\U0001f468\U0001f3fb"
MEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f46c\U0001f3fc"
MEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\U0001f91d\u200d\U0001f468\U0001f3fd"
MEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\U0001f91d\u200d\U0001f468\U0001f3fe"
MEN_HOLDING_HANDS_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\U0001f91d\u200d\U0001f468\U0001f3ff"
MEN_HOLDING_HANDS_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f91d\u200d\U0001f468\U0001f3fb"
MEN_HOLDING_HANDS_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\U0001f91d\u200d\U0001f468\U0001f3fc"
MEN_HOLDING_HANDS_MEDIUM_SKIN_TONE = "\U0001f46c\U0001f3fd"
MEN_HOLDING_HANDS_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\U0001f91d\u200d\U0001f468\U0001f3fe"
MEN_HOLDING_HANDS_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\U0001f91d\u200d\U0001f468\U0001f3ff"
MEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\U0001f91d\u200d\U0001f468\U0001f3fb"
MEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\U0001f91d\u200d\U0001f468\U0001f3fc"
MEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\U0001f91d\u200d\U0001f468\U0001f3fd"
MEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE = "\U0001f46c\U0001f3fe"
MEN_HOLDING_HANDS_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\U0001f91d\u200d\U0001f468\U0001f3ff"
MEN_HOLDING_HANDS_DARK_SKIN_TONE_LIGHT_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f91d\u200d\U0001f468\U0001f3fb"
MEN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\U0001f91d\u200d\U0001f468\U0001f3fc"
MEN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\U0001f91d\u200d\U0001f468\U0001f3fd"
MEN_HOLDING_HANDS_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\U0001f91d\u200d\U0001f468\U0001f3fe"
MEN_HOLDING_HANDS_DARK_SKIN_TONE = "\U0001f46c\U0001f3ff"
KISS = "\U0001f48f"
KISS_LIGHT_SKIN_TONE = "\U0001f48f\U0001f3fb"
KISS_MEDIUM_LIGHT_SKIN_TONE = "\U0001f48f\U0001f3fc"
KISS_MEDIUM_SKIN_TONE = "\U0001f48f\U0001f3fd"
KISS_MEDIUM_DARK_SKIN_TONE = "\U0001f48f\U0001f3fe"
KISS_DARK_SKIN_TONE = "\U0001f48f\U0001f3ff"
KISS_PERSON_PERSON_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fc"
KISS_PERSON_PERSON_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fd"
KISS_PERSON_PERSON_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fe"
KISS_PERSON_PERSON_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3ff"
KISS_PERSON_PERSON_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fb"
KISS_PERSON_PERSON_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fd"
KISS_PERSON_PERSON_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fe"
KISS_PERSON_PERSON_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3ff"
KISS_PERSON_PERSON_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fb"
KISS_PERSON_PERSON_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fc"
KISS_PERSON_PERSON_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fe"
KISS_PERSON_PERSON_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3ff"
KISS_PERSON_PERSON_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fb"
KISS_PERSON_PERSON_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fc"
KISS_PERSON_PERSON_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fd"
KISS_PERSON_PERSON_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3ff"
KISS_PERSON_PERSON_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fb"
KISS_PERSON_PERSON_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fc"
KISS_PERSON_PERSON_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fd"
KISS_PERSON_PERSON_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f9d1\U0001f3fe"
KISS_WOMAN_MAN = "\U0001f469\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468"
KISS_WOMAN_MAN_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_WOMAN_MAN_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_WOMAN_MAN_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_WOMAN_MAN_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_WOMAN_MAN_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_WOMAN_MAN_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_WOMAN_MAN_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_WOMAN_MAN_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_WOMAN_MAN_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_WOMAN_MAN_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_WOMAN_MAN_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_WOMAN_MAN_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_WOMAN_MAN_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_WOMAN_MAN_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_WOMAN_MAN_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_MAN_MAN = "\U0001f468\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468"
KISS_MAN_MAN_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_MAN_MAN_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_MAN_MAN_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_MAN_MAN_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_MAN_MAN_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_MAN_MAN_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_MAN_MAN_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_MAN_MAN_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_MAN_MAN_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_MAN_MAN_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_MAN_MAN_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_MAN_MAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_MAN_MAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_MAN_MAN_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_MAN_MAN_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_MAN_MAN_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fb"
KISS_MAN_MAN_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fc"
KISS_MAN_MAN_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fd"
KISS_MAN_MAN_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3fe"
KISS_MAN_MAN_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f468\U0001f3ff"
KISS_WOMAN_WOMAN = "\U0001f469\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469"
KISS_WOMAN_WOMAN_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fb"
KISS_WOMAN_WOMAN_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fc"
KISS_WOMAN_WOMAN_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fd"
KISS_WOMAN_WOMAN_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fe"
KISS_WOMAN_WOMAN_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3ff"
KISS_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fb"
KISS_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fc"
KISS_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fd"
KISS_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fe"
KISS_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3ff"
KISS_WOMAN_WOMAN_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fb"
KISS_WOMAN_WOMAN_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fc"
KISS_WOMAN_WOMAN_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fd"
KISS_WOMAN_WOMAN_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fe"
KISS_WOMAN_WOMAN_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3ff"
KISS_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fb"
KISS_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fc"
KISS_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fd"
KISS_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fe"
KISS_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3ff"
KISS_WOMAN_WOMAN_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fb"
KISS_WOMAN_WOMAN_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fc"
KISS_WOMAN_WOMAN_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fd"
KISS_WOMAN_WOMAN_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3fe"
KISS_WOMAN_WOMAN_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f48b\u200d\U0001f469\U0001f3ff"
COUPLE_WITH_HEART = "\U0001f491"
COUPLE_WITH_HEART_LIGHT_SKIN_TONE = "\U0001f491\U0001f3fb"
COUPLE_WITH_HEART_MEDIUM_LIGHT_SKIN_TONE = "\U0001f491\U0001f3fc"
COUPLE_WITH_HEART_MEDIUM_SKIN_TONE = "\U0001f491\U0001f3fd"
COUPLE_WITH_HEART_MEDIUM_DARK_SKIN_TONE = "\U0001f491\U0001f3fe"
COUPLE_WITH_HEART_DARK_SKIN_TONE = "\U0001f491\U0001f3ff"
COUPLE_WITH_HEART_PERSON_PERSON_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fc"
COUPLE_WITH_HEART_PERSON_PERSON_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fd"
COUPLE_WITH_HEART_PERSON_PERSON_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fe"
COUPLE_WITH_HEART_PERSON_PERSON_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3ff"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fb"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fd"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fe"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3ff"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fb"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fc"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fe"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3ff"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fb"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fc"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fd"
COUPLE_WITH_HEART_PERSON_PERSON_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3ff"
COUPLE_WITH_HEART_PERSON_PERSON_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fb"
COUPLE_WITH_HEART_PERSON_PERSON_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fc"
COUPLE_WITH_HEART_PERSON_PERSON_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fd"
COUPLE_WITH_HEART_PERSON_PERSON_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f9d1\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f9d1\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_MAN = "\U0001f469\u200d\u2764\ufe0f\u200d\U0001f468"
COUPLE_WITH_HEART_WOMAN_MAN_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_MAN_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_MAN_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_MAN_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_MAN_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE = "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_MAN_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_MAN_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_MAN_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_MAN_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_MAN_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_MAN_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_MAN_MAN = "\U0001f468\u200d\u2764\ufe0f\u200d\U0001f468"
COUPLE_WITH_HEART_MAN_MAN_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_MAN_MAN_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_MAN_MAN_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_MAN_MAN_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_MAN_MAN_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE = "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_SKIN_TONE = "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_DARK_SKIN_TONE = "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_MAN_MAN_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_MAN_MAN_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fb"
COUPLE_WITH_HEART_MAN_MAN_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fc"
COUPLE_WITH_HEART_MAN_MAN_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fd"
COUPLE_WITH_HEART_MAN_MAN_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3fe"
COUPLE_WITH_HEART_MAN_MAN_DARK_SKIN_TONE = "\U0001f468\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f468\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_WOMAN = "\U0001f469\u200d\u2764\ufe0f\u200d\U0001f469"
COUPLE_WITH_HEART_WOMAN_WOMAN_LIGHT_SKIN_TONE = "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_WOMAN_LIGHT_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_WOMAN_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_WOMAN_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_WOMAN_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fb\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_LIGHT_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fc\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_SKIN_TONE = "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fd\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_WOMAN_MEDIUM_DARK_SKIN_TONE_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3fe\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3ff"
COUPLE_WITH_HEART_WOMAN_WOMAN_DARK_SKIN_TONE_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fb"
COUPLE_WITH_HEART_WOMAN_WOMAN_DARK_SKIN_TONE_MEDIUM_LIGHT_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fc"
COUPLE_WITH_HEART_WOMAN_WOMAN_DARK_SKIN_TONE_MEDIUM_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fd"
COUPLE_WITH_HEART_WOMAN_WOMAN_DARK_SKIN_TONE_MEDIUM_DARK_SKIN_TONE = \
    "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3fe"
COUPLE_WITH_HEART_WOMAN_WOMAN_DARK_SKIN_TONE = "\U0001f469\U0001f3ff\u200d\u2764\ufe0f\u200d\U0001f469\U0001f3ff"
FAMILY = "\U0001f46a"
FAMILY_MAN_WOMAN_BOY = "\U0001f468\u200d\U0001f469\u200d\U0001f466"
FAMILY_MAN_WOMAN_GIRL = "\U0001f468\u200d\U0001f469\u200d\U0001f467"
FAMILY_MAN_WOMAN_GIRL_BOY = "\U0001f468\u200d\U0001f469\u200d\U0001f467\u200d\U0001f466"
FAMILY_MAN_WOMAN_BOY_BOY = "\U0001f468\u200d\U0001f469\u200d\U0001f466\u200d\U0001f466"
FAMILY_MAN_WOMAN_GIRL_GIRL = "\U0001f468\u200d\U0001f469\u200d\U0001f467\u200d\U0001f467"
FAMILY_MAN_MAN_BOY = "\U0001f468\u200d\U0001f468\u200d\U0001f466"
FAMILY_MAN_MAN_GIRL = "\U0001f468\u200d\U0001f468\u200d\U0001f467"
FAMILY_MAN_MAN_GIRL_BOY = "\U0001f468\u200d\U0001f468\u200d\U0001f467\u200d\U0001f466"
FAMILY_MAN_MAN_BOY_BOY = "\U0001f468\u200d\U0001f468\u200d\U0001f466\u200d\U0001f466"
FAMILY_MAN_MAN_GIRL_GIRL = "\U0001f468\u200d\U0001f468\u200d\U0001f467\u200d\U0001f467"
FAMILY_WOMAN_WOMAN_BOY = "\U0001f469\u200d\U0001f469\u200d\U0001f466"
FAMILY_WOMAN_WOMAN_GIRL = "\U0001f469\u200d\U0001f469\u200d\U0001f467"
FAMILY_WOMAN_WOMAN_GIRL_BOY = "\U0001f469\u200d\U0001f469\u200d\U0001f467\u200d\U0001f466"
FAMILY_WOMAN_WOMAN_BOY_BOY = "\U0001f469\u200d\U0001f469\u200d\U0001f466\u200d\U0001f466"
FAMILY_WOMAN_WOMAN_GIRL_GIRL = "\U0001f469\u200d\U0001f469\u200d\U0001f467\u200d\U0001f467"
FAMILY_MAN_BOY = "\U0001f468\u200d\U0001f466"
FAMILY_MAN_BOY_BOY = "\U0001f468\u200d\U0001f466\u200d\U0001f466"
FAMILY_MAN_GIRL = "\U0001f468\u200d\U0001f467"
FAMILY_MAN_GIRL_BOY = "\U0001f468\u200d\U0001f467\u200d\U0001f466"
FAMILY_MAN_GIRL_GIRL = "\U0001f468\u200d\U0001f467\u200d\U0001f467"
FAMILY_WOMAN_BOY = "\U0001f469\u200d\U0001f466"
FAMILY_WOMAN_BOY_BOY = "\U0001f469\u200d\U0001f466\u200d\U0001f466"
FAMILY_WOMAN_GIRL = "\U0001f469\u200d\U0001f467"
FAMILY_WOMAN_GIRL_BOY = "\U0001f469\u200d\U0001f467\u200d\U0001f466"
FAMILY_WOMAN_GIRL_GIRL = "\U0001f469\u200d\U0001f467\u200d\U0001f467"
SPEAKING_HEAD = "\U0001f5e3\ufe0f"
BUST_IN_SILHOUETTE = "\U0001f464"
BUSTS_IN_SILHOUETTE = "\U0001f465"
PEOPLE_HUGGING = "\U0001fac2"
FOOTPRINTS = "\U0001f463"
LIGHT_SKIN_TONE = "\U0001f3fb"
MEDIUM_LIGHT_SKIN_TONE = "\U0001f3fc"
MEDIUM_SKIN_TONE = "\U0001f3fd"
MEDIUM_DARK_SKIN_TONE = "\U0001f3fe"
DARK_SKIN_TONE = "\U0001f3ff"
RED_HAIR = "\U0001f9b0"
CURLY_HAIR = "\U0001f9b1"
WHITE_HAIR = "\U0001f9b3"
BALD = "\U0001f9b2"
MONKEY_FACE = "\U0001f435"
MONKEY = "\U0001f412"
GORILLA = "\U0001f98d"
ORANGUTAN = "\U0001f9a7"
DOG_FACE = "\U0001f436"
DOG = "\U0001f415"
GUIDE_DOG = "\U0001f9ae"
SERVICE_DOG = "\U0001f415\u200d\U0001f9ba"
POODLE = "\U0001f429"
WOLF = "\U0001f43a"
FOX = "\U0001f98a"
RACCOON = "\U0001f99d"
CAT_FACE = "\U0001f431"
CAT = "\U0001f408"
BLACK_CAT = "\U0001f408\u200d\u2b1b"
LION = "\U0001f981"
TIGER_FACE = "\U0001f42f"
TIGER = "\U0001f405"
LEOPARD = "\U0001f406"
HORSE_FACE = "\U0001f434"
HORSE = "\U0001f40e"
UNICORN = "\U0001f984"
ZEBRA = "\U0001f993"
DEER = "\U0001f98c"
BISON = "\U0001f9ac"
COW_FACE = "\U0001f42e"
OX = "\U0001f402"
WATER_BUFFALO = "\U0001f403"
COW = "\U0001f404"
PIG_FACE = "\U0001f437"
PIG = "\U0001f416"
BOAR = "\U0001f417"
PIG_NOSE = "\U0001f43d"
RAM = "\U0001f40f"
EWE = "\U0001f411"
GOAT = "\U0001f410"
CAMEL = "\U0001f42a"
TWO_HUMP_CAMEL = "\U0001f42b"
LLAMA = "\U0001f999"
GIRAFFE = "\U0001f992"
ELEPHANT = "\U0001f418"
MAMMOTH = "\U0001f9a3"
RHINOCEROS = "\U0001f98f"
HIPPOPOTAMUS = "\U0001f99b"
MOUSE_FACE = "\U0001f42d"
MOUSE = "\U0001f401"
RAT = "\U0001f400"
HAMSTER = "\U0001f439"
RABBIT_FACE = "\U0001f430"
RABBIT = "\U0001f407"
CHIPMUNK = "\U0001f43f\ufe0f"
BEAVER = "\U0001f9ab"
HEDGEHOG = "\U0001f994"
BAT = "\U0001f987"
BEAR = "\U0001f43b"
POLAR_BEAR = "\U0001f43b\u200d\u2744\ufe0f"
KOALA = "\U0001f428"
PANDA = "\U0001f43c"
SLOTH = "\U0001f9a5"
OTTER = "\U0001f9a6"
SKUNK = "\U0001f9a8"
KANGAROO = "\U0001f998"
BADGER = "\U0001f9a1"
PAW_PRINTS = "\U0001f43e"
TURKEY = "\U0001f983"
CHICKEN = "\U0001f414"
ROOSTER = "\U0001f413"
HATCHING_CHICK = "\U0001f423"
BABY_CHICK = "\U0001f424"
FRONT_FACING_BABY_CHICK = "\U0001f425"
BIRD = "\U0001f426"
PENGUIN = "\U0001f427"
DOVE = "\U0001f54a\ufe0f"
EAGLE = "\U0001f985"
DUCK = "\U0001f986"
SWAN = "\U0001f9a2"
OWL = "\U0001f989"
DODO = "\U0001f9a4"
FEATHER = "\U0001fab6"
FLAMINGO = "\U0001f9a9"
PEACOCK = "\U0001f99a"
PARROT = "\U0001f99c"
FROG = "\U0001f438"
CROCODILE = "\U0001f40a"
TURTLE = "\U0001f422"
LIZARD = "\U0001f98e"
SNAKE = "\U0001f40d"
DRAGON_FACE = "\U0001f432"
DRAGON = "\U0001f409"
SAUROPOD = "\U0001f995"
T_REX = "\U0001f996"
SPOUTING_WHALE = "\U0001f433"
WHALE = "\U0001f40b"
DOLPHIN = "\U0001f42c"
SEAL = "\U0001f9ad"
FISH = "\U0001f41f"
TROPICAL_FISH = "\U0001f420"
BLOWFISH = "\U0001f421"
SHARK = "\U0001f988"
OCTOPUS = "\U0001f419"
SPIRAL_SHELL = "\U0001f41a"
CORAL = "\U0001fab8"
SNAIL = "\U0001f40c"
BUTTERFLY = "\U0001f98b"
BUG = "\U0001f41b"
ANT = "\U0001f41c"
HONEYBEE = "\U0001f41d"
BEETLE = "\U0001fab2"
LADY_BEETLE = "\U0001f41e"
CRICKET = "\U0001f997"
COCKROACH = "\U0001fab3"
SPIDER = "\U0001f577\ufe0f"
SPIDER_WEB = "\U0001f578\ufe0f"
SCORPION = "\U0001f982"
MOSQUITO = "\U0001f99f"
FLY = "\U0001fab0"
WORM = "\U0001fab1"
MICROBE = "\U0001f9a0"
BOUQUET = "\U0001f490"
CHERRY_BLOSSOM = "\U0001f338"
WHITE_FLOWER = "\U0001f4ae"
LOTUS = "\U0001fab7"
ROSETTE = "\U0001f3f5\ufe0f"
ROSE = "\U0001f339"
WILTED_FLOWER = "\U0001f940"
HIBISCUS = "\U0001f33a"
SUNFLOWER = "\U0001f33b"
BLOSSOM = "\U0001f33c"
TULIP = "\U0001f337"
SEEDLING = "\U0001f331"
POTTED_PLANT = "\U0001fab4"
EVERGREEN_TREE = "\U0001f332"
DECIDUOUS_TREE = "\U0001f333"
PALM_TREE = "\U0001f334"
CACTUS = "\U0001f335"
SHEAF_OF_RICE = "\U0001f33e"
HERB = "\U0001f33f"
SHAMROCK = "\u2618\ufe0f"
FOUR_LEAF_CLOVER = "\U0001f340"
MAPLE_LEAF = "\U0001f341"
FALLEN_LEAF = "\U0001f342"
LEAF_FLUTTERING_IN_WIND = "\U0001f343"
EMPTY_NEST = "\U0001fab9"
NEST_WITH_EGGS = "\U0001faba"
GRAPES = "\U0001f347"
MELON = "\U0001f348"
WATERMELON = "\U0001f349"
TANGERINE = "\U0001f34a"
LEMON = "\U0001f34b"
BANANA = "\U0001f34c"
PINEAPPLE = "\U0001f34d"
MANGO = "\U0001f96d"
RED_APPLE = "\U0001f34e"
GREEN_APPLE = "\U0001f34f"
PEAR = "\U0001f350"
PEACH = "\U0001f351"
CHERRIES = "\U0001f352"
STRAWBERRY = "\U0001f353"
BLUEBERRIES = "\U0001fad0"
KIWI_FRUIT = "\U0001f95d"
TOMATO = "\U0001f345"
OLIVE = "\U0001fad2"
COCONUT = "\U0001f965"
AVOCADO = "\U0001f951"
EGGPLANT = "\U0001f346"
POTATO = "\U0001f954"
CARROT = "\U0001f955"
EAR_OF_CORN = "\U0001f33d"
HOT_PEPPER = "\U0001f336\ufe0f"
BELL_PEPPER = "\U0001fad1"
CUCUMBER = "\U0001f952"
LEAFY_GREEN = "\U0001f96c"
BROCCOLI = "\U0001f966"
GARLIC = "\U0001f9c4"
ONION = "\U0001f9c5"
MUSHROOM = "\U0001f344"
PEANUTS = "\U0001f95c"
BEANS = "\U0001fad8"
CHESTNUT = "\U0001f330"
BREAD = "\U0001f35e"
CROISSANT = "\U0001f950"
BAGUETTE_BREAD = "\U0001f956"
FLATBREAD = "\U0001fad3"
PRETZEL = "\U0001f968"
BAGEL = "\U0001f96f"
PANCAKES = "\U0001f95e"
WAFFLE = "\U0001f9c7"
CHEESE_WEDGE = "\U0001f9c0"
MEAT_ON_BONE = "\U0001f356"
POULTRY_LEG = "\U0001f357"
CUT_OF_MEAT = "\U0001f969"
BACON = "\U0001f953"
HAMBURGER = "\U0001f354"
FRENCH_FRIES = "\U0001f35f"
PIZZA = "\U0001f355"
HOT_DOG = "\U0001f32d"
SANDWICH = "\U0001f96a"
TACO = "\U0001f32e"
BURRITO = "\U0001f32f"
TAMALE = "\U0001fad4"
STUFFED_FLATBREAD = "\U0001f959"
FALAFEL = "\U0001f9c6"
EGG = "\U0001f95a"
COOKING = "\U0001f373"
SHALLOW_PAN_OF_FOOD = "\U0001f958"
POT_OF_FOOD = "\U0001f372"
FONDUE = "\U0001fad5"
BOWL_WITH_SPOON = "\U0001f963"
GREEN_SALAD = "\U0001f957"
POPCORN = "\U0001f37f"
BUTTER = "\U0001f9c8"
SALT = "\U0001f9c2"
CANNED_FOOD = "\U0001f96b"
BENTO_BOX = "\U0001f371"
RICE_CRACKER = "\U0001f358"
RICE_BALL = "\U0001f359"
COOKED_RICE = "\U0001f35a"
CURRY_RICE = "\U0001f35b"
STEAMING_BOWL = "\U0001f35c"
SPAGHETTI = "\U0001f35d"
ROASTED_SWEET_POTATO = "\U0001f360"
ODEN = "\U0001f362"
SUSHI = "\U0001f363"
FRIED_SHRIMP = "\U0001f364"
FISH_CAKE_WITH_SWIRL = "\U0001f365"
MOON_CAKE = "\U0001f96e"
DANGO = "\U0001f361"
DUMPLING = "\U0001f95f"
FORTUNE_COOKIE = "\U0001f960"
TAKEOUT_BOX = "\U0001f961"
CRAB = "\U0001f980"
LOBSTER = "\U0001f99e"
SHRIMP = "\U0001f990"
SQUID = "\U0001f991"
OYSTER = "\U0001f9aa"
SOFT_ICE_CREAM = "\U0001f366"
SHAVED_ICE = "\U0001f367"
ICE_CREAM = "\U0001f368"
DOUGHNUT = "\U0001f369"
COOKIE = "\U0001f36a"
BIRTHDAY_CAKE = "\U0001f382"
SHORTCAKE = "\U0001f370"
CUPCAKE = "\U0001f9c1"
PIE = "\U0001f967"
CHOCOLATE_BAR = "\U0001f36b"
CANDY = "\U0001f36c"
LOLLIPOP = "\U0001f36d"
CUSTARD = "\U0001f36e"
HONEY_POT = "\U0001f36f"
BABY_BOTTLE = "\U0001f37c"
GLASS_OF_MILK = "\U0001f95b"
HOT_BEVERAGE = "\u2615"
TEAPOT = "\U0001fad6"
TEACUP_WITHOUT_HANDLE = "\U0001f375"
SAKE = "\U0001f376"
BOTTLE_WITH_POPPING_CORK = "\U0001f37e"
WINE_GLASS = "\U0001f377"
COCKTAIL_GLASS = "\U0001f378"
TROPICAL_DRINK = "\U0001f379"
BEER_MUG = "\U0001f37a"
CLINKING_BEER_MUGS = "\U0001f37b"
CLINKING_GLASSES = "\U0001f942"
TUMBLER_GLASS = "\U0001f943"
POURING_LIQUID = "\U0001fad7"
CUP_WITH_STRAW = "\U0001f964"
BUBBLE_TEA = "\U0001f9cb"
BEVERAGE_BOX = "\U0001f9c3"
MATE = "\U0001f9c9"
ICE = "\U0001f9ca"
CHOPSTICKS = "\U0001f962"
FORK_AND_KNIFE_WITH_PLATE = "\U0001f37d\ufe0f"
FORK_AND_KNIFE = "\U0001f374"
SPOON = "\U0001f944"
KITCHEN_KNIFE = "\U0001f52a"
JAR = "\U0001fad9"
AMPHORA = "\U0001f3fa"
GLOBE_SHOWING_EUROPE_AFRICA = "\U0001f30d"
GLOBE_SHOWING_AMERICAS = "\U0001f30e"
GLOBE_SHOWING_ASIA_AUSTRALIA = "\U0001f30f"
GLOBE_WITH_MERIDIANS = "\U0001f310"
WORLD_MAP = "\U0001f5fa\ufe0f"
MAP_OF_JAPAN = "\U0001f5fe"
COMPASS = "\U0001f9ed"
SNOW_CAPPED_MOUNTAIN = "\U0001f3d4\ufe0f"
MOUNTAIN = "\u26f0\ufe0f"
VOLCANO = "\U0001f30b"
MOUNT_FUJI = "\U0001f5fb"
CAMPING = "\U0001f3d5\ufe0f"
BEACH_WITH_UMBRELLA = "\U0001f3d6\ufe0f"
DESERT = "\U0001f3dc\ufe0f"
DESERT_ISLAND = "\U0001f3dd\ufe0f"
NATIONAL_PARK = "\U0001f3de\ufe0f"
STADIUM = "\U0001f3df\ufe0f"
CLASSICAL_BUILDING = "\U0001f3db\ufe0f"
BUILDING_CONSTRUCTION = "\U0001f3d7\ufe0f"
BRICK = "\U0001f9f1"
ROCK = "\U0001faa8"
WOOD = "\U0001fab5"
HUT = "\U0001f6d6"
HOUSES = "\U0001f3d8\ufe0f"
DERELICT_HOUSE = "\U0001f3da\ufe0f"
HOUSE = "\U0001f3e0"
HOUSE_WITH_GARDEN = "\U0001f3e1"
OFFICE_BUILDING = "\U0001f3e2"
JAPANESE_POST_OFFICE = "\U0001f3e3"
POST_OFFICE = "\U0001f3e4"
HOSPITAL = "\U0001f3e5"
BANK = "\U0001f3e6"
HOTEL = "\U0001f3e8"
LOVE_HOTEL = "\U0001f3e9"
CONVENIENCE_STORE = "\U0001f3ea"
SCHOOL = "\U0001f3eb"
DEPARTMENT_STORE = "\U0001f3ec"
FACTORY = "\U0001f3ed"
JAPANESE_CASTLE = "\U0001f3ef"
CASTLE = "\U0001f3f0"
WEDDING = "\U0001f492"
TOKYO_TOWER = "\U0001f5fc"
STATUE_OF_LIBERTY = "\U0001f5fd"
CHURCH = "\u26ea"
MOSQUE = "\U0001f54c"
HINDU_TEMPLE = "\U0001f6d5"
SYNAGOGUE = "\U0001f54d"
SHINTO_SHRINE = "\u26e9\ufe0f"
KAABA = "\U0001f54b"
FOUNTAIN = "\u26f2"
TENT = "\u26fa"
FOGGY = "\U0001f301"
NIGHT_WITH_STARS = "\U0001f303"
CITYSCAPE = "\U0001f3d9\ufe0f"
SUNRISE_OVER_MOUNTAINS = "\U0001f304"
SUNRISE = "\U0001f305"
CITYSCAPE_AT_DUSK = "\U0001f306"
SUNSET = "\U0001f307"
BRIDGE_AT_NIGHT = "\U0001f309"
HOT_SPRINGS = "\u2668\ufe0f"
CAROUSEL_HORSE = "\U0001f3a0"
PLAYGROUND_SLIDE = "\U0001f6dd"
FERRIS_WHEEL = "\U0001f3a1"
ROLLER_COASTER = "\U0001f3a2"
BARBER_POLE = "\U0001f488"
CIRCUS_TENT = "\U0001f3aa"
LOCOMOTIVE = "\U0001f682"
RAILWAY_CAR = "\U0001f683"
HIGH_SPEED_TRAIN = "\U0001f684"
BULLET_TRAIN = "\U0001f685"
TRAIN = "\U0001f686"
METRO = "\U0001f687"
LIGHT_RAIL = "\U0001f688"
STATION = "\U0001f689"
TRAM = "\U0001f68a"
MONORAIL = "\U0001f69d"
MOUNTAIN_RAILWAY = "\U0001f69e"
TRAM_CAR = "\U0001f68b"
BUS = "\U0001f68c"
ONCOMING_BUS = "\U0001f68d"
TROLLEYBUS = "\U0001f68e"
MINIBUS = "\U0001f690"
AMBULANCE = "\U0001f691"
FIRE_ENGINE = "\U0001f692"
POLICE_CAR = "\U0001f693"
ONCOMING_POLICE_CAR = "\U0001f694"
TAXI = "\U0001f695"
ONCOMING_TAXI = "\U0001f696"
AUTOMOBILE = "\U0001f697"
ONCOMING_AUTOMOBILE = "\U0001f698"
SPORT_UTILITY_VEHICLE = "\U0001f699"
PICKUP_TRUCK = "\U0001f6fb"
DELIVERY_TRUCK = "\U0001f69a"
ARTICULATED_LORRY = "\U0001f69b"
TRACTOR = "\U0001f69c"
RACING_CAR = "\U0001f3ce\ufe0f"
MOTORCYCLE = "\U0001f3cd\ufe0f"
MOTOR_SCOOTER = "\U0001f6f5"
MANUAL_WHEELCHAIR = "\U0001f9bd"
MOTORIZED_WHEELCHAIR = "\U0001f9bc"
AUTO_RICKSHAW = "\U0001f6fa"
BICYCLE = "\U0001f6b2"
KICK_SCOOTER = "\U0001f6f4"
SKATEBOARD = "\U0001f6f9"
ROLLER_SKATE = "\U0001f6fc"
BUS_STOP = "\U0001f68f"
MOTORWAY = "\U0001f6e3\ufe0f"
RAILWAY_TRACK = "\U0001f6e4\ufe0f"
OIL_DRUM = "\U0001f6e2\ufe0f"
FUEL_PUMP = "\u26fd"
WHEEL = "\U0001f6de"
POLICE_CAR_LIGHT = "\U0001f6a8"
HORIZONTAL_TRAFFIC_LIGHT = "\U0001f6a5"
VERTICAL_TRAFFIC_LIGHT = "\U0001f6a6"
STOP_SIGN = "\U0001f6d1"
CONSTRUCTION = "\U0001f6a7"
ANCHOR = "\u2693"
RING_BUOY = "\U0001f6df"
SAILBOAT = "\u26f5"
CANOE = "\U0001f6f6"
SPEEDBOAT = "\U0001f6a4"
PASSENGER_SHIP = "\U0001f6f3\ufe0f"
FERRY = "\u26f4\ufe0f"
MOTOR_BOAT = "\U0001f6e5\ufe0f"
SHIP = "\U0001f6a2"
AIRPLANE = "\u2708\ufe0f"
SMALL_AIRPLANE = "\U0001f6e9\ufe0f"
AIRPLANE_DEPARTURE = "\U0001f6eb"
AIRPLANE_ARRIVAL = "\U0001f6ec"
PARACHUTE = "\U0001fa82"
SEAT = "\U0001f4ba"
HELICOPTER = "\U0001f681"
SUSPENSION_RAILWAY = "\U0001f69f"
MOUNTAIN_CABLEWAY = "\U0001f6a0"
AERIAL_TRAMWAY = "\U0001f6a1"
SATELLITE = "\U0001f6f0\ufe0f"
ROCKET = "\U0001f680"
FLYING_SAUCER = "\U0001f6f8"
BELLHOP_BELL = "\U0001f6ce\ufe0f"
LUGGAGE = "\U0001f9f3"
HOURGLASS_DONE = "\u231b"
HOURGLASS_NOT_DONE = "\u23f3"
WATCH = "\u231a"
ALARM_CLOCK = "\u23f0"
STOPWATCH = "\u23f1\ufe0f"
TIMER_CLOCK = "\u23f2\ufe0f"
MANTELPIECE_CLOCK = "\U0001f570\ufe0f"
TWELVE_O_CLOCK = "\U0001f55b"
TWELVE_THIRTY = "\U0001f567"
ONE_O_CLOCK = "\U0001f550"
ONE_THIRTY = "\U0001f55c"
TWO_O_CLOCK = "\U0001f551"
TWO_THIRTY = "\U0001f55d"
THREE_O_CLOCK = "\U0001f552"
THREE_THIRTY = "\U0001f55e"
FOUR_O_CLOCK = "\U0001f553"
FOUR_THIRTY = "\U0001f55f"
FIVE_O_CLOCK = "\U0001f554"
FIVE_THIRTY = "\U0001f560"
SIX_O_CLOCK = "\U0001f555"
SIX_THIRTY = "\U0001f561"
SEVEN_O_CLOCK = "\U0001f556"
SEVEN_THIRTY = "\U0001f562"
EIGHT_O_CLOCK = "\U0001f557"
EIGHT_THIRTY = "\U0001f563"
NINE_O_CLOCK = "\U0001f558"
NINE_THIRTY = "\U0001f564"
TEN_O_CLOCK = "\U0001f559"
TEN_THIRTY = "\U0001f565"
ELEVEN_O_CLOCK = "\U0001f55a"
ELEVEN_THIRTY = "\U0001f566"
NEW_MOON = "\U0001f311"
WAXING_CRESCENT_MOON = "\U0001f312"
FIRST_QUARTER_MOON = "\U0001f313"
WAXING_GIBBOUS_MOON = "\U0001f314"
FULL_MOON = "\U0001f315"
WANING_GIBBOUS_MOON = "\U0001f316"
LAST_QUARTER_MOON = "\U0001f317"
WANING_CRESCENT_MOON = "\U0001f318"
CRESCENT_MOON = "\U0001f319"
NEW_MOON_FACE = "\U0001f31a"
FIRST_QUARTER_MOON_FACE = "\U0001f31b"
LAST_QUARTER_MOON_FACE = "\U0001f31c"
THERMOMETER = "\U0001f321\ufe0f"
SUN = "\u2600\ufe0f"
FULL_MOON_FACE = "\U0001f31d"
SUN_WITH_FACE = "\U0001f31e"
RINGED_PLANET = "\U0001fa90"
STAR = "\u2b50"
GLOWING_STAR = "\U0001f31f"
SHOOTING_STAR = "\U0001f320"
MILKY_WAY = "\U0001f30c"
CLOUD = "\u2601\ufe0f"
SUN_BEHIND_CLOUD = "\u26c5"
CLOUD_WITH_LIGHTNING_AND_RAIN = "\u26c8\ufe0f"
SUN_BEHIND_SMALL_CLOUD = "\U0001f324\ufe0f"
SUN_BEHIND_LARGE_CLOUD = "\U0001f325\ufe0f"
SUN_BEHIND_RAIN_CLOUD = "\U0001f326\ufe0f"
CLOUD_WITH_RAIN = "\U0001f327\ufe0f"
CLOUD_WITH_SNOW = "\U0001f328\ufe0f"
CLOUD_WITH_LIGHTNING = "\U0001f329\ufe0f"
TORNADO = "\U0001f32a\ufe0f"
FOG = "\U0001f32b\ufe0f"
WIND_FACE = "\U0001f32c\ufe0f"
CYCLONE = "\U0001f300"
RAINBOW = "\U0001f308"
CLOSED_UMBRELLA = "\U0001f302"
UMBRELLA = "\u2602\ufe0f"
UMBRELLA_WITH_RAIN_DROPS = "\u2614"
UMBRELLA_ON_GROUND = "\u26f1\ufe0f"
HIGH_VOLTAGE = "\u26a1"
SNOWFLAKE = "\u2744\ufe0f"
SNOWMAN = "\u2603\ufe0f"
SNOWMAN_WITHOUT_SNOW = "\u26c4"
COMET = "\u2604\ufe0f"
FIRE = "\U0001f525"
DROPLET = "\U0001f4a7"
WATER_WAVE = "\U0001f30a"
JACK_O_LANTERN = "\U0001f383"
CHRISTMAS_TREE = "\U0001f384"
FIREWORKS = "\U0001f386"
SPARKLER = "\U0001f387"
FIRECRACKER = "\U0001f9e8"
SPARKLES = "\u2728"
BALLOON = "\U0001f388"
PARTY_POPPER = "\U0001f389"
CONFETTI_BALL = "\U0001f38a"
TANABATA_TREE = "\U0001f38b"
PINE_DECORATION = "\U0001f38d"
JAPANESE_DOLLS = "\U0001f38e"
CARP_STREAMER = "\U0001f38f"
WIND_CHIME = "\U0001f390"
MOON_VIEWING_CEREMONY = "\U0001f391"
RED_ENVELOPE = "\U0001f9e7"
RIBBON = "\U0001f380"
WRAPPED_GIFT = "\U0001f381"
REMINDER_RIBBON = "\U0001f397\ufe0f"
ADMISSION_TICKETS = "\U0001f39f\ufe0f"
TICKET = "\U0001f3ab"
MILITARY_MEDAL = "\U0001f396\ufe0f"
TROPHY = "\U0001f3c6"
SPORTS_MEDAL = "\U0001f3c5"
FIRST_PLACE_MEDAL = "\U0001f947"
SECOND_PLACE_MEDAL = "\U0001f948"
THIRD_PLACE_MEDAL = "\U0001f949"
SOCCER_BALL = "\u26bd"
BASEBALL = "\u26be"
SOFTBALL = "\U0001f94e"
BASKETBALL = "\U0001f3c0"
VOLLEYBALL = "\U0001f3d0"
AMERICAN_FOOTBALL = "\U0001f3c8"
RUGBY_FOOTBALL = "\U0001f3c9"
TENNIS = "\U0001f3be"
FLYING_DISC = "\U0001f94f"
BOWLING = "\U0001f3b3"
CRICKET_GAME = "\U0001f3cf"
FIELD_HOCKEY = "\U0001f3d1"
ICE_HOCKEY = "\U0001f3d2"
LACROSSE = "\U0001f94d"
PING_PONG = "\U0001f3d3"
BADMINTON = "\U0001f3f8"
BOXING_GLOVE = "\U0001f94a"
MARTIAL_ARTS_UNIFORM = "\U0001f94b"
GOAL_NET = "\U0001f945"
FLAG_IN_HOLE = "\u26f3"
ICE_SKATE = "\u26f8\ufe0f"
FISHING_POLE = "\U0001f3a3"
DIVING_MASK = "\U0001f93f"
RUNNING_SHIRT = "\U0001f3bd"
SKIS = "\U0001f3bf"
SLED = "\U0001f6f7"
CURLING_STONE = "\U0001f94c"
BULLSEYE = "\U0001f3af"
YO_YO = "\U0001fa80"
KITE = "\U0001fa81"
POOL_8_BALL = "\U0001f3b1"
CRYSTAL_BALL = "\U0001f52e"
MAGIC_WAND = "\U0001fa84"
NAZAR_AMULET = "\U0001f9ff"
HAMSA = "\U0001faac"
VIDEO_GAME = "\U0001f3ae"
JOYSTICK = "\U0001f579\ufe0f"
SLOT_MACHINE = "\U0001f3b0"
GAME_DIE = "\U0001f3b2"
PUZZLE_PIECE = "\U0001f9e9"
TEDDY_BEAR = "\U0001f9f8"
PINATA = "\U0001fa85"
MIRROR_BALL = "\U0001faa9"
NESTING_DOLLS = "\U0001fa86"
SPADE_SUIT = "\u2660\ufe0f"
HEART_SUIT = "\u2665\ufe0f"
DIAMOND_SUIT = "\u2666\ufe0f"
CLUB_SUIT = "\u2663\ufe0f"
CHESS_PAWN = "\u265f\ufe0f"
JOKER = "\U0001f0cf"
MAHJONG_RED_DRAGON = "\U0001f004"
FLOWER_PLAYING_CARDS = "\U0001f3b4"
PERFORMING_ARTS = "\U0001f3ad"
FRAMED_PICTURE = "\U0001f5bc\ufe0f"
ARTIST_PALETTE = "\U0001f3a8"
THREAD = "\U0001f9f5"
SEWING_NEEDLE = "\U0001faa1"
YARN = "\U0001f9f6"
KNOT = "\U0001faa2"
GLASSES = "\U0001f453"
SUNGLASSES = "\U0001f576\ufe0f"
GOGGLES = "\U0001f97d"
LAB_COAT = "\U0001f97c"
SAFETY_VEST = "\U0001f9ba"
NECKTIE = "\U0001f454"
T_SHIRT = "\U0001f455"
JEANS = "\U0001f456"
SCARF = "\U0001f9e3"
GLOVES = "\U0001f9e4"
COAT = "\U0001f9e5"
SOCKS = "\U0001f9e6"
DRESS = "\U0001f457"
KIMONO = "\U0001f458"
SARI = "\U0001f97b"
ONE_PIECE_SWIMSUIT = "\U0001fa71"
BRIEFS = "\U0001fa72"
SHORTS = "\U0001fa73"
BIKINI = "\U0001f459"
WOMAN_S_CLOTHES = "\U0001f45a"
PURSE = "\U0001f45b"
HANDBAG = "\U0001f45c"
CLUTCH_BAG = "\U0001f45d"
SHOPPING_BAGS = "\U0001f6cd\ufe0f"
BACKPACK = "\U0001f392"
THONG_SANDAL = "\U0001fa74"
MAN_S_SHOE = "\U0001f45e"
RUNNING_SHOE = "\U0001f45f"
HIKING_BOOT = "\U0001f97e"
FLAT_SHOE = "\U0001f97f"
HIGH_HEELED_SHOE = "\U0001f460"
WOMAN_S_SANDAL = "\U0001f461"
BALLET_SHOES = "\U0001fa70"
WOMAN_S_BOOT = "\U0001f462"
CROWN = "\U0001f451"
WOMAN_S_HAT = "\U0001f452"
TOP_HAT = "\U0001f3a9"
GRADUATION_CAP = "\U0001f393"
BILLED_CAP = "\U0001f9e2"
MILITARY_HELMET = "\U0001fa96"
RESCUE_WORKER_S_HELMET = "\u26d1\ufe0f"
PRAYER_BEADS = "\U0001f4ff"
LIPSTICK = "\U0001f484"
RING = "\U0001f48d"
GEM_STONE = "\U0001f48e"
MUTED_SPEAKER = "\U0001f507"
SPEAKER_LOW_VOLUME = "\U0001f508"
SPEAKER_MEDIUM_VOLUME = "\U0001f509"
SPEAKER_HIGH_VOLUME = "\U0001f50a"
LOUDSPEAKER = "\U0001f4e2"
MEGAPHONE = "\U0001f4e3"
POSTAL_HORN = "\U0001f4ef"
BELL = "\U0001f514"
BELL_WITH_SLASH = "\U0001f515"
MUSICAL_SCORE = "\U0001f3bc"
MUSICAL_NOTE = "\U0001f3b5"
MUSICAL_NOTES = "\U0001f3b6"
STUDIO_MICROPHONE = "\U0001f399\ufe0f"
LEVEL_SLIDER = "\U0001f39a\ufe0f"
CONTROL_KNOBS = "\U0001f39b\ufe0f"
MICROPHONE = "\U0001f3a4"
HEADPHONE = "\U0001f3a7"
RADIO = "\U0001f4fb"
SAXOPHONE = "\U0001f3b7"
ACCORDION = "\U0001fa97"
GUITAR = "\U0001f3b8"
MUSICAL_KEYBOARD = "\U0001f3b9"
TRUMPET = "\U0001f3ba"
VIOLIN = "\U0001f3bb"
BANJO = "\U0001fa95"
DRUM = "\U0001f941"
LONG_DRUM = "\U0001fa98"
MOBILE_PHONE = "\U0001f4f1"
MOBILE_PHONE_WITH_ARROW = "\U0001f4f2"
TELEPHONE = "\u260e\ufe0f"
TELEPHONE_RECEIVER = "\U0001f4de"
PAGER = "\U0001f4df"
FAX_MACHINE = "\U0001f4e0"
BATTERY = "\U0001f50b"
LOW_BATTERY = "\U0001faab"
ELECTRIC_PLUG = "\U0001f50c"
LAPTOP = "\U0001f4bb"
DESKTOP_COMPUTER = "\U0001f5a5\ufe0f"
PRINTER = "\U0001f5a8\ufe0f"
KEYBOARD = "\u2328\ufe0f"
COMPUTER_MOUSE = "\U0001f5b1\ufe0f"
TRACKBALL = "\U0001f5b2\ufe0f"
COMPUTER_DISK = "\U0001f4bd"
FLOPPY_DISK = "\U0001f4be"
OPTICAL_DISK = "\U0001f4bf"
DVD = "\U0001f4c0"
ABACUS = "\U0001f9ee"
MOVIE_CAMERA = "\U0001f3a5"
FILM_FRAMES = "\U0001f39e\ufe0f"
FILM_PROJECTOR = "\U0001f4fd\ufe0f"
CLAPPER_BOARD = "\U0001f3ac"
TELEVISION = "\U0001f4fa"
CAMERA = "\U0001f4f7"
CAMERA_WITH_FLASH = "\U0001f4f8"
VIDEO_CAMERA = "\U0001f4f9"
VIDEOCASSETTE = "\U0001f4fc"
MAGNIFYING_GLASS_TILTED_LEFT = "\U0001f50d"
MAGNIFYING_GLASS_TILTED_RIGHT = "\U0001f50e"
CANDLE = "\U0001f56f\ufe0f"
LIGHT_BULB = "\U0001f4a1"
FLASHLIGHT = "\U0001f526"
RED_PAPER_LANTERN = "\U0001f3ee"
DIYA_LAMP = "\U0001fa94"
NOTEBOOK_WITH_DECORATIVE_COVER = "\U0001f4d4"
CLOSED_BOOK = "\U0001f4d5"
OPEN_BOOK = "\U0001f4d6"
GREEN_BOOK = "\U0001f4d7"
BLUE_BOOK = "\U0001f4d8"
ORANGE_BOOK = "\U0001f4d9"
BOOKS = "\U0001f4da"
NOTEBOOK = "\U0001f4d3"
LEDGER = "\U0001f4d2"
PAGE_WITH_CURL = "\U0001f4c3"
SCROLL = "\U0001f4dc"
PAGE_FACING_UP = "\U0001f4c4"
NEWSPAPER = "\U0001f4f0"
ROLLED_UP_NEWSPAPER = "\U0001f5de\ufe0f"
BOOKMARK_TABS = "\U0001f4d1"
BOOKMARK = "\U0001f516"
LABEL = "\U0001f3f7\ufe0f"
MONEY_BAG = "\U0001f4b0"
COIN = "\U0001fa99"
YEN_BANKNOTE = "\U0001f4b4"
DOLLAR_BANKNOTE = "\U0001f4b5"
EURO_BANKNOTE = "\U0001f4b6"
POUND_BANKNOTE = "\U0001f4b7"
MONEY_WITH_WINGS = "\U0001f4b8"
CREDIT_CARD = "\U0001f4b3"
RECEIPT = "\U0001f9fe"
CHART_INCREASING_WITH_YEN = "\U0001f4b9"
ENVELOPE = "\u2709\ufe0f"
E_MAIL = "\U0001f4e7"
INCOMING_ENVELOPE = "\U0001f4e8"
ENVELOPE_WITH_ARROW = "\U0001f4e9"
OUTBOX_TRAY = "\U0001f4e4"
INBOX_TRAY = "\U0001f4e5"
PACKAGE = "\U0001f4e6"
CLOSED_MAILBOX_WITH_RAISED_FLAG = "\U0001f4eb"
CLOSED_MAILBOX_WITH_LOWERED_FLAG = "\U0001f4ea"
OPEN_MAILBOX_WITH_RAISED_FLAG = "\U0001f4ec"
OPEN_MAILBOX_WITH_LOWERED_FLAG = "\U0001f4ed"
POSTBOX = "\U0001f4ee"
BALLOT_BOX_WITH_BALLOT = "\U0001f5f3\ufe0f"
PENCIL = "\u270f\ufe0f"
BLACK_NIB = "\u2712\ufe0f"
FOUNTAIN_PEN = "\U0001f58b\ufe0f"
PEN = "\U0001f58a\ufe0f"
PAINTBRUSH = "\U0001f58c\ufe0f"
CRAYON = "\U0001f58d\ufe0f"
MEMO = "\U0001f4dd"
BRIEFCASE = "\U0001f4bc"
FILE_FOLDER = "\U0001f4c1"
OPEN_FILE_FOLDER = "\U0001f4c2"
CARD_INDEX_DIVIDERS = "\U0001f5c2\ufe0f"
CALENDAR = "\U0001f4c5"
TEAR_OFF_CALENDAR = "\U0001f4c6"
SPIRAL_NOTEPAD = "\U0001f5d2\ufe0f"
SPIRAL_CALENDAR = "\U0001f5d3\ufe0f"
CARD_INDEX = "\U0001f4c7"
CHART_INCREASING = "\U0001f4c8"
CHART_DECREASING = "\U0001f4c9"
BAR_CHART = "\U0001f4ca"
CLIPBOARD = "\U0001f4cb"
PUSHPIN = "\U0001f4cc"
ROUND_PUSHPIN = "\U0001f4cd"
PAPERCLIP = "\U0001f4ce"
LINKED_PAPERCLIPS = "\U0001f587\ufe0f"
STRAIGHT_RULER = "\U0001f4cf"
TRIANGULAR_RULER = "\U0001f4d0"
SCISSORS = "\u2702\ufe0f"
CARD_FILE_BOX = "\U0001f5c3\ufe0f"
FILE_CABINET = "\U0001f5c4\ufe0f"
WASTEBASKET = "\U0001f5d1\ufe0f"
LOCKED = "\U0001f512"
UNLOCKED = "\U0001f513"
LOCKED_WITH_PEN = "\U0001f50f"
LOCKED_WITH_KEY = "\U0001f510"
KEY = "\U0001f511"
OLD_KEY = "\U0001f5dd\ufe0f"
HAMMER = "\U0001f528"
AXE = "\U0001fa93"
PICK = "\u26cf\ufe0f"
HAMMER_AND_PICK = "\u2692\ufe0f"
HAMMER_AND_WRENCH = "\U0001f6e0\ufe0f"
DAGGER = "\U0001f5e1\ufe0f"
CROSSED_SWORDS = "\u2694\ufe0f"
WATER_PISTOL = "\U0001f52b"
BOOMERANG = "\U0001fa83"
BOW_AND_ARROW = "\U0001f3f9"
SHIELD = "\U0001f6e1\ufe0f"
CARPENTRY_SAW = "\U0001fa9a"
WRENCH = "\U0001f527"
SCREWDRIVER = "\U0001fa9b"
NUT_AND_BOLT = "\U0001f529"
GEAR = "\u2699\ufe0f"
CLAMP = "\U0001f5dc\ufe0f"
BALANCE_SCALE = "\u2696\ufe0f"
WHITE_CANE = "\U0001f9af"
LINK = "\U0001f517"
CHAINS = "\u26d3\ufe0f"
HOOK = "\U0001fa9d"
TOOLBOX = "\U0001f9f0"
MAGNET = "\U0001f9f2"
LADDER = "\U0001fa9c"
ALEMBIC = "\u2697\ufe0f"
TEST_TUBE = "\U0001f9ea"
PETRI_DISH = "\U0001f9eb"
DNA = "\U0001f9ec"
MICROSCOPE = "\U0001f52c"
TELESCOPE = "\U0001f52d"
SATELLITE_ANTENNA = "\U0001f4e1"
SYRINGE = "\U0001f489"
DROP_OF_BLOOD = "\U0001fa78"
PILL = "\U0001f48a"
ADHESIVE_BANDAGE = "\U0001fa79"
CRUTCH = "\U0001fa7c"
STETHOSCOPE = "\U0001fa7a"
X_RAY = "\U0001fa7b"
DOOR = "\U0001f6aa"
ELEVATOR = "\U0001f6d7"
MIRROR = "\U0001fa9e"
WINDOW = "\U0001fa9f"
BED = "\U0001f6cf\ufe0f"
COUCH_AND_LAMP = "\U0001f6cb\ufe0f"
CHAIR = "\U0001fa91"
TOILET = "\U0001f6bd"
PLUNGER = "\U0001faa0"
SHOWER = "\U0001f6bf"
BATHTUB = "\U0001f6c1"
MOUSE_TRAP = "\U0001faa4"
RAZOR = "\U0001fa92"
LOTION_BOTTLE = "\U0001f9f4"
SAFETY_PIN = "\U0001f9f7"
BROOM = "\U0001f9f9"
BASKET = "\U0001f9fa"
ROLL_OF_PAPER = "\U0001f9fb"
BUCKET = "\U0001faa3"
SOAP = "\U0001f9fc"
BUBBLES = "\U0001fae7"
TOOTHBRUSH = "\U0001faa5"
SPONGE = "\U0001f9fd"
FIRE_EXTINGUISHER = "\U0001f9ef"
SHOPPING_CART = "\U0001f6d2"
CIGARETTE = "\U0001f6ac"
COFFIN = "\u26b0\ufe0f"
HEADSTONE = "\U0001faa6"
FUNERAL_URN = "\u26b1\ufe0f"
MOAI = "\U0001f5ff"
PLACARD = "\U0001faa7"
IDENTIFICATION_CARD = "\U0001faaa"
ATM_SIGN = "\U0001f3e7"
LITTER_IN_BIN_SIGN = "\U0001f6ae"
POTABLE_WATER = "\U0001f6b0"
WHEELCHAIR_SYMBOL = "\u267f"
MEN_S_ROOM = "\U0001f6b9"
WOMEN_S_ROOM = "\U0001f6ba"
RESTROOM = "\U0001f6bb"
BABY_SYMBOL = "\U0001f6bc"
WATER_CLOSET = "\U0001f6be"
PASSPORT_CONTROL = "\U0001f6c2"
CUSTOMS = "\U0001f6c3"
BAGGAGE_CLAIM = "\U0001f6c4"
LEFT_LUGGAGE = "\U0001f6c5"
WARNING = "\u26a0\ufe0f"
CHILDREN_CROSSING = "\U0001f6b8"
NO_ENTRY = "\u26d4"
PROHIBITED = "\U0001f6ab"
NO_BICYCLES = "\U0001f6b3"
NO_SMOKING = "\U0001f6ad"
NO_LITTERING = "\U0001f6af"
NON_POTABLE_WATER = "\U0001f6b1"
NO_PEDESTRIANS = "\U0001f6b7"
NO_MOBILE_PHONES = "\U0001f4f5"
NO_ONE_UNDER_EIGHTEEN = "\U0001f51e"
RADIOACTIVE = "\u2622\ufe0f"
BIOHAZARD = "\u2623\ufe0f"
UP_ARROW = "\u2b06\ufe0f"
UP_RIGHT_ARROW = "\u2197\ufe0f"
RIGHT_ARROW = "\u27a1\ufe0f"
DOWN_RIGHT_ARROW = "\u2198\ufe0f"
DOWN_ARROW = "\u2b07\ufe0f"
DOWN_LEFT_ARROW = "\u2199\ufe0f"
LEFT_ARROW = "\u2b05\ufe0f"
UP_LEFT_ARROW = "\u2196\ufe0f"
UP_DOWN_ARROW = "\u2195\ufe0f"
LEFT_RIGHT_ARROW = "\u2194\ufe0f"
RIGHT_ARROW_CURVING_LEFT = "\u21a9\ufe0f"
LEFT_ARROW_CURVING_RIGHT = "\u21aa\ufe0f"
RIGHT_ARROW_CURVING_UP = "\u2934\ufe0f"
RIGHT_ARROW_CURVING_DOWN = "\u2935\ufe0f"
CLOCKWISE_VERTICAL_ARROWS = "\U0001f503"
COUNTERCLOCKWISE_ARROWS_BUTTON = "\U0001f504"
BACK_ARROW = "\U0001f519"
END_ARROW = "\U0001f51a"
ON_ARROW = "\U0001f51b"
SOON_ARROW = "\U0001f51c"
TOP_ARROW = "\U0001f51d"
PLACE_OF_WORSHIP = "\U0001f6d0"
ATOM_SYMBOL = "\u269b\ufe0f"
OM = "\U0001f549\ufe0f"
STAR_OF_DAVID = "\u2721\ufe0f"
WHEEL_OF_DHARMA = "\u2638\ufe0f"
YIN_YANG = "\u262f\ufe0f"
LATIN_CROSS = "\u271d\ufe0f"
ORTHODOX_CROSS = "\u2626\ufe0f"
STAR_AND_CRESCENT = "\u262a\ufe0f"
PEACE_SYMBOL = "\u262e\ufe0f"
MENORAH = "\U0001f54e"
DOTTED_SIX_POINTED_STAR = "\U0001f52f"
ARIES = "\u2648"
TAURUS = "\u2649"
GEMINI = "\u264a"
CANCER = "\u264b"
LEO = "\u264c"
VIRGO = "\u264d"
LIBRA = "\u264e"
SCORPIO = "\u264f"
SAGITTARIUS = "\u2650"
CAPRICORN = "\u2651"
AQUARIUS = "\u2652"
PISCES = "\u2653"
OPHIUCHUS = "\u26ce"
SHUFFLE_TRACKS_BUTTON = "\U0001f500"
REPEAT_BUTTON = "\U0001f501"
REPEAT_SINGLE_BUTTON = "\U0001f502"
PLAY_BUTTON = "\u25b6\ufe0f"
FAST_FORWARD_BUTTON = "\u23e9"
NEXT_TRACK_BUTTON = "\u23ed\ufe0f"
PLAY_OR_PAUSE_BUTTON = "\u23ef\ufe0f"
REVERSE_BUTTON = "\u25c0\ufe0f"
FAST_REVERSE_BUTTON = "\u23ea"
LAST_TRACK_BUTTON = "\u23ee\ufe0f"
UPWARDS_BUTTON = "\U0001f53c"
FAST_UP_BUTTON = "\u23eb"
DOWNWARDS_BUTTON = "\U0001f53d"
FAST_DOWN_BUTTON = "\u23ec"
PAUSE_BUTTON = "\u23f8\ufe0f"
STOP_BUTTON = "\u23f9\ufe0f"
RECORD_BUTTON = "\u23fa\ufe0f"
EJECT_BUTTON = "\u23cf\ufe0f"
CINEMA = "\U0001f3a6"
DIM_BUTTON = "\U0001f505"
BRIGHT_BUTTON = "\U0001f506"
ANTENNA_BARS = "\U0001f4f6"
VIBRATION_MODE = "\U0001f4f3"
MOBILE_PHONE_OFF = "\U0001f4f4"
FEMALE_SIGN = "\u2640\ufe0f"
MALE_SIGN = "\u2642\ufe0f"
TRANSGENDER_SYMBOL = "\u26a7\ufe0f"
MULTIPLY = "\u2716\ufe0f"
PLUS = "\u2795"
MINUS = "\u2796"
DIVIDE = "\u2797"
HEAVY_EQUALS_SIGN = "\U0001f7f0"
INFINITY = "\u267e\ufe0f"
DOUBLE_EXCLAMATION_MARK = "\u203c\ufe0f"
EXCLAMATION_QUESTION_MARK = "\u2049\ufe0f"
RED_QUESTION_MARK = "\u2753"
WHITE_QUESTION_MARK = "\u2754"
WHITE_EXCLAMATION_MARK = "\u2755"
RED_EXCLAMATION_MARK = "\u2757"
WAVY_DASH = "\u3030\ufe0f"
CURRENCY_EXCHANGE = "\U0001f4b1"
HEAVY_DOLLAR_SIGN = "\U0001f4b2"
MEDICAL_SYMBOL = "\u2695\ufe0f"
RECYCLING_SYMBOL = "\u267b\ufe0f"
FLEUR_DE_LIS = "\u269c\ufe0f"
TRIDENT_EMBLEM = "\U0001f531"
NAME_BADGE = "\U0001f4db"
JAPANESE_SYMBOL_FOR_BEGINNER = "\U0001f530"
HOLLOW_RED_CIRCLE = "\u2b55"
CHECK_MARK_BUTTON = "\u2705"
CHECK_BOX_WITH_CHECK = "\u2611\ufe0f"
CHECK_MARK = "\u2714\ufe0f"
CROSS_MARK = "\u274c"
CROSS_MARK_BUTTON = "\u274e"
CURLY_LOOP = "\u27b0"
DOUBLE_CURLY_LOOP = "\u27bf"
PART_ALTERNATION_MARK = "\u303d\ufe0f"
EIGHT_SPOKED_ASTERISK = "\u2733\ufe0f"
EIGHT_POINTED_STAR = "\u2734\ufe0f"
SPARKLE = "\u2747\ufe0f"
COPYRIGHT = "\xa9\ufe0f"
REGISTERED = "\xae\ufe0f"
TRADE_MARK = "\u2122\ufe0f"
KEYCAP_NUMBER_SIGN = "#\ufe0f\u20e3"
KEYCAP_ASTERISK = "*\ufe0f\u20e3"
KEYCAP_DIGIT_ZERO = "0\ufe0f\u20e3"
KEYCAP_DIGIT_ONE = "1\ufe0f\u20e3"
KEYCAP_DIGIT_TWO = "2\ufe0f\u20e3"
KEYCAP_DIGIT_THREE = "3\ufe0f\u20e3"
KEYCAP_DIGIT_FOUR = "4\ufe0f\u20e3"
KEYCAP_DIGIT_FIVE = "5\ufe0f\u20e3"
KEYCAP_DIGIT_SIX = "6\ufe0f\u20e3"
KEYCAP_DIGIT_SEVEN = "7\ufe0f\u20e3"
KEYCAP_DIGIT_EIGHT = "8\ufe0f\u20e3"
KEYCAP_DIGIT_NINE = "9\ufe0f\u20e3"
KEYCAP_10 = "\U0001f51f"
INPUT_LATIN_UPPERCASE = "\U0001f520"
INPUT_LATIN_LOWERCASE = "\U0001f521"
INPUT_NUMBERS = "\U0001f522"
INPUT_SYMBOLS = "\U0001f523"
INPUT_LATIN_LETTERS = "\U0001f524"
A_BUTTON_BLOOD_TYPE = "\U0001f170\ufe0f"
AB_BUTTON_BLOOD_TYPE = "\U0001f18e"
B_BUTTON_BLOOD_TYPE = "\U0001f171\ufe0f"
CL_BUTTON = "\U0001f191"
COOL_BUTTON = "\U0001f192"
FREE_BUTTON = "\U0001f193"
INFORMATION = "\u2139\ufe0f"
ID_BUTTON = "\U0001f194"
CIRCLED_M = "\u24c2\ufe0f"
NEW_BUTTON = "\U0001f195"
NG_BUTTON = "\U0001f196"
O_BUTTON_BLOOD_TYPE = "\U0001f17e\ufe0f"
OK_BUTTON = "\U0001f197"
P_BUTTON = "\U0001f17f\ufe0f"
SOS_BUTTON = "\U0001f198"
UP_BUTTON = "\U0001f199"
VS_BUTTON = "\U0001f19a"
JAPANESE_HERE_BUTTON = "\U0001f201"
JAPANESE_SERVICE_CHARGE_BUTTON = "\U0001f202\ufe0f"
JAPANESE_MONTHLY_AMOUNT_BUTTON = "\U0001f237\ufe0f"
JAPANESE_NOT_FREE_OF_CHARGE_BUTTON = "\U0001f236"
JAPANESE_RESERVED_BUTTON = "\U0001f22f"
JAPANESE_BARGAIN_BUTTON = "\U0001f250"
JAPANESE_DISCOUNT_BUTTON = "\U0001f239"
JAPANESE_FREE_OF_CHARGE_BUTTON = "\U0001f21a"
JAPANESE_PROHIBITED_BUTTON = "\U0001f232"
JAPANESE_ACCEPTABLE_BUTTON = "\U0001f251"
JAPANESE_APPLICATION_BUTTON = "\U0001f238"
JAPANESE_PASSING_GRADE_BUTTON = "\U0001f234"
JAPANESE_VACANCY_BUTTON = "\U0001f233"
JAPANESE_CONGRATULATIONS_BUTTON = "\u3297\ufe0f"
JAPANESE_SECRET_BUTTON = "\u3299\ufe0f"
JAPANESE_OPEN_FOR_BUSINESS_BUTTON = "\U0001f23a"
JAPANESE_NO_VACANCY_BUTTON = "\U0001f235"
RED_CIRCLE = "\U0001f534"
ORANGE_CIRCLE = "\U0001f7e0"
YELLOW_CIRCLE = "\U0001f7e1"
GREEN_CIRCLE = "\U0001f7e2"
BLUE_CIRCLE = "\U0001f535"
PURPLE_CIRCLE = "\U0001f7e3"
BROWN_CIRCLE = "\U0001f7e4"
BLACK_CIRCLE = "\u26ab"
WHITE_CIRCLE = "\u26aa"
RED_SQUARE = "\U0001f7e5"
ORANGE_SQUARE = "\U0001f7e7"
YELLOW_SQUARE = "\U0001f7e8"
GREEN_SQUARE = "\U0001f7e9"
BLUE_SQUARE = "\U0001f7e6"
PURPLE_SQUARE = "\U0001f7ea"
BROWN_SQUARE = "\U0001f7eb"
BLACK_LARGE_SQUARE = "\u2b1b"
WHITE_LARGE_SQUARE = "\u2b1c"
BLACK_MEDIUM_SQUARE = "\u25fc\ufe0f"
WHITE_MEDIUM_SQUARE = "\u25fb\ufe0f"
BLACK_MEDIUM_SMALL_SQUARE = "\u25fe"
WHITE_MEDIUM_SMALL_SQUARE = "\u25fd"
BLACK_SMALL_SQUARE = "\u25aa\ufe0f"
WHITE_SMALL_SQUARE = "\u25ab\ufe0f"
LARGE_ORANGE_DIAMOND = "\U0001f536"
LARGE_BLUE_DIAMOND = "\U0001f537"
SMALL_ORANGE_DIAMOND = "\U0001f538"
SMALL_BLUE_DIAMOND = "\U0001f539"
RED_TRIANGLE_POINTED_UP = "\U0001f53a"
RED_TRIANGLE_POINTED_DOWN = "\U0001f53b"
DIAMOND_WITH_A_DOT = "\U0001f4a0"
RADIO_BUTTON = "\U0001f518"
WHITE_SQUARE_BUTTON = "\U0001f533"
BLACK_SQUARE_BUTTON = "\U0001f532"
CHEQUERED_FLAG = "\U0001f3c1"
TRIANGULAR_FLAG = "\U0001f6a9"
CROSSED_FLAGS = "\U0001f38c"
BLACK_FLAG = "\U0001f3f4"
WHITE_FLAG = "\U0001f3f3\ufe0f"
RAINBOW_FLAG = "\U0001f3f3\ufe0f\u200d\U0001f308"
TRANSGENDER_FLAG = "\U0001f3f3\ufe0f\u200d\u26a7\ufe0f"
PIRATE_FLAG = "\U0001f3f4\u200d\u2620\ufe0f"
FLAG_ASCENSION_ISLAND = "\U0001f1e6\U0001f1e8"
FLAG_ANDORRA = "\U0001f1e6\U0001f1e9"
FLAG_UNITED_ARAB_EMIRATES = "\U0001f1e6\U0001f1ea"
FLAG_AFGHANISTAN = "\U0001f1e6\U0001f1eb"
FLAG_ANTIGUA_ANDAMP_BARBUDA = "\U0001f1e6\U0001f1ec"
FLAG_ANGUILLA = "\U0001f1e6\U0001f1ee"
FLAG_ALBANIA = "\U0001f1e6\U0001f1f1"
FLAG_ARMENIA = "\U0001f1e6\U0001f1f2"
FLAG_ANGOLA = "\U0001f1e6\U0001f1f4"
FLAG_ANTARCTICA = "\U0001f1e6\U0001f1f6"
FLAG_ARGENTINA = "\U0001f1e6\U0001f1f7"
FLAG_AMERICAN_SAMOA = "\U0001f1e6\U0001f1f8"
FLAG_AUSTRIA = "\U0001f1e6\U0001f1f9"
FLAG_AUSTRALIA = "\U0001f1e6\U0001f1fa"
FLAG_ARUBA = "\U0001f1e6\U0001f1fc"
FLAG_ALAND_ISLANDS = "\U0001f1e6\U0001f1fd"
FLAG_AZERBAIJAN = "\U0001f1e6\U0001f1ff"
FLAG_BOSNIA_ANDAMP_HERZEGOVINA = "\U0001f1e7\U0001f1e6"
FLAG_BARBADOS = "\U0001f1e7\U0001f1e7"
FLAG_BANGLADESH = "\U0001f1e7\U0001f1e9"
FLAG_BELGIUM = "\U0001f1e7\U0001f1ea"
FLAG_BURKINA_FASO = "\U0001f1e7\U0001f1eb"
FLAG_BULGARIA = "\U0001f1e7\U0001f1ec"
FLAG_BAHRAIN = "\U0001f1e7\U0001f1ed"
FLAG_BURUNDI = "\U0001f1e7\U0001f1ee"
FLAG_BENIN = "\U0001f1e7\U0001f1ef"
FLAG_ST_BARTHELEMY = "\U0001f1e7\U0001f1f1"
FLAG_BERMUDA = "\U0001f1e7\U0001f1f2"
FLAG_BRUNEI = "\U0001f1e7\U0001f1f3"
FLAG_BOLIVIA = "\U0001f1e7\U0001f1f4"
FLAG_CARIBBEAN_NETHERLANDS = "\U0001f1e7\U0001f1f6"
FLAG_BRAZIL = "\U0001f1e7\U0001f1f7"
FLAG_BAHAMAS = "\U0001f1e7\U0001f1f8"
FLAG_BHUTAN = "\U0001f1e7\U0001f1f9"
FLAG_BOUVET_ISLAND = "\U0001f1e7\U0001f1fb"
FLAG_BOTSWANA = "\U0001f1e7\U0001f1fc"
FLAG_BELARUS = "\U0001f1e7\U0001f1fe"
FLAG_BELIZE = "\U0001f1e7\U0001f1ff"
FLAG_CANADA = "\U0001f1e8\U0001f1e6"
FLAG_COCOS_KEELING_ISLANDS = "\U0001f1e8\U0001f1e8"
FLAG_CONGO_KINSHASA = "\U0001f1e8\U0001f1e9"
FLAG_CENTRAL_AFRICAN_REPUBLIC = "\U0001f1e8\U0001f1eb"
FLAG_CONGO_BRAZZAVILLE = "\U0001f1e8\U0001f1ec"
FLAG_SWITZERLAND = "\U0001f1e8\U0001f1ed"
FLAG_COTE_D_IVOIRE = "\U0001f1e8\U0001f1ee"
FLAG_COOK_ISLANDS = "\U0001f1e8\U0001f1f0"
FLAG_CHILE = "\U0001f1e8\U0001f1f1"
FLAG_CAMEROON = "\U0001f1e8\U0001f1f2"
FLAG_CHINA = "\U0001f1e8\U0001f1f3"
FLAG_COLOMBIA = "\U0001f1e8\U0001f1f4"
FLAG_CLIPPERTON_ISLAND = "\U0001f1e8\U0001f1f5"
FLAG_COSTA_RICA = "\U0001f1e8\U0001f1f7"
FLAG_CUBA = "\U0001f1e8\U0001f1fa"
FLAG_CAPE_VERDE = "\U0001f1e8\U0001f1fb"
FLAG_CURACAO = "\U0001f1e8\U0001f1fc"
FLAG_CHRISTMAS_ISLAND = "\U0001f1e8\U0001f1fd"
FLAG_CYPRUS = "\U0001f1e8\U0001f1fe"
FLAG_CZECHIA = "\U0001f1e8\U0001f1ff"
FLAG_GERMANY = "\U0001f1e9\U0001f1ea"
FLAG_DIEGO_GARCIA = "\U0001f1e9\U0001f1ec"
FLAG_DJIBOUTI = "\U0001f1e9\U0001f1ef"
FLAG_DENMARK = "\U0001f1e9\U0001f1f0"
FLAG_DOMINICA = "\U0001f1e9\U0001f1f2"
FLAG_DOMINICAN_REPUBLIC = "\U0001f1e9\U0001f1f4"
FLAG_ALGERIA = "\U0001f1e9\U0001f1ff"
FLAG_CEUTA_ANDAMP_MELILLA = "\U0001f1ea\U0001f1e6"
FLAG_ECUADOR = "\U0001f1ea\U0001f1e8"
FLAG_ESTONIA = "\U0001f1ea\U0001f1ea"
FLAG_EGYPT = "\U0001f1ea\U0001f1ec"
FLAG_WESTERN_SAHARA = "\U0001f1ea\U0001f1ed"
FLAG_ERITREA = "\U0001f1ea\U0001f1f7"
FLAG_SPAIN = "\U0001f1ea\U0001f1f8"
FLAG_ETHIOPIA = "\U0001f1ea\U0001f1f9"
FLAG_EUROPEAN_UNION = "\U0001f1ea\U0001f1fa"
FLAG_FINLAND = "\U0001f1eb\U0001f1ee"
FLAG_FIJI = "\U0001f1eb\U0001f1ef"
FLAG_FALKLAND_ISLANDS = "\U0001f1eb\U0001f1f0"
FLAG_MICRONESIA = "\U0001f1eb\U0001f1f2"
FLAG_FAROE_ISLANDS = "\U0001f1eb\U0001f1f4"
FLAG_FRANCE = "\U0001f1eb\U0001f1f7"
FLAG_GABON = "\U0001f1ec\U0001f1e6"
FLAG_UNITED_KINGDOM = "\U0001f1ec\U0001f1e7"
FLAG_GRENADA = "\U0001f1ec\U0001f1e9"
FLAG_GEORGIA = "\U0001f1ec\U0001f1ea"
FLAG_FRENCH_GUIANA = "\U0001f1ec\U0001f1eb"
FLAG_GUERNSEY = "\U0001f1ec\U0001f1ec"
FLAG_GHANA = "\U0001f1ec\U0001f1ed"
FLAG_GIBRALTAR = "\U0001f1ec\U0001f1ee"
FLAG_GREENLAND = "\U0001f1ec\U0001f1f1"
FLAG_GAMBIA = "\U0001f1ec\U0001f1f2"
FLAG_GUINEA = "\U0001f1ec\U0001f1f3"
FLAG_GUADELOUPE = "\U0001f1ec\U0001f1f5"
FLAG_EQUATORIAL_GUINEA = "\U0001f1ec\U0001f1f6"
FLAG_GREECE = "\U0001f1ec\U0001f1f7"
FLAG_SOUTH_GEORGIA_ANDAMP_SOUTH_SANDWICH_ISLANDS = "\U0001f1ec\U0001f1f8"
FLAG_GUATEMALA = "\U0001f1ec\U0001f1f9"
FLAG_GUAM = "\U0001f1ec\U0001f1fa"
FLAG_GUINEA_BISSAU = "\U0001f1ec\U0001f1fc"
FLAG_GUYANA = "\U0001f1ec\U0001f1fe"
FLAG_HONG_KONG_SAR_CHINA = "\U0001f1ed\U0001f1f0"
FLAG_HEARD_ANDAMP_MCDONALD_ISLANDS = "\U0001f1ed\U0001f1f2"
FLAG_HONDURAS = "\U0001f1ed\U0001f1f3"
FLAG_CROATIA = "\U0001f1ed\U0001f1f7"
FLAG_HAITI = "\U0001f1ed\U0001f1f9"
FLAG_HUNGARY = "\U0001f1ed\U0001f1fa"
FLAG_CANARY_ISLANDS = "\U0001f1ee\U0001f1e8"
FLAG_INDONESIA = "\U0001f1ee\U0001f1e9"
FLAG_IRELAND = "\U0001f1ee\U0001f1ea"
FLAG_ISRAEL = "\U0001f1ee\U0001f1f1"
FLAG_ISLE_OF_MAN = "\U0001f1ee\U0001f1f2"
FLAG_INDIA = "\U0001f1ee\U0001f1f3"
FLAG_BRITISH_INDIAN_OCEAN_TERRITORY = "\U0001f1ee\U0001f1f4"
FLAG_IRAQ = "\U0001f1ee\U0001f1f6"
FLAG_IRAN = "\U0001f1ee\U0001f1f7"
FLAG_ICELAND = "\U0001f1ee\U0001f1f8"
FLAG_ITALY = "\U0001f1ee\U0001f1f9"
FLAG_JERSEY = "\U0001f1ef\U0001f1ea"
FLAG_JAMAICA = "\U0001f1ef\U0001f1f2"
FLAG_JORDAN = "\U0001f1ef\U0001f1f4"
FLAG_JAPAN = "\U0001f1ef\U0001f1f5"
FLAG_KENYA = "\U0001f1f0\U0001f1ea"
FLAG_KYRGYZSTAN = "\U0001f1f0\U0001f1ec"
FLAG_CAMBODIA = "\U0001f1f0\U0001f1ed"
FLAG_KIRIBATI = "\U0001f1f0\U0001f1ee"
FLAG_COMOROS = "\U0001f1f0\U0001f1f2"
FLAG_ST_KITTS_ANDAMP_NEVIS = "\U0001f1f0\U0001f1f3"
FLAG_NORTH_KOREA = "\U0001f1f0\U0001f1f5"
FLAG_SOUTH_KOREA = "\U0001f1f0\U0001f1f7"
FLAG_KUWAIT = "\U0001f1f0\U0001f1fc"
FLAG_CAYMAN_ISLANDS = "\U0001f1f0\U0001f1fe"
FLAG_KAZAKHSTAN = "\U0001f1f0\U0001f1ff"
FLAG_LAOS = "\U0001f1f1\U0001f1e6"
FLAG_LEBANON = "\U0001f1f1\U0001f1e7"
FLAG_ST_LUCIA = "\U0001f1f1\U0001f1e8"
FLAG_LIECHTENSTEIN = "\U0001f1f1\U0001f1ee"
FLAG_SRI_LANKA = "\U0001f1f1\U0001f1f0"
FLAG_LIBERIA = "\U0001f1f1\U0001f1f7"
FLAG_LESOTHO = "\U0001f1f1\U0001f1f8"
FLAG_LITHUANIA = "\U0001f1f1\U0001f1f9"
FLAG_LUXEMBOURG = "\U0001f1f1\U0001f1fa"
FLAG_LATVIA = "\U0001f1f1\U0001f1fb"
FLAG_LIBYA = "\U0001f1f1\U0001f1fe"
FLAG_MOROCCO = "\U0001f1f2\U0001f1e6"
FLAG_MONACO = "\U0001f1f2\U0001f1e8"
FLAG_MOLDOVA = "\U0001f1f2\U0001f1e9"
FLAG_MONTENEGRO = "\U0001f1f2\U0001f1ea"
FLAG_ST_MARTIN = "\U0001f1f2\U0001f1eb"
FLAG_MADAGASCAR = "\U0001f1f2\U0001f1ec"
FLAG_MARSHALL_ISLANDS = "\U0001f1f2\U0001f1ed"
FLAG_NORTH_MACEDONIA = "\U0001f1f2\U0001f1f0"
FLAG_MALI = "\U0001f1f2\U0001f1f1"
FLAG_MYANMAR_BURMA = "\U0001f1f2\U0001f1f2"
FLAG_MONGOLIA = "\U0001f1f2\U0001f1f3"
FLAG_MACAO_SAR_CHINA = "\U0001f1f2\U0001f1f4"
FLAG_NORTHERN_MARIANA_ISLANDS = "\U0001f1f2\U0001f1f5"
FLAG_MARTINIQUE = "\U0001f1f2\U0001f1f6"
FLAG_MAURITANIA = "\U0001f1f2\U0001f1f7"
FLAG_MONTSERRAT = "\U0001f1f2\U0001f1f8"
FLAG_MALTA = "\U0001f1f2\U0001f1f9"
FLAG_MAURITIUS = "\U0001f1f2\U0001f1fa"
FLAG_MALDIVES = "\U0001f1f2\U0001f1fb"
FLAG_MALAWI = "\U0001f1f2\U0001f1fc"
FLAG_MEXICO = "\U0001f1f2\U0001f1fd"
FLAG_MALAYSIA = "\U0001f1f2\U0001f1fe"
FLAG_MOZAMBIQUE = "\U0001f1f2\U0001f1ff"
FLAG_NAMIBIA = "\U0001f1f3\U0001f1e6"
FLAG_NEW_CALEDONIA = "\U0001f1f3\U0001f1e8"
FLAG_NIGER = "\U0001f1f3\U0001f1ea"
FLAG_NORFOLK_ISLAND = "\U0001f1f3\U0001f1eb"
FLAG_NIGERIA = "\U0001f1f3\U0001f1ec"
FLAG_NICARAGUA = "\U0001f1f3\U0001f1ee"
FLAG_NETHERLANDS = "\U0001f1f3\U0001f1f1"
FLAG_NORWAY = "\U0001f1f3\U0001f1f4"
FLAG_NEPAL = "\U0001f1f3\U0001f1f5"
FLAG_NAURU = "\U0001f1f3\U0001f1f7"
FLAG_NIUE = "\U0001f1f3\U0001f1fa"
FLAG_NEW_ZEALAND = "\U0001f1f3\U0001f1ff"
FLAG_OMAN = "\U0001f1f4\U0001f1f2"
FLAG_PANAMA = "\U0001f1f5\U0001f1e6"
FLAG_PERU = "\U0001f1f5\U0001f1ea"
FLAG_FRENCH_POLYNESIA = "\U0001f1f5\U0001f1eb"
FLAG_PAPUA_NEW_GUINEA = "\U0001f1f5\U0001f1ec"
FLAG_PHILIPPINES = "\U0001f1f5\U0001f1ed"
FLAG_PAKISTAN = "\U0001f1f5\U0001f1f0"
FLAG_POLAND = "\U0001f1f5\U0001f1f1"
FLAG_ST_PIERRE_ANDAMP_MIQUELON = "\U0001f1f5\U0001f1f2"
FLAG_PITCAIRN_ISLANDS = "\U0001f1f5\U0001f1f3"
FLAG_PUERTO_RICO = "\U0001f1f5\U0001f1f7"
FLAG_PALESTINIAN_TERRITORIES = "\U0001f1f5\U0001f1f8"
FLAG_PORTUGAL = "\U0001f1f5\U0001f1f9"
FLAG_PALAU = "\U0001f1f5\U0001f1fc"
FLAG_PARAGUAY = "\U0001f1f5\U0001f1fe"
FLAG_QATAR = "\U0001f1f6\U0001f1e6"
FLAG_REUNION = "\U0001f1f7\U0001f1ea"
FLAG_ROMANIA = "\U0001f1f7\U0001f1f4"
FLAG_SERBIA = "\U0001f1f7\U0001f1f8"
FLAG_RUSSIA = "\U0001f1f7\U0001f1fa"
FLAG_RWANDA = "\U0001f1f7\U0001f1fc"
FLAG_SAUDI_ARABIA = "\U0001f1f8\U0001f1e6"
FLAG_SOLOMON_ISLANDS = "\U0001f1f8\U0001f1e7"
FLAG_SEYCHELLES = "\U0001f1f8\U0001f1e8"
FLAG_SUDAN = "\U0001f1f8\U0001f1e9"
FLAG_SWEDEN = "\U0001f1f8\U0001f1ea"
FLAG_SINGAPORE = "\U0001f1f8\U0001f1ec"
FLAG_ST_HELENA = "\U0001f1f8\U0001f1ed"
FLAG_SLOVENIA = "\U0001f1f8\U0001f1ee"
FLAG_SVALBARD_ANDAMP_JAN_MAYEN = "\U0001f1f8\U0001f1ef"
FLAG_SLOVAKIA = "\U0001f1f8\U0001f1f0"
FLAG_SIERRA_LEONE = "\U0001f1f8\U0001f1f1"
FLAG_SAN_MARINO = "\U0001f1f8\U0001f1f2"
FLAG_SENEGAL = "\U0001f1f8\U0001f1f3"
FLAG_SOMALIA = "\U0001f1f8\U0001f1f4"
FLAG_SURINAME = "\U0001f1f8\U0001f1f7"
FLAG_SOUTH_SUDAN = "\U0001f1f8\U0001f1f8"
FLAG_SAO_TOME_ANDAMP_PRINCIPE = "\U0001f1f8\U0001f1f9"
FLAG_EL_SALVADOR = "\U0001f1f8\U0001f1fb"
FLAG_SINT_MAARTEN = "\U0001f1f8\U0001f1fd"
FLAG_SYRIA = "\U0001f1f8\U0001f1fe"
FLAG_ESWATINI = "\U0001f1f8\U0001f1ff"
FLAG_TRISTAN_DA_CUNHA = "\U0001f1f9\U0001f1e6"
FLAG_TURKS_ANDAMP_CAICOS_ISLANDS = "\U0001f1f9\U0001f1e8"
FLAG_CHAD = "\U0001f1f9\U0001f1e9"
FLAG_FRENCH_SOUTHERN_TERRITORIES = "\U0001f1f9\U0001f1eb"
FLAG_TOGO = "\U0001f1f9\U0001f1ec"
FLAG_THAILAND = "\U0001f1f9\U0001f1ed"
FLAG_TAJIKISTAN = "\U0001f1f9\U0001f1ef"
FLAG_TOKELAU = "\U0001f1f9\U0001f1f0"
FLAG_TIMOR_LESTE = "\U0001f1f9\U0001f1f1"
FLAG_TURKMENISTAN = "\U0001f1f9\U0001f1f2"
FLAG_TUNISIA = "\U0001f1f9\U0001f1f3"
FLAG_TONGA = "\U0001f1f9\U0001f1f4"
FLAG_TURKEY = "\U0001f1f9\U0001f1f7"
FLAG_TRINIDAD_ANDAMP_TOBAGO = "\U0001f1f9\U0001f1f9"
FLAG_TUVALU = "\U0001f1f9\U0001f1fb"
FLAG_TAIWAN = "\U0001f1f9\U0001f1fc"
FLAG_TANZANIA = "\U0001f1f9\U0001f1ff"
FLAG_UKRAINE = "\U0001f1fa\U0001f1e6"
FLAG_UGANDA = "\U0001f1fa\U0001f1ec"
FLAG_U_S_OUTLYING_ISLANDS = "\U0001f1fa\U0001f1f2"
FLAG_UNITED_NATIONS = "\U0001f1fa\U0001f1f3"
FLAG_UNITED_STATES = "\U0001f1fa\U0001f1f8"
FLAG_URUGUAY = "\U0001f1fa\U0001f1fe"
FLAG_UZBEKISTAN = "\U0001f1fa\U0001f1ff"
FLAG_VATICAN_CITY = "\U0001f1fb\U0001f1e6"
FLAG_ST_VINCENT_ANDAMP_GRENADINES = "\U0001f1fb\U0001f1e8"
FLAG_VENEZUELA = "\U0001f1fb\U0001f1ea"
FLAG_BRITISH_VIRGIN_ISLANDS = "\U0001f1fb\U0001f1ec"
FLAG_U_S_VIRGIN_ISLANDS = "\U0001f1fb\U0001f1ee"
FLAG_VIETNAM = "\U0001f1fb\U0001f1f3"
FLAG_VANUATU = "\U0001f1fb\U0001f1fa"
FLAG_WALLIS_ANDAMP_FUTUNA = "\U0001f1fc\U0001f1eb"
FLAG_SAMOA = "\U0001f1fc\U0001f1f8"
FLAG_KOSOVO = "\U0001f1fd\U0001f1f0"
FLAG_YEMEN = "\U0001f1fe\U0001f1ea"
FLAG_MAYOTTE = "\U0001f1fe\U0001f1f9"
FLAG_SOUTH_AFRICA = "\U0001f1ff\U0001f1e6"
FLAG_ZAMBIA = "\U0001f1ff\U0001f1f2"
FLAG_ZIMBABWE = "\U0001f1ff\U0001f1fc"
FLAG_ENGLAND = "\U0001f3f4\U000e0067\U000e0062\U000e0065\U000e006e\U000e0067\U000e007f"
FLAG_SCOTLAND = "\U0001f3f4\U000e0067\U000e0062\U000e0073\U000e0063\U000e0074\U000e007f"
FLAG_WALES = "\U0001f3f4\U000e0067\U000e0062\U000e0077\U000e006c\U000e0073\U000e007f"
REGIONAL_INDICATOR_SYMBOL_LETTER_A = "\U0001f1e6"
REGIONAL_INDICATOR_SYMBOL_LETTER_B = "\U0001f1e7"
REGIONAL_INDICATOR_SYMBOL_LETTER_C = "\U0001f1e8"
REGIONAL_INDICATOR_SYMBOL_LETTER_D = "\U0001f1e9"
REGIONAL_INDICATOR_SYMBOL_LETTER_E = "\U0001f1ea"
REGIONAL_INDICATOR_SYMBOL_LETTER_F = "\U0001f1eb"
REGIONAL_INDICATOR_SYMBOL_LETTER_G = "\U0001f1ec"
REGIONAL_INDICATOR_SYMBOL_LETTER_H = "\U0001f1ed"
REGIONAL_INDICATOR_SYMBOL_LETTER_I = "\U0001f1ee"
REGIONAL_INDICATOR_SYMBOL_LETTER_J = "\U0001f1ef"
REGIONAL_INDICATOR_SYMBOL_LETTER_K = "\U0001f1f0"
REGIONAL_INDICATOR_SYMBOL_LETTER_L = "\U0001f1f1"
REGIONAL_INDICATOR_SYMBOL_LETTER_M = "\U0001f1f2"
REGIONAL_INDICATOR_SYMBOL_LETTER_N = "\U0001f1f3"
REGIONAL_INDICATOR_SYMBOL_LETTER_O = "\U0001f1f4"
REGIONAL_INDICATOR_SYMBOL_LETTER_P = "\U0001f1f5"
REGIONAL_INDICATOR_SYMBOL_LETTER_Q = "\U0001f1f6"
REGIONAL_INDICATOR_SYMBOL_LETTER_R = "\U0001f1f7"
REGIONAL_INDICATOR_SYMBOL_LETTER_S = "\U0001f1f8"
REGIONAL_INDICATOR_SYMBOL_LETTER_T = "\U0001f1f9"
REGIONAL_INDICATOR_SYMBOL_LETTER_U = "\U0001f1fa"
REGIONAL_INDICATOR_SYMBOL_LETTER_V = "\U0001f1fb"
REGIONAL_INDICATOR_SYMBOL_LETTER_W = "\U0001f1fc"
REGIONAL_INDICATOR_SYMBOL_LETTER_X = "\U0001f1fd"
REGIONAL_INDICATOR_SYMBOL_LETTER_Y = "\U0001f1fe"
REGIONAL_INDICATOR_SYMBOL_LETTER_Z = "\U0001f1ff"
TAG_RIGHT_CURLY_BRACKET = "\U000e007d"
DIGIT_FIVE = "5\ufe0f"
TAG_LATIN_CAPITAL_LETTER_U = "\U000e0055"
TAG_LATIN_CAPITAL_LETTER_Q = "\U000e0051"
TAG_LATIN_CAPITAL_LETTER_K = "\U000e004b"
COMBINING_ENCLOSING_KEYCAP = "\u20e3"
TAG_LATIN_CAPITAL_LETTER_C = "\U000e0043"
TAG_ASTERISK = "\U000e002a"
TAG_FULL_STOP = "\U000e002e"
TAG_CIRCUMFLEX_ACCENT = "\U000e005e"
DIGIT_ONE = "1\ufe0f"
TAG_COMMA = "\U000e002c"
DIGIT_ZERO = "0\ufe0f"
TAG_EQUALS_SIGN = "\U000e003d"
TAG_LATIN_CAPITAL_LETTER_O = "\U000e004f"
TAG_COMMERCIAL_AT = "\U000e0040"
DIGIT_EIGHT = "8\ufe0f"
TAG_NUMBER_SIGN = "\U000e0023"
TAG_LATIN_CAPITAL_LETTER_T = "\U000e0054"
TAG_LATIN_CAPITAL_LETTER_N = "\U000e004e"
DIGIT_SIX = "6\ufe0f"
TAG_PERCENT_SIGN = "\U000e0025"
VARIATION_SELECTOR_16 = "\ufe0f"
TAG_LATIN_CAPITAL_LETTER_W = "\U000e0057"
TAG_DOLLAR_SIGN = "\U000e0024"
TAG_LOW_LINE = "\U000e005f"
TAG_DIGIT_EIGHT = "\U000e0038"
TAG_LATIN_CAPITAL_LETTER_M = "\U000e004d"
TAG_LATIN_CAPITAL_LETTER_A = "\U000e0041"
TAG_REVERSE_SOLIDUS = "\U000e005c"
TAG_SOLIDUS = "\U000e002f"
TAG_LATIN_CAPITAL_LETTER_H = "\U000e0048"
TAG_DIGIT_NINE = "\U000e0039"
TAG_LEFT_CURLY_BRACKET = "\U000e007b"
TAG_LATIN_CAPITAL_LETTER_E = "\U000e0045"
TAG_LATIN_SMALL_LETTER_W = "\U000e0077"
TAG_DIGIT_ZERO = "\U000e0030"
TAG_LATIN_CAPITAL_LETTER_B = "\U000e0042"
TAG_LATIN_CAPITAL_LETTER_F = "\U000e0046"
TAG_LATIN_CAPITAL_LETTER_Y = "\U000e0059"
TAG_TILDE = "\U000e007e"
TAG_LATIN_SMALL_LETTER_P = "\U000e0070"
TAG_LATIN_CAPITAL_LETTER_Z = "\U000e005a"
TAG_GREATER_THAN_SIGN = "\U000e003e"
TAG_LATIN_SMALL_LETTER_S = "\U000e0073"
TAG_LATIN_SMALL_LETTER_G = "\U000e0067"
TAG_APOSTROPHE = "\U000e0027"
TAG_RIGHT_PARENTHESIS = "\U000e0029"
TAG_DIGIT_THREE = "\U000e0033"
TAG_LEFT_PARENTHESIS = "\U000e0028"
TAG_DIGIT_SEVEN = "\U000e0037"
TAG_LATIN_SMALL_LETTER_O = "\U000e006f"
TAG_DIGIT_SIX = "\U000e0036"
TAG_DIGIT_TWO = "\U000e0032"
TAG_LATIN_SMALL_LETTER_F = "\U000e0066"
TAG_LATIN_SMALL_LETTER_K = "\U000e006b"
TAG_LATIN_SMALL_LETTER_Y = "\U000e0079"
TAG_SPACE = "\U000e0020"
TAG_LATIN_SMALL_LETTER_I = "\U000e0069"
DIGIT_TWO = "2\ufe0f"
TAG_DIGIT_ONE = "\U000e0031"
TAG_RIGHT_SQUARE_BRACKET = "\U000e005d"
TAG_LATIN_SMALL_LETTER_R = "\U000e0072"
HASH_SIGN = "#\ufe0f"
TAG_SEMICOLON = "\U000e003b"
TAG_LATIN_CAPITAL_LETTER_L = "\U000e004c"
TAG_HYPHEN_MINUS = "\U000e002d"
ASTERISK = "*\ufe0f"
TAG_LATIN_SMALL_LETTER_A = "\U000e0061"
TAG_EXCLAMATION_MARK = "\U000e0021"
TAG_LATIN_CAPITAL_LETTER_V = "\U000e0056"
TAG_LATIN_SMALL_LETTER_C = "\U000e0063"
TAG_GRAVE_ACCENT = "\U000e0060"
ZERO_WIDTH_JOINER = "\u200d"
TAG_LATIN_CAPITAL_LETTER_G = "\U000e0047"
DIGIT_NINE = "9\ufe0f"
TAG_VERTICAL_LINE = "\U000e007c"
TAG_LATIN_SMALL_LETTER_Z = "\U000e007a"
TAG_LATIN_CAPITAL_LETTER_X = "\U000e0058"
TAG_LATIN_SMALL_LETTER_J = "\U000e006a"
TAG_LATIN_CAPITAL_LETTER_P = "\U000e0050"
TAG_AMPERSAND = "\U000e0026"
TAG_LATIN_SMALL_LETTER_L = "\U000e006c"
TAG_LATIN_SMALL_LETTER_X = "\U000e0078"
DIGIT_SEVEN = "7\ufe0f"
TAG_LATIN_CAPITAL_LETTER_J = "\U000e004a"
TAG_LATIN_SMALL_LETTER_T = "\U000e0074"
TAG_QUESTION_MARK = "\U000e003f"
TAG_LATIN_SMALL_LETTER_B = "\U000e0062"
TAG_LEFT_SQUARE_BRACKET = "\U000e005b"
TAG_LATIN_SMALL_LETTER_D = "\U000e0064"
TAG_LATIN_SMALL_LETTER_E = "\U000e0065"
TAG_LATIN_SMALL_LETTER_M = "\U000e006d"
TAG_LESS_THAN_SIGN = "\U000e003c"
TAG_DIGIT_FIVE = "\U000e0035"
TAG_LATIN_CAPITAL_LETTER_D = "\U000e0044"
TAG_LATIN_SMALL_LETTER_N = "\U000e006e"
TAG_PLUS_SIGN = "\U000e002b"
TAG_COLON = "\U000e003a"
DIGIT_THREE = "3\ufe0f"
TAG_LATIN_SMALL_LETTER_Q = "\U000e0071"
TAG_LATIN_CAPITAL_LETTER_R = "\U000e0052"
TAG_LATIN_CAPITAL_LETTER_S = "\U000e0053"
DIGIT_FOUR = "4\ufe0f"
TAG_LATIN_CAPITAL_LETTER_I = "\U000e0049"
TAG_QUOTATION_MARK = "\U000e0022"
CANCEL_TAG = "\U000e007f"
TAG_LATIN_SMALL_LETTER_V = "\U000e0076"
TAG_LATIN_SMALL_LETTER_H = "\U000e0068"
TAG_LATIN_SMALL_LETTER_U = "\U000e0075"
TAG_DIGIT_FOUR = "\U000e0034"
